/*
   Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "annotationwidget.h"
#include "resourceannotationmodel.h"
#include "annotation.h"
#include "annotationrelevancefiltermodel.h"
#include <kpixmapsequenceoverlaypainter.h>
#include <kpixmapsequence.h>

#include <QtGui/QTreeView>
#include <QtGui/QVBoxLayout>
#include <QtGui/QSortFilterProxyModel>
#include <QtGui/QKeyEvent>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>
#include <QtGui/QScrollBar>
#include <QtCore/QTimer>

#include <klineedit.h>
#include <kdebug.h>
#include <klocale.h>
#include <kicon.h>

Q_DECLARE_METATYPE( Nepomuk::Annotation* )


class Nepomuk::AnnotationWidget::Private
{
public:
    QTreeView* m_annotationsView;
    KLineEdit* m_filterEdit;

    Nepomuk::ResourceAnnotationModel* m_model;
    Nepomuk::AnnotationRelevanceFilterModel* m_sortModel;

    bool m_infoMessageShown;
    bool m_eatFocusOut;
    bool m_stickyAnnotations;

    // true if the user navigated the list already and we do not
    // want to confuse them with resetting the selection
    bool navigationPerformed;

    KPixmapSequenceOverlayPainter* m_busyPainter;

    void _k_slotAnnotationFinished( Nepomuk::Annotation* anno );
    void _k_slotExecuteAnnotation( Nepomuk::Annotation* anno );
    void _k_slotFilterChanged( const QString& filter );
    void _k_slotAnnotationsChanged();

    void executeAnnotationAtIndex( const QModelIndex& index );
    void showInfoMessage( bool show );
    void showAnnotations();

    AnnotationWidget* q;
};



void Nepomuk::AnnotationWidget::Private::_k_slotAnnotationFinished( Nepomuk::Annotation* anno )
{
    kDebug() << "Annotation" << anno->label() << "done";
    emit q->annotationAdded( m_model->resource() );
}


void Nepomuk::AnnotationWidget::Private::_k_slotFilterChanged( const QString& filter )
{
    kDebug() << filter;
    navigationPerformed = false;

    if ( !filter.isEmpty() )
        m_busyPainter->start();
    else
        m_busyPainter->stop();

    m_model->setFilter( filter );

    if ( !filter.isEmpty() || m_stickyAnnotations )
        showAnnotations();
    else
        m_annotationsView->hide();
}


void Nepomuk::AnnotationWidget::Private::_k_slotAnnotationsChanged()
{
    // if the model changes, ie. new annotations are added asyncroneously,
    // update the size of the popup
    if ( m_annotationsView->isVisible() )
        showAnnotations();
}


void Nepomuk::AnnotationWidget::Private::_k_slotExecuteAnnotation( Nepomuk::Annotation* anno )
{
    kDebug() << "Creating annotation" << anno;
    q->createAnnotation( anno );
}


void Nepomuk::AnnotationWidget::Private::executeAnnotationAtIndex( const QModelIndex& index )
{
    // take the annotation from the list
    Nepomuk::Annotation* anno = m_model->takeAnnotation( m_sortModel->mapToSource( index ) );
    connect( anno, SIGNAL( finished( Nepomuk::Annotation* ) ),
             q, SLOT( _k_slotAnnotationFinished( Nepomuk::Annotation* ) ) );

    // create the annotation async
    kDebug() << "Creating annotation" << anno;
    QMetaObject::invokeMethod( q, "_k_slotExecuteAnnotation", Qt::QueuedConnection, Q_ARG( Nepomuk::Annotation*, anno ) );

    // select the next in the list
    int rc = m_sortModel->rowCount();
    if ( rc ) {
        m_annotationsView->selectionModel()->select( m_sortModel->index( index.row() < rc ? index.row() : rc-1, 0 ),
                                                     QItemSelectionModel::Clear|QItemSelectionModel::Select );
    }
}


void Nepomuk::AnnotationWidget::Private::showAnnotations()
{
    m_eatFocusOut = true;

    const QRect screen = QApplication::desktop()->availableGeometry( m_filterEdit );
    int h = (m_annotationsView->sizeHintForRow(0) * qMin(7, m_annotationsView->model()->rowCount()) + 3) + 3;
    QScrollBar* hsb = m_annotationsView->horizontalScrollBar();
    if (hsb && hsb->isVisible())
        h += m_annotationsView->horizontalScrollBar()->sizeHint().height();

    int rh = m_filterEdit->height();
    int rw = m_filterEdit->width();
    QPoint pos = m_filterEdit->mapToGlobal(QPoint(0, m_filterEdit->height() - 2));
    int w = qMax( m_filterEdit->width(), static_cast<QAbstractItemView*>( m_annotationsView )->sizeHintForColumn( 0 ) + m_annotationsView->frameWidth()*2 );

    if ((pos.x() + rw) > (screen.x() + screen.width()))
        pos.setX(screen.x() + screen.width() - w);
    if (pos.x() < screen.x())
        pos.setX(screen.x());
    if (((pos.y() + rh) > (screen.y() + screen.height())) && ((pos.y() - h - rh) >= 0))
        pos.setY(pos.y() - qMax(h, m_annotationsView->minimumHeight()) - rh + 2);

    m_annotationsView->setGeometry(pos.x(), pos.y(), w, h);

    if ( !navigationPerformed )
        m_annotationsView->selectionModel()->select( m_sortModel->index( 0, 0 ),
                                                     QItemSelectionModel::Clear|QItemSelectionModel::SelectCurrent );

    if (!m_annotationsView->isVisible()) {
        m_annotationsView->show();
    }

    m_eatFocusOut = false;
}


void Nepomuk::AnnotationWidget::Private::showInfoMessage( bool show )
{
    kDebug() << show;
    if ( show != m_infoMessageShown ) {
        kDebug() << "changing state";
        m_infoMessageShown = show;
        if ( show ) {
            m_filterEdit->setText( i18nc( "The user can enter this line edit to annotate a resource. "
                                          "This means to add meta data such as tags or relations to other resources.",
                                          "Annotate..." ) );
            QPalette p = q->palette();
            p.setColor( QPalette::Text, p.color( QPalette::Disabled, QPalette::Text ) );
            m_filterEdit->setPalette( p );
            QFont f( q->font() );
            f.setItalic( true );
            m_filterEdit->setFont( f );
        }
        else {
            m_filterEdit->setText( QString() );
            m_filterEdit->setPalette( q->palette() );
            m_filterEdit->setFont( q->font() );
        }
    }
}


Nepomuk::AnnotationWidget::AnnotationWidget( QWidget* parent )
    : QWidget( parent ),
      d( new Private() )
{
    d->q = this;
    d->m_infoMessageShown = false;
    d->m_stickyAnnotations = false;
    d->m_eatFocusOut = false;

    d->navigationPerformed = false;

    d->m_filterEdit = new KLineEdit( this );
    QVBoxLayout* lay = new QVBoxLayout( this );
    lay->setMargin( 0 );
    lay->addWidget( d->m_filterEdit );

    d->m_busyPainter = new KPixmapSequenceOverlayPainter( this );
    d->m_busyPainter->setWidget( d->m_filterEdit );
    d->m_busyPainter->setAlignment( Qt::AlignRight|Qt::AlignVCenter );

    d->m_model = new Nepomuk::ResourceAnnotationModel( d->m_annotationsView );
    d->m_sortModel = new AnnotationRelevanceFilterModel( d->m_model );
    d->m_sortModel->setSourceModel( d->m_model );
    d->m_sortModel->setSortRole( Nepomuk::AnnotationModel::RelevanceRole );

    d->m_annotationsView = new QTreeView;
    d->m_annotationsView->setModel( d->m_sortModel );
    d->m_annotationsView->setSortingEnabled( true );
    d->m_annotationsView->setHeaderHidden( true );
    d->m_annotationsView->setRootIsDecorated( false );
    d->m_annotationsView->hide();
    d->m_annotationsView->setParent(0, Qt::Popup);
    d->m_annotationsView->setFocusPolicy(Qt::NoFocus);
    d->m_annotationsView->setSelectionMode( QAbstractItemView::SingleSelection );

    d->m_filterEdit->installEventFilter( this );
    d->m_annotationsView->installEventFilter( this );
    d->m_annotationsView->viewport()->installEventFilter( this );

    connect( d->m_filterEdit, SIGNAL( textEdited( QString ) ),
             this, SLOT( _k_slotFilterChanged( QString ) ) );
    connect( d->m_model, SIGNAL( rowsInserted( const QModelIndex&, int, int ) ),
             this, SLOT( _k_slotAnnotationsChanged() ) );

    connect( d->m_model, SIGNAL( finished() ),
             d->m_busyPainter, SLOT( stop() ) );

    d->showInfoMessage( true );

    qRegisterMetaType<Nepomuk::Annotation*>();
}


Nepomuk::AnnotationWidget::~AnnotationWidget()
{
    delete d;
}


bool Nepomuk::AnnotationWidget::eventFilter( QObject* watched, QEvent* event )
{
    switch( event->type() ) {
    case QEvent::Hide:
        if ( watched == d->m_annotationsView ) {
            d->navigationPerformed = false;
        }
        break;

    case QEvent::KeyPress: {
        if ( watched == d->m_annotationsView ) {
            QKeyEvent* ke = static_cast<QKeyEvent*>( event );
            switch( ke->key() ) {
            case Qt::Key_Up:
            case Qt::Key_Down:
            case Qt::Key_PageUp:
            case Qt::Key_PageDown:
                d->navigationPerformed = true;
                return false;

            case Qt::Key_Return:
            case Qt::Key_Enter: {
                if ( d->m_annotationsView->selectionModel()->hasSelection() ) {
                    d->executeAnnotationAtIndex( d->m_annotationsView->selectionModel()->selectedIndexes().first() );
                }
                return true;
            }

            case Qt::Key_Escape:
                d->m_filterEdit->clear();
                d->_k_slotFilterChanged(QString());
                return true;

            default:
                (static_cast<QObject *>(d->m_filterEdit))->event(ke);
                return true;
            }
        }
        break;
    }

    case QEvent::MouseButtonRelease: {
        QMouseEvent* me = static_cast<QMouseEvent*>( event );
        if ( d->m_annotationsView->isVisible() &&
             !d->m_filterEdit->rect().contains( d->m_filterEdit->mapFromGlobal( me->globalPos() ) ) &&
            !d->m_annotationsView->rect().contains( d->m_annotationsView->mapFromGlobal( me->globalPos() ) ) ) {
            d->m_annotationsView->hide();
        }
        break;
    }

    case QEvent::MouseButtonDblClick: {
        QMouseEvent* me = static_cast<QMouseEvent*>( event );
        if ( me->button() == Qt::LeftButton ) {
            if ( d->m_annotationsView->isVisible() ) {
                QPoint pos = d->m_annotationsView->viewport()->mapFromGlobal( me->globalPos() );
                QModelIndex index = d->m_annotationsView->indexAt( pos );
                if ( index.isValid() ) {
                    d->executeAnnotationAtIndex( index );
                    return true;
                }
            }

            if ( d->m_filterEdit->rect().contains( d->m_filterEdit->mapFromGlobal( me->globalPos() ) ) ) {
                d->showAnnotations();
                return true;
            }
        }
        break;
    }

    case QEvent::FocusIn:
        if ( watched == d->m_filterEdit ) {
            d->showInfoMessage( false );
            if ( d->m_stickyAnnotations &&
                 !d->m_annotationsView->isVisible() ) {
                d->showAnnotations();
            }
        }
        break;

    case QEvent::FocusOut:
        if ( d->m_eatFocusOut )
            return true;

        d->m_filterEdit->clear();
        d->_k_slotFilterChanged(QString());
        d->showInfoMessage( true );
        break;

    default:
        return false;
    }
    return false;
}


void Nepomuk::AnnotationWidget::setResource( const Resource& resource )
{
    d->m_model->setResource( resource );

    // FIXME: use a custom filter model or some other means to support AnnotationWidget subclasses
    d->m_sortModel->setFilterResource( resource );

    if ( !d->m_infoMessageShown )
        d->m_filterEdit->clear();
    if ( !d->m_stickyAnnotations )
        d->m_annotationsView->hide();
}


Nepomuk::Resource Nepomuk::AnnotationWidget::resource() const
{
    return d->m_model->resource();
}


void Nepomuk::AnnotationWidget::createAnnotation( Nepomuk::Annotation* anno )
{
    kDebug() << "Creating annotation" << anno;
    anno->create( d->m_model->resource() );
}

#include "annotationwidget.moc"
