/*
   Copyright (c) 2008-2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "resourceannotationmodel.h"
#include "annotation.h"
#include "annotationpluginwrapper.h"
#include "annotationplugin.h"
#include "annotationpluginfactory.h"
#include "annotationrequest.h"
#include "strigitextextractionjob.h"
#include "nfo.h"
#include "nie.h"
#include "pimo.h"

#include <kicon.h>
#include <kdebug.h>
#include <kurl.h>

#include <Nepomuk/Resource>
#include <Nepomuk/Variant>

#include <QtCore/QFile>

#include <Soprano/Vocabulary/Xesam>


class Nepomuk::ResourceAnnotationModel::Private
{
public:
    Private( ResourceAnnotationModel* parent )
        : m_autoUpdate( true ),
          m_strigiJob( 0 ),
          q( parent ) {
    }

    AnnotationPluginWrapper* m_wrapper;
    AnnotationPluginFactory* pluginFactory;

    Nepomuk::Resource resource;
    QString content;
    QString filter;

    bool m_autoUpdate;

    StrigiTextExtractionJob* m_strigiJob;

    void updateAnnotations( const QString& content );
    void fireUpStrigi( const KUrl& url );
    void _k_strigiResult( KJob* );

private:
    ResourceAnnotationModel* q;
};


void Nepomuk::ResourceAnnotationModel::Private::updateAnnotations( const QString& content )
{
    AnnotationRequest request( resource );
    request.setFilter( filter );
    request.setText( content );
    m_wrapper->getPossibleAnnotations( request );
}


void Nepomuk::ResourceAnnotationModel::Private::fireUpStrigi( const KUrl& url )
{
    // fire up the text analysis
    if ( m_strigiJob )
        m_strigiJob->disconnect( q );
    m_strigiJob = new StrigiTextExtractionJob();
    connect( m_strigiJob, SIGNAL( result( KJob* ) ),
             q, SLOT( _k_strigiResult( KJob* ) ) );
    m_strigiJob->setUrl( url );
    m_strigiJob->start();
}


void Nepomuk::ResourceAnnotationModel::Private::_k_strigiResult( KJob* )
{
    QString text = m_strigiJob->text();
    kDebug() << text.left(10);
    m_strigiJob = 0;
    updateAnnotations( text );
}


Nepomuk::ResourceAnnotationModel::ResourceAnnotationModel( QObject* parent )
    : AnnotationModel( parent ),
      d( new Private(this) )
{
    d->pluginFactory = new AnnotationPluginFactory( this );
    d->m_wrapper = new AnnotationPluginWrapper( this );
    connect( d->m_wrapper, SIGNAL( newAnnotation( Nepomuk::Annotation* ) ),
             this, SLOT( addNewAnnotation( Nepomuk::Annotation* ) ) );
    connect( d->m_wrapper, SIGNAL( newAnnotation( Nepomuk::Annotation* ) ),
             this, SIGNAL( newAnnotation( Nepomuk::Annotation* ) ) );
    connect( d->m_wrapper, SIGNAL( finished() ),
             this, SIGNAL( finished() ) );
}


Nepomuk::ResourceAnnotationModel::~ResourceAnnotationModel()
{
    delete d;
}


Nepomuk::Resource Nepomuk::ResourceAnnotationModel::resource() const
{
    return d->resource;
}


void Nepomuk::ResourceAnnotationModel::setResource( const Nepomuk::Resource& res )
{
    d->resource = res;
    if ( d->m_autoUpdate )
        updateAnnotations();
}


void Nepomuk::ResourceAnnotationModel::setContent( const QString& content )
{
    d->content = content;
    if ( d->m_autoUpdate )
        updateAnnotations();
}


void Nepomuk::ResourceAnnotationModel::setFilter( const QString& filter )
{
    d->filter = filter;
    if ( d->m_autoUpdate )
        updateAnnotations();
}


void Nepomuk::ResourceAnnotationModel::updateAnnotations()
{
    clear( true );
    doUpdateAnnotations();
}


void Nepomuk::ResourceAnnotationModel::doUpdateAnnotations()
{
    d->m_wrapper->setPlugins( AnnotationPluginFactory::instance()->getPluginsSupportingAnnotationForResource( resource().resourceUri() ) );

    if ( d->content.isEmpty() ) {
        QString content = d->resource.property( Nepomuk::Vocabulary::NIE::plainTextContent() ).toString();
        if ( content.isEmpty() ) {
            Resource fileRes = d->resource;
            if ( fileRes.hasType( Nepomuk::Vocabulary::PIMO::Thing() ) &&
                 fileRes.hasProperty( Nepomuk::Vocabulary::PIMO::groundingOccurrence() ) ) {
                fileRes = fileRes.property( Nepomuk::Vocabulary::PIMO::groundingOccurrence() ).toResource();
            }

            QUrl url = fileRes.property( Nepomuk::Vocabulary::NIE::url() ).toResource().resourceUri();
            if ( QFile::exists( url.toLocalFile() ) ) {
                d->fireUpStrigi( url );
                return;
            }
            url = fileRes.property( Soprano::Vocabulary::Xesam::url() ).toResource().resourceUri();
            if ( QFile::exists( url.toLocalFile() ) ) {
                d->fireUpStrigi( url );
                return;
            }
            // fallback in case file urls are used as resource uris
            else if ( QFile::exists( fileRes.resourceUri().toLocalFile() ) ) {
                d->fireUpStrigi( fileRes.resourceUri() );
                return;
            }
        }
    }

    // fallback: no strigi started
    d->updateAnnotations( d->content );
}


void Nepomuk::ResourceAnnotationModel::setAutoUpdate( bool b )
{
    d->m_autoUpdate = b;
}

#include "resourceannotationmodel.moc"
