/*
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "resourceeditor.h"
#include "ui_resourceeditor.h"
#include "resourcerelationview.h"
#include "resourceannotationmodel.h"
#include "annotationdelegate.h"
#include "annotationpluginwrapper.h"
#include "annotationpluginfactory.h"
#include "annotation.h"
#include "annotationrequest.h"
#include "annotationrelevancefiltermodel.h"
#include "annotationwidget.h"

#include <klineedit.h>
#include <kdialog.h>
#include <krun.h>
#include <kicon.h>

#include <kratingwidget.h>
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Types/Class>

#include <QtGui/QHBoxLayout>
#include <QtGui/QDragEnterEvent>
#include <QtGui/QStackedWidget>
#include <QtCore/QProcess>

#include <Soprano/Util/SignalCacheModel>

namespace {
    QString typesToText( const QList<QUrl>& types )
    {
        QList<Nepomuk::Types::Class> typeClasses;
        foreach( const QUrl& type, types ) {
            typeClasses << Nepomuk::Types::Class( type );
        }

        // remove all types that are supertypes of others in the list
        QList<Nepomuk::Types::Class> normalizedTypes;
        for ( int i = 0; i < typeClasses.count(); ++i ) {
            Nepomuk::Types::Class& type = typeClasses[i];
            bool use = true;
            for ( int j = 0; j < typeClasses.count(); ++j ) {
                if ( type != typeClasses[j] &&
                     typeClasses[j].isSubClassOf( type ) ) {
                    use = false;
                    break;
                }
            }
            if ( use ) {
                normalizedTypes << type;
            }
        }

        // extract the labels
        QStringList typeStrings;
        for ( int i = 0; i < normalizedTypes.count(); ++i ) {
            typeStrings << normalizedTypes[i].label();
        }

        return typeStrings.join( ", " );
    }
}


class Nepomuk::ResourceEditor::Private : public Ui::ResourceEditor
{
public:
    Nepomuk::ResourceEditor* q;

    Resource m_resource;

    Nepomuk::ResourceAnnotationModel* m_annotationModel;
    Nepomuk::AnnotationPluginWrapper* m_annotator;
    Nepomuk::AnnotationDelegate* m_annotationDelegate;
    Nepomuk::AnnotationRelevanceFilterModel* m_annotationFilterModel;

    void updateWidgets();
    void updateAnnotations();
    void _k_ratingChanged( int );
    void _k_labelChanged( bool accepted );
    void _k_descriptionChanged( bool accepted );
    void _k_slotDataChanged();
    void _k_annotationSuggestionModelChanged();
    void _k_resourceEditRequested( const Nepomuk::Resource& res );
    void _k_openNepomukBrowser();
    void _k_annotationWidgetToggled( bool );
};


void Nepomuk::ResourceEditor::Private::updateWidgets()
{
    m_labelEditor->setText( m_resource.label() );
    m_descriptionEditor->setText( m_resource.description() );
    m_ratingWidget->setRating( m_resource.rating() );
    m_titleLabel->setText( QString( "%1 (%2)" ).arg( m_resource.genericLabel(), typesToText( m_resource.types() ) ) );
    m_titleLabel->setUrl( m_resource.resourceUri().toString() );
}


void Nepomuk::ResourceEditor::Private::updateAnnotations()
{
    m_suggestionWidgetContainer->hide();
    m_annotationModel->updateAnnotations();
}


void Nepomuk::ResourceEditor::Private::_k_ratingChanged( int rating )
{
    if ( m_resource.isValid() ) {
        m_resource.setRating( rating );
        emit q->annotationsChanged();
    }
}


void Nepomuk::ResourceEditor::Private::_k_labelChanged( bool accepted )
{
    if ( accepted ) {
        m_resource.setLabel( m_labelEditor->text() );
        emit q->annotationsChanged();
    }
}


void Nepomuk::ResourceEditor::Private::_k_descriptionChanged( bool accepted )
{
    if ( accepted ) {
        m_resource.setDescription( m_descriptionEditor->text() );
        emit q->annotationsChanged();
    }
}


void Nepomuk::ResourceEditor::Private::_k_slotDataChanged()
{
    updateWidgets();
}


void Nepomuk::ResourceEditor::Private::_k_annotationSuggestionModelChanged()
{
    m_suggestionWidgetContainer->show();
}


void Nepomuk::ResourceEditor::Private::_k_resourceEditRequested( const Nepomuk::Resource& res )
{
    Nepomuk::Thing thing = Resource( res ).pimoThing();
    KDialog dlg( q );
    Nepomuk::ResourceEditor* re = new Nepomuk::ResourceEditor( &dlg );
    re->setResource( thing );
    dlg.setMainWidget( re );
    dlg.setButtons( KDialog::Close );
    dlg.setCaption( i18n( "Editing Resource %1", thing.genericLabel() ) );
    dlg.exec();
}


void Nepomuk::ResourceEditor::Private::_k_openNepomukBrowser()
{
    QProcess::startDetached( "kde-open", QStringList() << m_resource.resourceUri().toString() );
}


void Nepomuk::ResourceEditor::Private::_k_annotationWidgetToggled( bool on )
{
    if ( on )
        m_annotationWidgetStack->setCurrentWidget( m_statementEditor );
    else
        m_annotationWidgetStack->setCurrentWidget( m_annotationWidget );
}


Nepomuk::ResourceEditor::ResourceEditor( QWidget* parent )
    : QWidget( parent ),
      d( new Private() )
{
    // setup ui
    d->q = this;
    d->setupUi( this );

    d->m_annotationModel = new Nepomuk::ResourceAnnotationModel( d->m_suggestionView );
    d->m_annotationModel->setAutoUpdate( false );
    d->m_annotationFilterModel = new Nepomuk::AnnotationRelevanceFilterModel( d->m_suggestionView );
    d->m_annotationFilterModel->setMinRelevance( 0.01 );
    d->m_annotationFilterModel->setSortRole( Nepomuk::AnnotationModel::RelevanceRole );
    d->m_annotationFilterModel->setDynamicSortFilter( true );
    d->m_annotationFilterModel->sort( 0, Qt::DescendingOrder );
    d->m_annotationFilterModel->setSourceModel( d->m_annotationModel );
    d->m_suggestionView->setModel( d->m_annotationFilterModel );
    d->m_annotationDelegate = new Nepomuk::AnnotationDelegate( d->m_suggestionView );
    d->m_suggestionView->setItemDelegate( d->m_annotationDelegate );
    d->m_suggestionView->setDragDropMode( QAbstractItemView::DragOnly );

    d->m_relationsView->installEventFilter( this );
    d->m_relationsView->setAcceptDrops( true );

    d->m_annotationToggleButton->setIcon( KIcon( "edit-redo" ) );

    // connect to data changes
    Soprano::Util::SignalCacheModel* scm = new Soprano::Util::SignalCacheModel( ResourceManager::instance()->mainModel() );
    scm->setParent( this );
    scm->setCacheTime( 500 ); // updating twice a second is enough
    connect( scm,
             SIGNAL( statementsAdded() ),
             this, SLOT( _k_slotDataChanged() ) );
    connect( scm,
             SIGNAL( statementsRemoved() ),
             this, SLOT( _k_slotDataChanged() ) );


    // connect the gui
    connect( d->m_ratingWidget, SIGNAL( ratingChanged( int ) ),
             this, SLOT( _k_ratingChanged( int ) ) );
    connect( d->m_labelEditor, SIGNAL( editingFinished( bool ) ),
             this, SLOT( _k_labelChanged( bool ) ) );
    connect( d->m_descriptionEditor, SIGNAL( editingFinished( bool ) ),
             this, SLOT( _k_descriptionChanged( bool ) ) );
    connect( d->m_annotationModel, SIGNAL( rowsInserted(QModelIndex, int, int) ),
             this, SLOT( _k_annotationSuggestionModelChanged() ) );
    connect( d->m_relationsView, SIGNAL( resourceEditRequested( Nepomuk::Resource ) ),
             this, SLOT( _k_resourceEditRequested( Nepomuk::Resource ) ) );
    connect( d->m_titleLabel, SIGNAL( leftClickedUrl() ),
             this, SLOT( _k_openNepomukBrowser() ) );
    connect( d->m_annotationToggleButton, SIGNAL( toggled( bool ) ),
             this, SLOT( _k_annotationWidgetToggled( bool ) ) );

    // FIXME: metadata not implemented yet
    d->m_metadataButton->hide();
}


Nepomuk::ResourceEditor::~ResourceEditor()
{
    delete d;
}


Nepomuk::Resource Nepomuk::ResourceEditor::resource() const
{
    return d->m_resource;
}


void Nepomuk::ResourceEditor::setResource( const Nepomuk::Resource& res,
                                           const QString& content,
                                           const QList<Nepomuk::Annotation*>& manualAnnotations )
{
    d->m_resource = res;
    d->m_annotationModel->setResource( res );
    d->m_annotationModel->setContent( content );
    d->m_annotationDelegate->setResource( res );
    d->m_annotationFilterModel->setFilterResource( res );
    d->m_relationsView->setResource( res );
    d->m_tagWidget->setTaggedResource( res );
    d->m_statementEditor->setResource( res );
    d->m_annotationWidget->setResource( res );
    d->updateWidgets();
    if ( manualAnnotations.isEmpty() ) {
        d->updateAnnotations();
    }
    else {
        d->m_annotationModel->clear( true );
        foreach( Annotation* anno, manualAnnotations )
            d->m_annotationModel->addNewAnnotation( anno );
    }
}


Nepomuk::AnnotationModel* Nepomuk::ResourceEditor::annotationSuggestionModel() const
{
    return d->m_annotationModel;
}


void Nepomuk::ResourceEditor::setTitleVisible( bool visible )
{
    d->m_titleLabel->setVisible( visible );
}


bool Nepomuk::ResourceEditor::eventFilter( QObject* obj, QEvent* event )
{
    if ( obj == d->m_relationsView ) {
        if ( event->type() == QEvent::DragEnter ) {
            QDragEnterEvent* de = static_cast<QDragEnterEvent*>( event );
            event->setAccepted( de->mimeData()->hasFormat( d->m_annotationModel->mimeTypes()[0] ) );
            return true;
        }
        else if ( event->type() == QEvent::Drop ) {
            QDropEvent* de = static_cast<QDragEnterEvent*>( event );
            if ( de->mimeData()->hasFormat( d->m_annotationModel->mimeTypes()[0] ) ) {
                QModelIndexList indices = d->m_annotationModel->indexListFromMimeData( de->mimeData() );
                foreach( const QModelIndex& index, indices ) {
                    Annotation* anno = d->m_annotationModel->annotationForIndex( index );
                    anno->create( d->m_resource );
                    delete anno;
                }
                return true;
            }
        }
    }

    return QWidget::eventFilter( obj, event );
}

#include "resourceeditor.moc"
