// Copyright (C) 2009 EPITA Research and Development Laboratory (LRDE)
//
// This file is part of Olena.
//
// Olena is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation, version 2 of the License.
//
// Olena is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Olena.  If not, see <http://www.gnu.org/licenses/>.
//
// As a special exception, you may use this file as part of a free
// software project without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to produce
// an executable, this file does not by itself cause the resulting
// executable to be covered by the GNU General Public License.  This
// exception does not however invalidate any other reasons why the
// executable file might be covered by the GNU General Public License.

#ifndef SCRIBO_THEO_OBJ2TXT_HH
# define SCRIBO_THEO_OBJ2TXT_HH

/// \file
///
/// Pass text lines to the Tesseract OCR.

# include <ostream>
# include <string>
# include <sstream>

# include <mln/core/image/dmorph/image_if.hh>
# include <mln/core/concept/neighborhood.hh>
# include <mln/core/site_set/box.hh>
# include <mln/data/fill.hh>
# include <mln/pw/all.hh>
# include <mln/make/w_window2d_int.hh>
# include <mln/border/resize.hh>

# include <scribo/core/macros.hh>
# include <scribo/text/clean.hh>

# ifndef THEODEBUG
#  include <tesseract/baseapi.h>
# endif


namespace scribo
{

  namespace text
  {

    using namespace mln;


    /// Pass text lines to the Tesseract OCR
    ///
    /// \param[in] objects     Text lines.
    /// \param[in] language    The language which should be recognized by
    ///		               Tesseract (fra, en, ...)
    ///
    /// \return A character string.
    //
    template <typename L>
    std::string
    obj2txt(const object_image(L)& objects,
	    const char *language);



# ifndef MLN_INCLUDE_ONLY


    template <typename L>
    std::string
    obj2txt(const object_image(L)& objects,
	    const char *language)
    {
      trace::entering("scribo::text::obj2txt");

      mln_precondition(objects.is_valid());

      // Initialize Tesseract.
# ifndef THEODEBUG
      TessBaseAPI::InitWithLanguage(NULL, NULL, language, NULL, false, 0, NULL);
# endif

      typedef mln_ch_value(L,bool) I;
      int vals[] = { 0, 9, 0, 9, 0,
		     9, 6, 4, 6, 9,
		     0, 4, 0, 4, 0,
		     9, 6, 4, 6, 9,
		     0, 9, 0, 9, 0 };
      w_window2d_int dmap_win = mln::make::w_window2d_int(vals);

      std::stringstream txt;

# ifdef THEODEBUG
      char c_ = 'a';
# endif

      /// Use text bboxes with Tesseract
      for_all_ncomponents(i, objects.nlabels())
	{
	  I text_ima(objects.bbox(i));
	  data::fill(text_ima, false);
	  data::fill((text_ima | (pw::value(objects) == pw::cst(i))).rw(),
		     true);
	
	  /// Improve text quality.
	  /// text_ima_cleaned domain is larger than text_ima's.
	  I text_ima_cleaned = text::clean(text_ima, dmap_win);
	  border::resize(text_ima_cleaned, 0); // Make sure there is no border.

	  // Recognize characters.
	  char* s;

# ifdef THEODEBUG
	  s = (char*)malloc(2);
	  *s = c_++;
	  *(s+1) = '\0';
# else
	  s = TessBaseAPI::TesseractRect((unsigned char*) text_ima_cleaned.buffer(),
					 sizeof (bool),			      // Pixel size.
					 text_ima_cleaned.ncols() * sizeof (bool), // Row_offset
					 0,					      // Left
					 0,					      // Top
					 text_ima_cleaned.ncols(),		      // n cols
					 text_ima_cleaned.nrows());		      // n rows
# endif

	  if (s != 0)
	    txt << s << std::endl;

	  // The string has been allocated by Tesseract. We must free it.
	  free(s);
	}

      trace::exiting("scribo::text::obj2txt");
      return txt.str();
    }


# endif // ! MLN_INCLUDE_ONLY

  } // end of namespace scribo::text

} // end of namespace scribo


#endif // ! SCRIBO_TEXT_OBJ2TXT_HH
