/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "olenatextextractionjob.h"

#include <QtCore/QThread>
#include <QtCore/QMutex>
#include <QtCore/QMutexLocker>
#include <QtCore/QFile>

#include <KIO/NetAccess>
#include <KDebug>

#include <string>

#include <mln/io/ppm/load.hh>
#include <mln/io/ppm/save.hh>
#include <mln/labeling/colorize.hh>

#include "ppm2pbm.hh"
#include "pbm2obj.hh"
#include "obj2txt.hh"


namespace {
    /**
     * Helper class to work around the fact that Tesseract is not thread-safe.
     */
    class ScriboHandler
    {
    public:
        QString extractText( const mln::labeled_image< mln::image2d<mln::value::label_16> >& obj );

    private:
        QMutex m_mutex;
    };

    QString ScriboHandler::extractText( const mln::labeled_image< mln::image2d<mln::value::label_16> >& obj )
    {
        QMutexLocker lock( &m_mutex );
        // FIXME: do not default to "fra" here but use the current locale
        return QString::fromUtf8( scribo::text::obj2txt(obj, "fra").c_str() );
    }

    K_GLOBAL_STATIC( ScriboHandler, s_scriboHandler )
}


class Nepomuk::OlenaTextExtractionJob::Private : public QThread
{
public:
    KUrl m_url;
    QString m_tmpFilePath;

    QString m_extractedText;

    mln::image2d<mln::value::rgb8> qImageToMlnImage( const QImage& image );

private:
    void run();
};


mln::image2d<mln::value::rgb8> Nepomuk::OlenaTextExtractionJob::Private::qImageToMlnImage( const QImage& image )
{
    QImage img( image );
    if ( img.format() != QImage::Format_RGB888 )
        img = image.convertToFormat( QImage::Format_RGB888 );

    const int rows = img.height();
    const int cols = img.width();

    mln::image2d<mln::value::rgb8> mlnImg( rows, cols );

    // trueg: the commented code, while nicer, crashes in the return statement below!
//     ::qMemCopy( mlnImg.buffer(),
//                 img.scanLine(0),
//                 img.numBytes() );

    mln_piter_(mln::box2d) p(mlnImg.domain());
    for_all(p) {
        QRgb c = img.pixel(p.col(), p.row());
        mlnImg(p) = mln::value::rgb8(qRed(c), qGreen(c), qBlue(c));
    }

    return mlnImg;
}


void Nepomuk::OlenaTextExtractionJob::Private::run()
{
//     mln::trace::quiet = false;
//     mln::trace::full_trace = true;

    mln::image2d<mln::value::rgb8> img = qImageToMlnImage( QImage( m_tmpFilePath ) );

    kDebug() << "1";

    // color image -> binary image
    mln::image2d<bool> bima = scribo::text::ppm2pbm(img);

    kDebug() << "2";

    // binary image -> objects
    mln::labeled_image< mln::image2d<mln::value::label_16> > obj = scribo::text::pbm2obj(bima);

    kDebug() << "3";

    m_extractedText = s_scriboHandler->extractText(obj);

    kDebug() << m_extractedText;
}


Nepomuk::OlenaTextExtractionJob::OlenaTextExtractionJob( QObject* parent )
    : KJob( parent ),
      d( new Private() )
{
    connect( d, SIGNAL( finished() ), this, SLOT( slotFinished() ) );
}


Nepomuk::OlenaTextExtractionJob::~OlenaTextExtractionJob()
{
    d->wait();
    delete d;
}


Nepomuk::OlenaTextExtractionJob* Nepomuk::OlenaTextExtractionJob::extractText( const KUrl& url )
{
    OlenaTextExtractionJob* job = new OlenaTextExtractionJob();
    job->d->m_url = url;
    return job;
}


void Nepomuk::OlenaTextExtractionJob::start()
{
    kDebug();
    if ( KIO::NetAccess::download( d->m_url, d->m_tmpFilePath, 0 ) ) {
        kDebug() << "Starting thread";
        d->start();
    }
    else {
        kDebug() << "Failed to download" << d->m_url << KIO::NetAccess::lastErrorString();
        emitResult();
    }
}


void Nepomuk::OlenaTextExtractionJob::cancel()
{
}


QString Nepomuk::OlenaTextExtractionJob::text() const
{
    return d->m_extractedText;
}


void Nepomuk::OlenaTextExtractionJob::slotFinished()
{
    KIO::NetAccess::removeTempFile( d->m_tmpFilePath );
    emitResult();
}

#include "olenatextextractionjob.moc"
