/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2007-2010 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2009 Thomas Riccardi <riccardi.thomas@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "kio_nepomuktree.h"

#include "pimo.h"
#include "nie.h"

#include <nepomuk/variant.h>
#include <nepomuk/resourcemanager.h>
#include <Nepomuk/Types/Class>

#include <KUrl>
#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>
#include <kio/global.h>
#include <klocale.h>
#include <kio/job.h>
#include <KUser>
#include <KDebug>
#include <kio/netaccess.h>

#include <QtCore/QEventLoop>

#include <Soprano/Vocabulary/Xesam>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/QueryResultIterator>
#include <Soprano/Model>
#include <Soprano/Node>

#include <Soprano/Vocabulary/NRL>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/RDFS>

#include <sys/types.h>
#include <unistd.h>

using namespace KIO;

class Nepomuk::NepomukTreeProtocol::NepomukTreeProtocol::Private
{
public:
    // groupType : example : tmo:Task
    QUrl m_groupType;
    QUrl m_groupTypeDefault;
    bool m_groupTypeInheritance;
    // superGroup : example : tmo:superTask
    QUrl m_superGroup;
    QUrl m_superGroupDefault;
    bool m_superInheritance;
    // subGroup is guessed with nrl:inverseProperty, example : tmo:subTask
    QUrl m_subGroup;
    QUrl m_subGroupDefault;
    // link : example : pimo:isRelated
    QUrl m_link;
    bool m_linkInheritance;
};

Nepomuk::NepomukTreeProtocol::NepomukTreeProtocol(const QByteArray& poolSocket, const QByteArray& appSocket)
    : KIO::ForwardingSlaveBase("nepomuktree", poolSocket, appSocket),
      d(new Private)
{
    kDebug();
    // FIXME: connect to main model and listen for changes

    // isEmpty == not given in parameter
    d->m_groupType = QUrl();
    d->m_groupTypeDefault = Nepomuk::Vocabulary::PIMO::Task();
    d->m_superGroup = QUrl();
    d->m_superGroupDefault = Nepomuk::Vocabulary::PIMO::partOf();
    d->m_subGroup = QUrl();
    d->m_subGroupDefault = Nepomuk::Vocabulary::PIMO::hasPart();
    d->m_link = QUrl();
}


Nepomuk::NepomukTreeProtocol::~NepomukTreeProtocol()
{
    kDebug();
    delete d;
}


void Nepomuk::NepomukTreeProtocol::listDir(const KUrl& url)
{
    kDebug() << "_______________" << url;

    QString fileName;
    QString group;
    if (splitUrl(url, group, fileName)) {
        if (fileName.isEmpty()) {
            listGroup(group);
        } else {
            ForwardingSlaveBase::listDir(url);
        }
    }
}


void Nepomuk::NepomukTreeProtocol::get(const KUrl& url)
{
    kDebug() << url;
    ForwardingSlaveBase::get(url);
}


void Nepomuk::NepomukTreeProtocol::mimetype(const KUrl& url)
{
    kDebug() << url;
    QString path = url.path();
    if (path.isEmpty() || path == "/") {
        mimeType("inode/directory");
        finished();
    } else {
        QString fileName;
        QString group;
        if (splitUrl(url, group, fileName)) {
            if (fileName.isEmpty()) {
                // this is a group
                mimeType("inode/directory");
                finished();
            } else {
                // mimetype of the resource, at least try
                KUrl newUrl;
                if (rewriteUrl(url, newUrl)) {
                    // FIXME : maybe call kio::mimetype() directly
                    mimeType(statResource(newUrl).stringValue(KIO::UDSEntry::UDS_MIME_TYPE));
                    finished();
                } else {
                    // FIXME : if rewriteUrl return false, it's maybe not the thing to do
                    ForwardingSlaveBase::mimetype(url);
                }
            }
        }
    }
}


void Nepomuk::NepomukTreeProtocol::stat(const KUrl& url)
{
    kDebug() << url;
    QString path = url.path();
    kDebug() << "path" << path;
    if (path.isEmpty() || path == "/") {
        //
        // stat the root path
        //
        kDebug() << "stat the root path";
        KIO::UDSEntry uds;
        uds.insert(KIO::UDSEntry::UDS_NAME, QString::fromLatin1("/"));
        uds.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
        uds.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("inode/directory"));
        uds.insert(KIO::UDSEntry::UDS_USER, KUser().loginName());

        statEntry(uds);
        finished();
    } else {
        QString fileName;
        QString group;
        if (splitUrl(url, group, fileName)) {
            if (fileName.isEmpty()) {
                // stat the group
                statEntry(createUDSEntryForGroup(nameToUri(group)));
            } else {
                // stat the resource, at least try
                KUrl newUrl;
                // FIXME : do something if rewriteUrl returns false?
                rewriteUrl(url, newUrl);
                statEntry(statResource(newUrl));
            }
        }
        finished();
    }
}


bool Nepomuk::NepomukTreeProtocol::rewriteUrl(const KUrl& url, KUrl& newURL)
{
    kDebug() << url;

    QString fileName;
    QString group;
    if (splitUrl(url, group, fileName, false)) {
        if (!group.isEmpty() && !fileName.isEmpty()) {
            // FIXME : if resource.genericLabel() contains '/', we will not find it, because of the rewrites done in statResource. We could select all resources related to the group, and search&modify name, but this can lead to several resources : this is not unique... (and probably statResource introduce a bug here : having several udsEntries

            // HACK
            // FIXME: Create a correct sparql query : need to have a unique uds_name for resources too, not only groups
            QString query = selectLinkedResourcesSPARQL(group);

            kDebug() << query;
            Soprano::QueryResultIterator it = ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql);

            while (it.next()) {
                kDebug() << "#####" << it.binding(0).uri();
                Nepomuk::Resource res(it.binding(0).uri());
                if (res.genericLabel() == fileName) {
                    newURL = res.resourceUri();

                    kDebug() << "###############" << res << newURL;
                    return true;
                }
            }
        }

        newURL = fileName; // FIXME : this should never happend : the while loop should always find, and if not found : return false?
        return true;
    } else {
        return false;
    }
}

void Nepomuk::NepomukTreeProtocol::setGroupType(const QUrl& groupType)
{
    d->m_groupType = groupType;
}

bool Nepomuk::NepomukTreeProtocol::isGroupTypeModified()
{
    return d->m_groupType.isEmpty();
}

QUrl Nepomuk::NepomukTreeProtocol::groupType()
{
    return d->m_groupType.isEmpty() ? d->m_groupTypeDefault : d->m_groupType;
}


void Nepomuk::NepomukTreeProtocol::setSuperGroup(const QUrl& superGroup)
{
    d->m_superGroup = superGroup;
}

QUrl Nepomuk::NepomukTreeProtocol::superGroup()
{
    return d->m_superGroup.isEmpty() ? d->m_superGroupDefault : d->m_superGroup;
}


void Nepomuk::NepomukTreeProtocol::setSubGroup(const QUrl& subGroup)
{
    d->m_subGroup = subGroup;
}

QUrl Nepomuk::NepomukTreeProtocol::subGroup()
{
    return d->m_subGroup.isEmpty() ? d->m_subGroupDefault : d->m_subGroup;
}

void Nepomuk::NepomukTreeProtocol::setLink(const QUrl& link)
{
    d->m_link = link;
}

QUrl Nepomuk::NepomukTreeProtocol::link()
{
    return d->m_link;
}

QString Nepomuk::NepomukTreeProtocol::checkTypeSPARQL(const QString& group)
{
    return checkTypeSPARQLAux(Soprano::Node::resourceToN3(nameToUri(group)));
}

QString Nepomuk::NepomukTreeProtocol::checkTypeSPARQLAux(const QString& group)
{
    // warning : do not use "?r" as variable in a query,
    // because it's used in several callers (selectRootGroupsSPARQL)

    QString query;

    if (d->m_groupTypeInheritance) {
        query += QString("%1 a ?st . ?st %2 %3 . ")
                 .arg(group,
                      Soprano::Node::resourceToN3(Soprano::Vocabulary::RDFS::subClassOf()),
                      Soprano::Node::resourceToN3(groupType()));
    }
    else {
        query += QString("%1 a %2 . ")
                 .arg(group,
                      Soprano::Node::resourceToN3(groupType()));
    }

    return query;
}

QString Nepomuk::NepomukTreeProtocol::checkParentRelationSPARQL(const QString& group, const QString& parentGroup)
{
    return checkParentRelationSPARQLAux(Soprano::Node::resourceToN3(nameToUri(group)), Soprano::Node::resourceToN3(nameToUri(parentGroup)));
}

QString Nepomuk::NepomukTreeProtocol::checkNoParentSPARQL(const QString& group)
{
    QString query;

    query += QString("optional { ");
    query += checkParentRelationSPARQLAux(Soprano::Node::resourceToN3(nameToUri(group)), "?sr");
    query += QString("} . filter ( !bound (?sr) ) . ");

    return query;
}

QString Nepomuk::NepomukTreeProtocol::checkParentRelationSPARQLAux(const QString& group, const QString& parentGroup)
{
    // warning : do not use "?sr" as variable in a query,
    // because it's used in several callers (selectRootGroupsSPARQL, checkNoParentSPARQL)
    QString query;
    QString queryTemplate = (d->m_superInheritance) ?
                            QString("{ { %1 <%2> %3 . } UNION { %1 ?p %3 . ?p <%4> <%2> . } } ") :
                            QString("{ %1 <%2> %3 . } ");

    // superGroup
    // { ?t pimo:partOf ?st . } UNION { ?t ?p ?st . ?p rdfs:subPropertyOf pimo:partOf . }
    // pimo:partOf or type defined
    query += queryTemplate
             .arg(group)
             .arg(superGroup().toString())
             .arg(parentGroup)
             .arg(Soprano::Vocabulary::RDFS::subPropertyOf().toString());

    query += QString("UNION ");

    //TODO: replace the hardcoded subGroup by something like UNION { %1 ?p ?r . ?r ?pp %1 . ?p nrl:inverseProperty ?pp . }

    //subGroup
    // { ?t pimo:hasPart ?st . } UNION { ?t ?p ?st . ?p rdfs:subPropertyOf pimo:hasPart . }
    // pimo:hasPart or type defined
    query += queryTemplate
             .arg(parentGroup)
             .arg(subGroup().toString())
             .arg(group)
             .arg(Soprano::Vocabulary::RDFS::subPropertyOf().toString());


    return query;
}


bool Nepomuk::NepomukTreeProtocol::preCheckUri(const QUrl& uri)
{
    return uri.isValid() && !uri.isRelative();
}


bool Nepomuk::NepomukTreeProtocol::parseBool(const QString& str, bool defaultValue)
{
    bool b = defaultValue;

    if (str == "1") {
        b = true;
    } else if (str == "0") {
        b = false;
    }

    return b;
}

bool Nepomuk::NepomukTreeProtocol::parseGroupType(const QString& str)
{
    bool error = true;
    QUrl groupType = QUrl(str);
    kDebug() << str << groupType;

    if (groupType.isEmpty()) {
        error = false;
    }

    else if (preCheckUri(groupType)) {     //FIXME ERROR: something else for errors ?
        QString query = QString("ask where { <%1> a <%2> . }")
                        .arg(groupType.toString())
                        .arg(Soprano::Vocabulary::RDFS::Class().toString());
        kDebug() << "____ type check" << query;

        if (ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql).boolValue()) {
            //FIXME ERROR: better error messages :
            // Error : wrong type parameter
            setGroupType(groupType);
            error = false;
        }

        kDebug() << "____ type check result" << !error;
    }

    kDebug() << !error;
    return !error;
}

// has to be called after parseGroupType
bool Nepomuk::NepomukTreeProtocol::parseSuperGroup(const QString& str)
{
    bool error = true;
    QUrl superGroup = QUrl(str);
    kDebug() << str << superGroup;

    if (superGroup.isEmpty()) {
        error = false;
    }

    else if (preCheckUri(superGroup) &&    //FIXME ERROR: something else for errors ?
             !isGroupTypeModified() // we need to have a groupType if a superGroup is given //FIXME ERROR: better error messages : Error : need type parameter if superGroup parameter is used
        ) {
        QString query = QString("select ?r where { "
                                "<%1> a <%2> . "
                                "{ <%1> <%3> <%4> . } UNION { <%1> <%3> ?range . <%4> <%5> ?range . } . "
                                "<%1> <%6> ?r . }")
                        .arg(superGroup.toString())
                        .arg(Soprano::Vocabulary::RDF::Property().toString())
                        .arg(Soprano::Vocabulary::RDFS::range().toString())
                        .arg(groupType().toString())
                        .arg(Soprano::Vocabulary::RDFS::subClassOf().toString())
                        .arg(Soprano::Vocabulary::NRL::inverseProperty().toString());
        kDebug() << "____ super check" << query;

        Soprano::QueryResultIterator it = ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql);
        //FIXME ERROR: better error messages :
        // Error : wrong super parameter
        if (it.next()) {   // we consider there is only one nrl:inverseProperty maximum
            setSuperGroup(superGroup);
            setSubGroup(it.binding(0).uri());
            kDebug() << "____ super check inverse property:" << d->m_subGroup;
            error = false;
        }

        kDebug() << "____ super check result" << !error;
    }

    kDebug() << !error;
    return !error;
}

bool Nepomuk::NepomukTreeProtocol::parseLink(const QString& str)
{
    bool error = true;
    QUrl link = QUrl(str);
    kDebug() << str << link;

    if (link.isEmpty()) {
        error = false;
    }

    else if (preCheckUri(link)    //FIXME ERROR: something else for errors ?
        ) {
        QString query = QString("ask where { <%1> a <%2> . }")
                        .arg(link.toString())
                        .arg(Soprano::Vocabulary::RDF::Property().toString());
        kDebug() << "____ link check" << query;

        if (ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql).boolValue()) {
            //FIXME ERROR: better error messages :
            // Error : wrong link parameter
            setLink(link);
            error = false;
        }

        kDebug() << "____ link check result" << !error;
    }

    kDebug() << !error;
    return !error;
}

bool Nepomuk::NepomukTreeProtocol::splitUrl(const KUrl& url, QString& group, QString& fileName, bool signalError)
{
    kDebug() << url;
    fileName = url.fileName(KUrl::ObeyTrailingSlash);
    QString dir = url.directory(KUrl::ObeyTrailingSlash);
    QStringList groups = dir.split('/', QString::SkipEmptyParts);

    kDebug() << "@@@@@@@@@@@@@@@" << url << dir << groups << fileName;

    // parse the url paramenters
    bool preError(false);

    preError |= !parseGroupType(url.queryItemValue("type"));
    d->m_groupTypeInheritance = parseBool(url.queryItemValue("typeInheritance"), true);
    kDebug() << "groupType" << !preError << d->m_groupType << ", inheritance:" << d->m_groupTypeInheritance;

    preError |= !parseSuperGroup(url.queryItemValue("super"));
    d->m_superInheritance = parseBool(url.queryItemValue("superInheritance"), true);
    kDebug() << "superGroup" << !preError << d->m_superGroup << ", inheritance:" << d->m_superInheritance;

    preError |= !parseLink(url.queryItemValue("link"));
    d->m_linkInheritance = parseBool(url.queryItemValue("linkInheritance"), false);
    kDebug() << "link" << !preError << d->m_link << ", inheritance:" << d->m_linkInheritance;


    // check if the groups exists and have the asked parent relations
    // quick check on groups uri : is it a real absolute url?
    foreach(const QString& group, groups) {
        preError |= !preCheckUri(nameToUri(group));
    }
    if (!preError && !groups.isEmpty()) {
        QStringList tmpGroups(groups);
        QString query("ask where { ");
        // first element : correct type
        query += checkTypeSPARQL(tmpGroups.first());

        // no parent
        query += checkNoParentSPARQL(tmpGroups.first());

        QString prevGroup = tmpGroups.first();
        tmpGroups.removeFirst();

        foreach(const QString& group, tmpGroups) {
            query += checkTypeSPARQL(group);
            query += checkParentRelationSPARQL(group, prevGroup);
        }
        query += '}';
        kDebug() << "____1" << query;

        preError = !ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql).boolValue();
        kDebug() << "____1 result" << !preError;
    }

    // we need to check if there is a group called fileName.
    if (!preError && !fileName.isEmpty() &&
        preCheckUri(nameToUri(fileName))     // quick check : is it a real absolute url?
        ) {

        QString query = QString("ask where { ");
        query += checkTypeSPARQL(fileName);

        if (!groups.isEmpty()) {
            query += checkParentRelationSPARQL(fileName, groups.last());
        } else {
            // check if no parent
            query += checkNoParentSPARQL(fileName);
        }
        query += '}';
        kDebug()  << "____2" << query;

        if (ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql).boolValue()) {
            groups << fileName;
            fileName = QString();
            kDebug()  << "____2 result true";
        } else {
            kDebug()  << "____2 result false";
        }
    }
    kDebug() << url << preError << groups << fileName;
    if (!preError && (fileName.isEmpty() || !groups.isEmpty())) {
        if (!groups.isEmpty()) {
            group = groups.last(); // we need only the last group
        } else {
            group = QString();
        }

        kDebug() << url << group << fileName;
        return true;
    } else {
        // FIXME ERROR: do nicer errors like : "group '..' doesn't exist", or "group '..' has no parent '...'"
        if (signalError) {
            error(ERR_MALFORMED_URL, url.url());
        }

        return false;
    }
}


QString Nepomuk::NepomukTreeProtocol::selectLinkedResourcesSPARQL(const QString& group)
{
    QString query = QString("select DISTINCT ?r where { ");

    // warning, do not use ?p, since it's used after

    // QString queryTemplate = QString( "{ ?r %1 <%2> . ?r a ?type . } UNION { <%2> ?pp ?r . ?r ?pp %1 . %1 <%3> ?pp . ?r a ?type . } . " )
    //     .replace( "%3", Soprano::Vocabulary::NRL::inverseProperty().toString() );

    QString queryTemplate = QString("?r %1 %2 . ?r a ?type . ");

    // filter the resources that have the type groupType, and the link with the group is of superGroup
    // this to avoid duplicates
    QString queryTemplateFilter = QString("FILTER ( ?type != <%1> ) . ")
                                  .arg(groupType().toString());
    queryTemplateFilter += QString("FILTER ( %1 != <%2> || %1 != <%3> ) . ")
                           .replace("%2", superGroup().toString())
                           .replace("%3", subGroup().toString());

    kDebug() << queryTemplate;

    if (link().isEmpty()) {
        query += (queryTemplate + queryTemplateFilter)
                 .arg("?p")
                 .arg(Soprano::Node::resourceToN3(nameToUri(group)));
    } else {
        if (d->m_linkInheritance) {
            queryTemplate = "{ " + queryTemplate + "} ";
            queryTemplate += QString("UNION { ?r ?sp %1 . ?sp %2 %3 . ?r a ?type . } . ")
                             .arg(Soprano::Node::resourceToN3(nameToUri(group)))
                             .arg(Soprano::Node::resourceToN3(Soprano::Vocabulary::RDFS::subPropertyOf()))
                             .arg(Soprano::Node::resourceToN3(link()));
        }

        query += (queryTemplate + queryTemplateFilter)
                 .arg(Soprano::Node::resourceToN3(link()))
                 .arg(Soprano::Node::resourceToN3(nameToUri(group)));
    }

    // we remove literal values: this is a resource navigator
    query += QString("FILTER ( !isLiteral( ?r ) ) . ");

    query += QString("}");

    kDebug() << query;

    return query;
}

QString Nepomuk::NepomukTreeProtocol::selectSubGroupsSPARQL(const QString& group)
{
    QString query = QString("select DISTINCT ?r where { ");

    query += checkTypeSPARQL(group);

    query += checkParentRelationSPARQLAux("?r", Soprano::Node::resourceToN3(nameToUri(group)));

    query += QString("}");

    return query;
}

QString Nepomuk::NepomukTreeProtocol::selectRootGroupsSPARQL()
{
    QString query = QString("select DISTINCT ?r where { ");

    query += checkTypeSPARQLAux("?r");

    query += QString("optional { ");
    query += checkParentRelationSPARQLAux("?r", "?sr");
    query += QString("} . filter ( !bound (?sr) ) . ");

    query += QString("}");

    return query;
}

void Nepomuk::NepomukTreeProtocol::listGroup(const QString& groupName)
{
    kDebug() << groupName;

    if (!groupName.isEmpty()) {
        // list things related to the group
        QString query = selectLinkedResourcesSPARQL(groupName);

        kDebug() << "1 list things related" << query;
        Soprano::QueryResultIterator it = ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql);
        while (it.next()) {
            listEntry(statResource(it.binding(0).uri()), false);
        }


        // list sub-folders
        query = selectSubGroupsSPARQL(groupName);

        kDebug() << "2 list sub folders" << query;
        it = ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql);
        while (it.next()) {
            listEntry(createUDSEntryForGroup(it.binding(0).uri()), false);
        }
    }


    // display all groups that have no superGroup
    else {
        QString query = selectRootGroupsSPARQL();

        kDebug() << "0 list root groups" << query;
        Soprano::QueryResultIterator it = ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql);
        while (it.next()) {
            listEntry(createUDSEntryForGroup(it.binding(0).uri()), false);
        }
    }

    listEntry(KIO::UDSEntry(), true);

    finished();
}

// WARNING : this is a copy of ../tags/kio_tags.cpp
// TODO : move this in nepomuk:/?uri=<uri> and use this
// TODO : use a unique uds_name with percent encoding, like for the groups
// TODO : if empty name given by the kio job, set something else (kio_http do that sometime...)
// TODO : if no creation/modification date for a resource not file, do not set it
// TODO : use UDS_DISPLAY_TYPE ?
KIO::UDSEntry Nepomuk::NepomukTreeProtocol::statResource(const KUrl& uri)
{
    kDebug() << uri;

    KIO::UDSEntry uds;

    Nepomuk::Resource res(uri);
    KUrl url = res.property(Nepomuk::Vocabulary::NIE::url()).toResource().resourceUri();
    if (url.isEmpty()) {
        url = res.property(Soprano::Vocabulary::Xesam::url()).toResource().resourceUri();
        if (url.isEmpty())
            url = uri;
    }
    bool isFile = false;
    if (!url.isEmpty() && url.scheme() != "akonadi") {   // do not stat akonadi resouces here, way too slow, even hangs if akonadi is not running
        kDebug() << "listing file" << url;
        if (KIO::StatJob* job = KIO::stat(url, KIO::HideProgressInfo)) {
            job->setAutoDelete(false);
            if (KIO::NetAccess::synchronousRun(job, 0)) {
                uds = job->statResult();
                if (url.isLocalFile()) {
                    uds.insert(KIO::UDSEntry::UDS_LOCAL_PATH, url.toLocalFile());
                    uds.insert(KIO::UDSEntry::UDS_URL, url.url());
                }
                isFile = true;
            } else {
                kDebug() << "failed to stat" << url;
            }
            delete job;
        }
    }

    //
    // The nepomuk resource listing is the same as in the nepomuk kio slave.
    // So either only depend on that or let the nepomuk kio slave fail on each
    // stat.
    //
    QString label = res.genericLabel();
    QString name = label;

    // make sure name is not the URI (which is the fallback of genericLabel() and will lead to crashes in KDirModel)
    if (name.contains('/')) {
        name = name.section('/', -1);
        if (name.isEmpty())
            name = res.resourceUri().fragment();
        if (name.isEmpty())
            name = res.resourceUri().toString().replace('/', '_');
    }

    uds.insert(KIO::UDSEntry::UDS_NAME, name);

    if (!isFile) {
        kDebug() << "listing resource" << uri;

        uds.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, label);
        uds.insert(KIO::UDSEntry::UDS_CREATION_TIME, res.property(Soprano::Vocabulary::NAO::created()).toDateTime().toTime_t());
        uds.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, res.property(Soprano::Vocabulary::NAO::lastModified()).toDateTime().toTime_t());
        uds.insert(KIO::UDSEntry::UDS_ACCESS, 0700);
        uds.insert(KIO::UDSEntry::UDS_USER, KUser().loginName());
        uds.insert(KIO::UDSEntry::UDS_MIME_TYPE, "application/x-nepomuk-resource");

        // Use nice display types like "Person", "Project" and so on
        Nepomuk::Types::Class type( res.resourceType() );
        if (!type.label().isEmpty())
            uds.insert( KIO::UDSEntry::UDS_DISPLAY_TYPE, type.label() );
    }

    uds.insert(KIO::UDSEntry::UDS_TARGET_URL, uri.url());
#if KDE_IS_VERSION(4,3,70)
    uds.insert(KIO::UDSEntry::UDS_NEPOMUK_URI, KUrl(res.resourceUri()).url());
#endif

    // HACK : real directories are not directories in this kio
    if (uds.stringValue(KIO::UDSEntry::UDS_MIME_TYPE) == QString::fromLatin1("inode/directory") ||
        uds.numberValue(KIO::UDSEntry::UDS_FILE_TYPE) == S_IFDIR) {
        uds.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString());
        uds.insert(KIO::UDSEntry::UDS_FILE_TYPE, 0);
    }


    //
    // An icon set on the resource overrides the file's icon
    //
    QString icon = res.genericIcon();
    if (!icon.isEmpty()) {
        uds.insert(KIO::UDSEntry::UDS_ICON_NAME, icon);
    }

    //
    // FIXME: make sure we have no duplicate names
    //

    return uds;
}


QUrl Nepomuk::NepomukTreeProtocol::nameToUri(const QString& name)
{
    QUrl uri = QUrl::fromEncoded(QByteArray::fromPercentEncoding(name.toAscii(), '~'));
    kDebug() << name << uri;
    return uri;
}

QString Nepomuk::NepomukTreeProtocol::uriToName(const QUrl& uri)
{
    QString name = QString::fromAscii(uri.toEncoded().toPercentEncoding("", "~", '~'));
    kDebug() << uri << name;
    return name;
}


KIO::UDSEntry Nepomuk::NepomukTreeProtocol::createUDSEntryForGroup(const QUrl& groupUri)
{

    Nepomuk::Resource group(groupUri, groupType());

    kDebug() << groupUri << group << group.resourceUri() << group.genericLabel() << group.exists();
    // create a virtual folder stat thingi

    QString name = uriToName(groupUri);
    kDebug() << "oooooooooooo" << name;

    KIO::UDSEntry uds;
    uds.insert(KIO::UDSEntry::UDS_NAME, name);
    uds.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, group.genericLabel());
#if KDE_IS_VERSION(4,3,70)
    uds.insert(KIO::UDSEntry::UDS_NEPOMUK_URI, KUrl(groupUri).url());
#endif

    Nepomuk::Resource t(groupType());
    if (t.genericLabel() != t.resourceUri().toString()) {
        //uds.insert( KIO::UDSEntry::UDS_DISPLAY_TYPE, t.genericLabel() );
    }

    if (group.property(Soprano::Vocabulary::NAO::created()).isValid()) {
        uds.insert(KIO::UDSEntry::UDS_CREATION_TIME, group.property(Soprano::Vocabulary::NAO::created()).toDateTime().toTime_t());
    }
    if (group.property(Soprano::Vocabulary::NAO::lastModified()).isValid()) {
        uds.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, group.property(Soprano::Vocabulary::NAO::lastModified()).toDateTime().toTime_t());
    }

    uds.insert(KIO::UDSEntry::UDS_ACCESS, 0700);
    uds.insert(KIO::UDSEntry::UDS_USER, KUser().loginName());
    uds.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
    uds.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("inode/directory"));

    // UDS_EXTRA is the group URI
    uds.insert(KIO::UDSEntry::UDS_EXTRA, groupUri.toString());

    QString icon = group.genericIcon();
    if (!icon.isEmpty())
        uds.insert(KIO::UDSEntry::UDS_ICON_NAME, icon);
    else
        uds.insert(KIO::UDSEntry::UDS_ICON_NAME, "nepomuk");

    return uds;
}



extern "C"
{
    KDE_EXPORT int kdemain(int argc, char **argv) {
        // necessary to use other kio slaves
        KComponentData("kio_nepomuktree");
        QCoreApplication app(argc, argv);

        kDebug(7102) << "Starting nepomukTree slave " << getpid();

        if (argc != 4) {
            kError() << "Usage: kio_nepomukTree protocol domain-socket1 domain-socket2";
            exit(-1);
        }

        if (Nepomuk::ResourceManager::instance()->init()) {
            kError() << "Unable to initialized KMetaData.";
            exit(-1);
        }

        Nepomuk::NepomukTreeProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();

        kDebug(7102) << "NepomukTree slave Done";

        return 0;
    }
}

#include "kio_nepomuktree.moc"
