/*
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2007-2008 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "kio_tags.h"
#include "nie.h"

#include <nepomuk/tag.h>
#include <nepomuk/variant.h>
#include <nepomuk/resourcemanager.h>

#include <KUrl>
#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>
#include <kio/global.h>
#include <klocale.h>
#include <kio/job.h>
#include <KUser>
#include <KDebug>
#include <kio/netaccess.h>

#include <QtCore/QEventLoop>

#include <Soprano/Vocabulary/Xesam>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/QueryResultIterator>
#include <Soprano/Model>
#include <Soprano/Node>

#include <sys/types.h>
#include <unistd.h>

using namespace KIO;


namespace {
    KIO::UDSEntry createUDSEntryForTag( const Nepomuk::Tag& tag ) {
        kDebug() << tag.resourceUri() << tag.genericLabel() << tag.exists();
        // create a virtual folder stat thingi
        KIO::UDSEntry uds;
        uds.insert( KIO::UDSEntry::UDS_NAME, tag.genericLabel() );
        uds.insert( KIO::UDSEntry::UDS_DISPLAY_NAME, i18n( "Things tagged with '%1'", uds.stringValue( KIO::UDSEntry::UDS_NAME ) ) );
        uds.insert( KIO::UDSEntry::UDS_CREATION_TIME, tag.property( Soprano::Vocabulary::NAO::created() ).toDateTime().toTime_t() );
        uds.insert( KIO::UDSEntry::UDS_MODIFICATION_TIME, tag.property( Soprano::Vocabulary::NAO::lastModified() ).toDateTime().toTime_t() );
        uds.insert( KIO::UDSEntry::UDS_ACCESS, 0700 );
        uds.insert( KIO::UDSEntry::UDS_USER, KUser().loginName() );
        uds.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
        uds.insert( KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1( "inode/directory" ) );
        QString icon = tag.genericIcon();
        if ( !icon.isEmpty() )
            uds.insert( KIO::UDSEntry::UDS_ICON_NAME, icon );
        return uds;
    }
}



Nepomuk::TagsProtocol::TagsProtocol( const QByteArray& poolSocket, const QByteArray& appSocket )
    : KIO::ForwardingSlaveBase( "tags", poolSocket, appSocket )
{
    kDebug();
    // FIXME: connect to main model and listen for changes
}


Nepomuk::TagsProtocol::~TagsProtocol()
{
    kDebug();
}


void Nepomuk::TagsProtocol::listDir( const KUrl& url )
{
    kDebug() << url;

    QString path;
    QStringList tags;
    if ( splitUrl( url, tags, path ) ) {
        if ( path.isEmpty() ) {
            listTags( tags );
        }
        else {
            ForwardingSlaveBase::listDir( url );
        }
    }
}


void Nepomuk::TagsProtocol::mkdir( const KUrl &url, int permissions )
{
    kDebug() << url;

    QString path;
    QStringList tags;
    if ( splitUrl( url, tags, path ) ) {
        if ( path.isEmpty() && !tags.isEmpty() ) {
            Tag( tags.last() ).setLabel( tags.last() );
            finished();
        }
        else {
            ForwardingSlaveBase::mkdir( url, permissions );
        }
    }
}


void Nepomuk::TagsProtocol::get( const KUrl& url )
{
    kDebug() << url;
    ForwardingSlaveBase::get( url );
}


void Nepomuk::TagsProtocol::put( const KUrl& url, int permissions, KIO::JobFlags flags )
{
    kDebug();
    Q_UNUSED( url );
    Q_UNUSED( permissions );
    Q_UNUSED( flags );

    error( ERR_UNSUPPORTED_ACTION, i18n( "Cannot put URL '%1' into a tag without a source URL", url.prettyUrl() ) );
}


void Nepomuk::TagsProtocol::copy( const KUrl& src, const KUrl& dest, int permissions, KIO::JobFlags flags )
{
    kDebug() << src << dest;
    QString path;
    QStringList tags;
    if ( splitUrl( src, tags, path ) ) {
        QString newPath;
        QStringList newTags;
        if ( splitUrl( dest, newTags, newPath ) ) {
            if ( newTags.isEmpty() ) {
                ForwardingSlaveBase::copy( src, dest, permissions, flags );
            }
            else {
                Resource res( path );
                foreach( const QString& tag, newTags ) {
                    res.addTag( Tag( tag ) );
                }
                finished();
            }
        }
    }
}


void Nepomuk::TagsProtocol::rename( const KUrl& src, const KUrl& dest, KIO::JobFlags flags )
{
    kDebug() << src << dest;
    QString path;
    QStringList tags;
    if ( splitUrl( src, tags, path ) ) {
        if ( path.isEmpty() ) {
            QString newPath;
            QStringList newTags;
            if ( splitUrl( dest, newTags, newPath ) ) {
                if ( path.isEmpty() && !newTags.isEmpty() ) {
                    // rename tag
                    Tag theTag( tags.last() );
                    theTag.setLabel( newTags.last() );
                    theTag.setIdentifiers( QStringList( newTags.last() ) );
                    finished();
                }
                else {
                    error( ERR_CANNOT_RENAME, i18n( "No new tag name specified in URL %1." ).arg( dest.url() ) );
                }
            }
        }
        else {
            ForwardingSlaveBase::rename( src, dest, flags );
        }
    }
}


void Nepomuk::TagsProtocol::del( const KUrl& url, bool isfile )
{
    kDebug() << url << isfile;
    QString path;
    QStringList tags;
    if ( splitUrl( url, tags, path ) ) {
        if ( tags.isEmpty() ) {
            ForwardingSlaveBase::del( url, isfile );
        }
        else {
            if ( path.isEmpty() ) {
                // delete the tag altogether
                Tag( tags.last() ).remove();
                finished();
            }
            else {
                // "untag" the resource
                KUrl targetUrl;
                if ( rewriteUrl( url, targetUrl ) ) {
                    Resource res( targetUrl );
                    if ( res.isValid() ) {
                        QList<Tag> currentTags = res.tags();
                        foreach( const QString& tag, tags ) {
                            currentTags.removeAll( Tag( tag ) );
                        }
                        res.setTags( currentTags );
                        finished();
                        return;
                    }
                }
                error( ERR_MALFORMED_URL, url.url() );
            }
        }
    }
}


void Nepomuk::TagsProtocol::mimetype( const KUrl& url )
{
    kDebug() << url;
    QStringList tags;
    QString path = url.path();
    if ( path.isEmpty() || path == "/" ) {
        mimeType( "inode/directory" );
        finished();
    }
    else if ( splitUrl( url, tags, path ) ) {
        if ( path.isEmpty() ) {
            mimeType( "inode/directory" );
            finished();
        }
        else {
            ForwardingSlaveBase::mimetype( url );
        }
    }
}


void Nepomuk::TagsProtocol::stat( const KUrl& url )
{
    kDebug() << url;
    QString path = url.path();
    if ( path.isEmpty() || path == "/" ) {
        //
        // stat the root path
        //
        KIO::UDSEntry uds;
        uds.insert( KIO::UDSEntry::UDS_NAME, QString::fromLatin1( "/" ) );
        uds.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
        uds.insert( KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1( "inode/directory" ) );

        statEntry( uds );
        finished();
    }
    else {
        QStringList tags;
        if ( splitUrl( url, tags, path ) ) {
            if ( path.isEmpty() ) {
                statEntry( createUDSEntryForTag( tags.last() ) );
            }
            else {
                KUrl newUrl;
                rewriteUrl( url, newUrl );
                statEntry( statResource( newUrl ) );
            }
        }
        finished();
    }
}


bool Nepomuk::TagsProtocol::rewriteUrl( const KUrl& url, KUrl& newURL )
{
    kDebug() << url;
    QString path;
    QStringList tags;
    if ( splitUrl( url, tags, path, false ) ) {
        if ( !tags.isEmpty() ) {
            // HACK
            // FIXME: Create a correct sparql query
            Tag t( tags.last() );
            foreach( Resource res, t.tagOf() ) {
                if ( res.genericLabel() == path ) {
                    newURL = res.resourceUri();
                    return true;
                }
            }
        }

        newURL = path;
        return true;
    }
    else {
        return false;
    }
}


bool Nepomuk::TagsProtocol::splitUrl( const KUrl& url, QStringList& tags, QString& fileName, bool signalError )
{
    kDebug() << url;
    fileName = url.fileName( KUrl::ObeyTrailingSlash );
    QString dir = url.directory( KUrl::ObeyTrailingSlash );
    tags = dir.split( '/', QString::SkipEmptyParts );

    // we need to check if there is a tag called fileName. I don't think there is another way
    if ( !fileName.isEmpty() ) {
        if ( ResourceManager::instance()->mainModel()->executeQuery( QString( "ask where { ?r a <%1> . { ?r <%2> %3 . } UNION { ?r <%4> %3 . } }" )
                                                                     .arg( Soprano::Vocabulary::NAO::Tag().toString() )
                                                                     .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
                                                                     .arg( Soprano::Node( Soprano::LiteralValue( fileName ) ).toN3() )
                                                                     .arg( Soprano::Vocabulary::NAO::identifier().toString() ),
                                                                     Soprano::Query::QueryLanguageSparql ).boolValue() ) {
            tags << fileName;
            fileName = QString();
        }
    }

    if ( fileName.isEmpty() || !tags.isEmpty() ) {
        kDebug() << url << tags << fileName;
        return true;
    }
    else {
        if ( signalError ) {
            error( ERR_MALFORMED_URL, i18n( "Unable to handle URL %1", url.url() ) );
        }

        return false;
    }
}


void Nepomuk::TagsProtocol::listTags( const QStringList& tagLabels )
{
    kDebug() << tagLabels;

    // list tagged things
    QList<Tag> tags;
    if ( !tagLabels.isEmpty() ) {
        QString query = "select ?r where { ";
        foreach( const QString& tag, tagLabels ) {
            tags << Tag( tag );
            query += QString( "?r <%1> <%2> . " )
                     .arg( Soprano::Vocabulary::NAO::hasTag().toString() )
                     .arg( QString::fromAscii( tags.last().resourceUri().toEncoded() ) );
        }
        query += '}';
        kDebug() << query;
        Soprano::QueryResultIterator it = ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
        while ( it.next() ) {
            listEntry( statResource( it.binding( 0 ).uri() ), false );
        }
    }


    // list sub-folders
    QList<Tag> allTags = Tag::allTags();
    for ( QList<Tag>::iterator it = allTags.begin();
          it != allTags.end(); ++it ) {
        if ( !tags.contains( *it ) ) {
            kDebug() << "listing subfolder" << it->resourceUri() << it->genericLabel() << it->exists();
            listEntry( createUDSEntryForTag( *it ), false );
        }
        else {
            kDebug() << "NOT listing subfolder" << it->resourceUri() << it->genericLabel() << it->exists();
        }
    }


    listEntry( KIO::UDSEntry(), true );

    finished();
}


KIO::UDSEntry Nepomuk::TagsProtocol::statResource( const KUrl& uri )
{
    kDebug() << uri;

    KIO::UDSEntry uds;

    Nepomuk::Resource res( uri );
    KUrl url = res.property( Nepomuk::Vocabulary::NIE::url() ).toResource().resourceUri();
    if ( url.isEmpty() ) {
        url = res.property( Soprano::Vocabulary::Xesam::url() ).toResource().resourceUri();
        if ( url.isEmpty() )
            url = uri;
    }
    bool isFile = false;
    if ( !url.isEmpty() && url.scheme() != "akonadi" ) { // do not stat akonadi resouces here, way too slow, even hangs if akonadi is not running
        kDebug() << "listing file" << url;
        if ( KIO::StatJob* job = KIO::stat( url, KIO::HideProgressInfo ) ) {
            job->setAutoDelete( false );
            if ( KIO::NetAccess::synchronousRun( job, 0 ) ) {
                uds = job->statResult();
                if ( url.isLocalFile() ) {
                    uds.insert( KIO::UDSEntry::UDS_LOCAL_PATH, url.toLocalFile() );
                }
                isFile = true;
            }
            else {
                kDebug() << "failed to stat" << url;
            }
            delete job;
        }
    }

    //
    // The nepomuk resource listing is the same as in the nepomuk kio slave.
    // So either only depend on that or let the nepomuk kio slave fail on each
    // stat.
    //
    if ( !isFile ) {
        kDebug() << "listing resource" << uri;

        QString label = res.genericLabel();
        QString name = label;

        // make sure name is not the URI (which is the fallback of genericLabel() and will lead to crashes in KDirModel)
        if ( name.contains( '/' ) ) {
            name = name.section( '/', -1 );
            if ( name.isEmpty() )
                name = res.resourceUri().fragment();
            if ( name.isEmpty() )
                name = res.resourceUri().toString().replace( '/', '_' );
        }

        uds.insert( KIO::UDSEntry::UDS_NAME, name );
        uds.insert( KIO::UDSEntry::UDS_DISPLAY_NAME, label );
        uds.insert( KIO::UDSEntry::UDS_CREATION_TIME, res.property( Soprano::Vocabulary::NAO::created() ).toDateTime().toTime_t() );
        uds.insert( KIO::UDSEntry::UDS_MODIFICATION_TIME, res.property( Soprano::Vocabulary::NAO::lastModified() ).toDateTime().toTime_t() );
        uds.insert( KIO::UDSEntry::UDS_ACCESS, 0700 );
        uds.insert( KIO::UDSEntry::UDS_USER, KUser().loginName() );
        uds.insert( KIO::UDSEntry::UDS_MIME_TYPE, "application/x-nepomuk-resource" );
        QString icon = res.genericIcon();
        if ( !icon.isEmpty() ) {
            uds.insert( KIO::UDSEntry::UDS_ICON_NAME, icon );
        }
    }

#if KDE_IS_VERSION(4,3,70)
    uds.insert( KIO::UDSEntry::UDS_NEPOMUK_URI, uri.url() );
#endif

    //
    // FIXME: make sure we have no duplicate names
    //

    return uds;
}


extern "C"
{
    KDE_EXPORT int kdemain( int argc, char **argv )
    {
        // necessary to use other kio slaves
        KComponentData( "kio_tags" );
        QCoreApplication app( argc, argv );

        kDebug(7102) << "Starting tags slave " << getpid();

        if (argc != 4) {
            kError() << "Usage: kio_tags protocol domain-socket1 domain-socket2";
            exit(-1);
        }

        if ( Nepomuk::ResourceManager::instance()->init() ) {
            kError() << "Unable to initialized KMetaData.";
            exit( -1 );
        }

        Nepomuk::TagsProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();

        kDebug(7102) << "Tags slave Done";

        return 0;
    }
}

#include "kio_tags.moc"
