/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007-2010 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "resourcemodel.h"

#include <QtCore/QUrl>
#include <QtCore/QList>
#include <QtCore/QMimeData>

#include <KUrl>
#include <KDebug>
#include <kcategorizedsortfilterproxymodel.h>
#include <KIcon>
#include <KLocale>

#include <nepomuk/resource.h>
#include <nepomuk/class.h>
#include <nepomuk/variant.h>

#include <Soprano/Vocabulary/RDFS>
#include <Soprano/Vocabulary/NAO>


class Nepomuk::ResourceModel::Private
{
public:
};


Nepomuk::ResourceModel::ResourceModel( QObject* parent )
    : QAbstractItemModel( parent ),
      d( new Private() )
{
}


Nepomuk::ResourceModel::~ResourceModel()
{
    delete d;
}


int Nepomuk::ResourceModel::columnCount( const QModelIndex& ) const
{
    return 1;
}


QVariant Nepomuk::ResourceModel::data( const QModelIndex& index, int role ) const
{
    QUrl uri = resourceForIndex( index );
    Nepomuk::Resource res( uri );

    if( uri.isValid() ) {
        switch( role ) {
        case Qt::DisplayRole:
        case Qt::EditRole:
            return Resource( uri ).genericLabel();

        case Qt::DecorationRole:
            return KIcon( Resource( uri ).genericIcon() );

        case Qt::ToolTipRole:
            return KUrl( uri ).prettyUrl();

        case KCategorizedSortFilterProxyModel::CategorySortRole:
            // FIXME: sort files before other stuff and so on

        case KCategorizedSortFilterProxyModel::CategoryDisplayRole: {
            Q_ASSERT( !res.resourceType().isEmpty() );
            Nepomuk::Types::Class c( res.resourceType() );
            QString cat = c.label();
            if ( cat.isEmpty() ) {
                cat = c.name();
            }
            if ( c.uri() == Soprano::Vocabulary::RDFS::Resource() || cat.isEmpty() ) {
                cat = i18n( "Miscellaneous" );
            }

            return cat;
        }

        case ResourceUriRole:
            return uri;

        case ResourceCreationDate:
            return res.property( Soprano::Vocabulary::NAO::created() ).toDateTime();
        }
    }
    return QVariant();
}


QModelIndex Nepomuk::ResourceModel::parent( const QModelIndex& ) const
{
    return QModelIndex();
}


Qt::ItemFlags Nepomuk::ResourceModel::flags( const QModelIndex& index ) const
{
    if ( index.isValid() ) {
        return QAbstractItemModel::flags( index ) | Qt::ItemIsDragEnabled;
    }
    else {
        return QAbstractItemModel::flags( index );
    }
}


QMimeData* Nepomuk::ResourceModel::mimeData( const QModelIndexList& indexes ) const
{
    KUrl::List uris;
    foreach ( const QModelIndex& index, indexes ) {
        if (index.isValid()) {
            uris << index.data( ResourceUriRole ).toUrl();
        }
    }

    QMimeData* mimeData = new QMimeData();
    uris.populateMimeData( mimeData );

    QByteArray data;
    QDataStream s( &data, QIODevice::WriteOnly );
    s << uris;
    mimeData->setData( mimeTypes().first(), data );

    return mimeData;
}


//bool dropMimeData( const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent );
QStringList Nepomuk::ResourceModel::mimeTypes() const
{
    return( QStringList()
            << QLatin1String( "application/x-nepomuk-resource-uri" )
            << KUrl::List::mimeDataTypes() );
}

#include "resourcemodel.moc"
