/*
   Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "ontologyimportclient.h"

#include <KApplication>
#include <KAboutData>
#include <KCmdLineArgs>
#include <KPassivePopup>
#include <KIcon>
#include <KDebug>
#include <KMessageBox>

#include <QtGui/QApplication>


Nepomuk::OntologyImportClient::OntologyImportClient( QObject* parent )
    : QObject( parent )
{
    m_ontologyManagerInterface = new org::kde::nepomuk::OntologyManager( "org.kde.nepomuk.services.nepomukontologyloader",
                                                                         "/nepomukontologyloader",
                                                                         QDBusConnection::sessionBus(),
                                                                         this );
    connect( m_ontologyManagerInterface, SIGNAL( ontologyUpdated( QString ) ),
             this, SLOT( slotOntologyUpdated( QString ) ) );
    connect( m_ontologyManagerInterface, SIGNAL( ontologyUpdateFailed( QString, QString ) ),
             this, SLOT( slotOntologyUpdateFailed( QString, QString ) ) );

    if ( !m_ontologyManagerInterface->isValid() ) {
        KMessageBox::sorry( 0, i18nc( "@info error message", "Failed to contact Nepomuk ontology service. Is the Nepomuk Server running?" ) );
        qApp->quit();
    }
}


Nepomuk::OntologyImportClient::~OntologyImportClient()
{
}


void Nepomuk::OntologyImportClient::importOntology( const KUrl& url )
{
    kDebug() << url;
    m_importOntologyUrl = url;
    m_ontologyManagerInterface->importOntology( QString::fromAscii( url.toEncoded() ) );
    kDebug() << "queued";
}


void Nepomuk::OntologyImportClient::slotOntologyUpdated( const QString& uri )
{
    kDebug() << uri;
    if ( m_importOntologyUrl == QUrl::fromEncoded( uri.toAscii() ) ) {
        connect( KPassivePopup::message( i18nc( "@title:window", "Ontoloy Import" ),
                                         i18nc( "%1 is an ontology URL",
                                               "Successfully imported ontology %1 into Nepomuk", uri ),
                                         KIcon( "nepomuk" ).pixmap( 22, 22 ),
                                         ( QWidget* )0 ),
                 SIGNAL( destroyed( QObject* ) ),
                 qApp,
                 SLOT( quit() ) );
    }
}



void Nepomuk::OntologyImportClient::slotOntologyUpdateFailed( const QString& uri, const QString& errorMsg )
{
    kDebug() << uri << errorMsg;
    if ( m_importOntologyUrl == QUrl::fromEncoded( uri.toAscii() ) ) {
        connect( KPassivePopup::message( i18nc( "@title:window", "Ontoloy Import" ),
                                         i18nc( "%1 is an ontology URL, %2 is an error message",
                                                "Failed to import ontology %1 into Nepomuk (%2)", uri, errorMsg ),
                                         KIcon( "nepomuk" ).pixmap( 22, 22 ),
                                         ( QWidget* )0 ),
                 SIGNAL( destroyed( QObject* ) ),
                 qApp,
                 SLOT( quit() ) );
    }
}



int main( int argc, char** argv )
{
    KAboutData aboutData( "ontologyimportclient", 0,
                          ki18n("Nepomuk Ontology Importer"),
                          "0.1",
                          ki18n("Simple Client to the Nepomuk Ontology Manager Service"),
                          KAboutData::License_GPL,
                          ki18n("(c) 2008, Sebastian Trüg"),
                          KLocalizedString(),
                          "http://nepomuk.kde.org" );
    aboutData.addAuthor(ki18n("Sebastian Trüg"),ki18n("Maintainer"), "trueg@kde.org");
    aboutData.setProgramIconName( "nepomuk" );

    KCmdLineArgs::init( argc, argv, &aboutData );
    KCmdLineOptions options;
	options.add("+url", ki18n("URL pointing to the ontology to import") );
    KCmdLineArgs::addCmdLineOptions( options );

    KApplication app;

    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	if( args->count() != 1 ) {
		args->usageError( i18n( "No ontology URL specified" ) );
		return 1;
	}

    Nepomuk::OntologyImportClient client;
    client.importOntology( args->url( 0 ) );
    return app.exec();
}
