/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "resourceformatting.h"
#include "resourceformattingrule.h"
#include "resourceformattingrulefactory.h"
#include "nrdo.h"

#include <nepomuk/variant.h>
#include <nepomuk/resource.h>
#include <nepomuk/property.h>

#include <KDebug>
#include <kio/global.h>


K_GLOBAL_STATIC( Nepomuk::ResourceFormattingRuleFactory, s_factory )


QString Nepomuk::formatResource( const Resource& res, ResourceFormattingFlags flags )
{
    kDebug() << res;

    QList<ResourceFormattingRule> rules = s_factory->rulesForType( res.resourceType() );
    if ( rules.isEmpty() ) {
        QList<QUrl> types = res.types();
        types.removeAll( res.resourceType() );
        foreach( const QUrl& type, types ) {
            rules += s_factory->rulesForType( type );
        }
    }

    // let the rules do the work, the first one to produce an output wins
    QString s;
    foreach( const ResourceFormattingRule& rule, rules ) {
        kDebug() << "Using rule" << rule.type().uri() << rule.formatTemplate() << rule.properties();
        s = rule.formatResource( res, flags );
        if ( !s.isEmpty() )
            break;
    }

    // fallback
    if ( s.isEmpty() ) {
        if ( flags & FormatIncludeLinks )
            s = QString( "<a href=\"%1\">%2</a>" ).arg( res.resourceUri().toString() ).arg( res.genericLabel() );
        else
            s = res.genericLabel();
    }

    return s;
}


namespace {
    QString formatValueWithUnit( const Nepomuk::Variant& val, const QUrl& unit, Nepomuk::ResourceFormattingFlags flags )
    {
        kDebug() << val << unit;
        if ( val.simpleType() == qMetaTypeId<Nepomuk::Resource>() ) {
            QStringList s;
            foreach( const Nepomuk::Resource& res, val.toResourceList() ) {
                s << Nepomuk::formatResource( res, flags );
            }
            return s.join( ", " );
        }
        else {
            // FIXME: here we need to handle units and stuff
            // FIXME: handle lists
            switch( val.simpleType() ) {
            case QVariant::Date:
                return KGlobal::locale()->formatDate( val.toDateList().first(), KLocale::FancyLongDate );

            case QVariant::Time:
                return KGlobal::locale()->formatTime( val.toTimeList().first() );

            case QVariant::DateTime:
                return KGlobal::locale()->formatDateTime( val.toDateTimeList().first().toLocalTime(), KLocale::FancyShortDate );

            case QVariant::Int:
            case QVariant::LongLong:
            case QVariant::UInt:
            case QVariant::ULongLong:
                if ( unit == Nepomuk::Vocabulary::NRDO::Bytes() ) {
                    return KIO::convertSize( val.toInt64() );
                }

                // fallthrough

            default:
                return val.toString();
            }
        }
    }
}


QString Nepomuk::formatValue( const Variant& val, ResourceFormattingFlags flags )
{
    kDebug() << val;
    return formatValueWithUnit( val, QUrl(), flags );
}


QString Nepomuk::formatValue( const Resource& res, const Nepomuk::Types::Property& property, ResourceFormattingFlags flags )
{
#ifdef HAVE_EXPERIMANTAL_STUFF
    return formatValueWithUnit( res.property( property.uri() ), property.unit(), flags );
#else
    return formatValueWithUnit( res.property( property.uri() ), QUrl(), flags );
#endif
}
