/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "resourceformattingrulefactory.h"
#include "resourceformattingrule.h"
#include "resourceformattingrule_p.h"

#include <KStandardDirs>
#include <KGlobal>
#include <KDebug>

#include <QtCore/QRegExp>
#include <QtCore/QMap>
#include <QtCore/QFile>

#include <nepomuk/property.h>


class Nepomuk::ResourceFormattingRuleFactory::Private
{
public:
    QList<ResourceFormattingRule> rules;

    QUrl parseUri( const QString& s, QMap<QString, QString>& prefixes );
    bool parseRule( const QString& rule, QMap<QString, QString>& prefixes );
};


QUrl Nepomuk::ResourceFormattingRuleFactory::Private::parseUri( const QString& s, QMap<QString, QString>& prefixes )
{
    QRegExp prefixedRx( "(\\w+)\\:(\\w+)" );
    QRegExp uriRx( "<([^\\\"\'\\s<>]+)>" );
    if ( prefixedRx.exactMatch( s ) ) {
        QString prefix = prefixedRx.cap( 1 );
        QString name = prefixedRx.cap( 2 );
        kDebug() << "Checking prefixed uri:" << prefix << name;
        if ( prefixes.contains( prefix ) ) {
            return prefixes[prefix] + name;
        }
        else {
            kDebug() << "Prefix" << prefix << "not defined";
            return QUrl();
        }
    }
    else if ( uriRx.exactMatch( s ) ) {
        return uriRx.cap( 1 );
    }
    else {
        kDebug() << "Failed to parse URI" << s;
        return QUrl();
    }
}


bool Nepomuk::ResourceFormattingRuleFactory::Private::parseRule( const QString& ruleString, QMap<QString, QString>& prefixes )
{
    QRegExp prefixRx( "@prefix\\s+(\\w+)\\s*\\:\\s*(<[^\\s<>]+>)\\s*." );
    QRegExp patternRx( "%([^%]+)%" );

    if ( prefixRx.exactMatch( ruleString ) ) {
        QString prefix = prefixRx.cap( 1 );
        QString uri = prefixRx.cap( 2 );
        uri = uri.mid( 1, uri.length()-2 );
        prefixes.insert( prefix, uri );
        kDebug() << "Found prefix:" << prefix << uri;
        return true;
    }
    else {
        int pos = ruleString.indexOf( "->" );
        if ( pos <= 0 ) {
            kDebug() << "Rule missing -> token.";
            return false;
        }
        QUrl type = parseUri( ruleString.left( pos ).simplified(), prefixes );
        if ( type.isEmpty() ) {
            return false;
        }
        QStringList patterns = ruleString.mid( pos+2 ).split( ';' );
        foreach( QString pattern, patterns ) {
            kDebug() << "Checking pattern" << pattern;
            ResourceFormattingRule rule;
            rule.d->type = type;
            pattern = pattern.simplified();
            QUrl property = parseUri( pattern, prefixes );
            if ( !property.isEmpty() ) {
                rule.d->formatTemplate = "%1";
                rule.d->properties << ( QList<Types::Property>() << Types::Property( property ) );
                rules << rule;
                kDebug() << "Successfully parsed rule pattern" << pattern << "into rule:"
                         << rule.type().uri() << rule.d->formatTemplate << rule.d->properties;
            }
            else if ( pattern[0] == '"' && pattern[pattern.count()-1] == '"' ) {
                pattern = pattern.mid( 1, pattern.count()-2 );
                int pos = 0;
                int patternCnt = 1;
                while ( ( pos = patternRx.indexIn( pattern, pos ) ) >= 0 ) {
                    QList<Types::Property> properties;
                    QStringList propStrings = patternRx.cap( 1 ).split( "->" );
                    foreach( QString propString, propStrings ) {
                        propString = propString.simplified();
                        QUrl prop = parseUri( propString, prefixes );
                        if ( prop.isEmpty() ) {
                            kDebug() << "Failed to parse pattern URI" << propString;
                            return false;
                        }
                        properties << prop;
                    }
                    if ( properties.isEmpty() ) {
                        kDebug() << "empty pattern:" << pattern;
                        return false;
                    }
                    rule.d->properties << properties;
                    // replace pattern
                    pattern.replace( pos, patternRx.matchedLength(), "%" + QString::number( patternCnt++ ) );
                    pos += 2;
                }

                rule.d->formatTemplate = pattern;
                rules << rule;
                kDebug() << "Successfully parsed rule pattern" << pattern << "into rule:"
                         << rule.type().uri() << rule.d->formatTemplate << rule.d->properties;
            }
            else {
                kDebug() << "Failed to parse pattern" << pattern;
                return false;
            }
        }
        return true;
    }
}



Nepomuk::ResourceFormattingRuleFactory::ResourceFormattingRuleFactory()
    : d( new Private() )
{
    loadAllRules();
}


Nepomuk::ResourceFormattingRuleFactory::~ResourceFormattingRuleFactory()
{
    delete d;
}


QList<Nepomuk::ResourceFormattingRule> Nepomuk::ResourceFormattingRuleFactory::rulesForType( Types::Class type ) const
{
    QList<Nepomuk::ResourceFormattingRule> rules;

    foreach( const ResourceFormattingRule& rule, d->rules ) {
        kDebug() << "checking rule" << rule.type().uri() << rule.d->formatTemplate << rule.d->properties << "for type" << type;
        // perfectly matching rules are sorted by original order
        if ( rule.type() == type ) {
            int i = 0;
            while ( i < rules.count() && rules[i].type() == type ) {
                ++i;
            }
            rules.insert( i, rule );
        }

        // FIXME: sort by branch length
        //        we want the rule that fits best, thus, if there are classes A, B, C where C subClassOf B subClassOf A
        //        a rule for C is better than a rule for B is better than a rule for A
        else if ( type.isSubClassOf( rule.type() ) ) {
            rules << rule;
        }
    }
    return rules;
}


void Nepomuk::ResourceFormattingRuleFactory::loadAllRules()
{
    d->rules.clear();

    QStringList ruleFiles = KGlobal::dirs()->findAllResources( "data", "nepomuk/formatting/*.rules" );

    foreach( const QString& ruleFile, ruleFiles ) {
        QFile file( ruleFile );
        if ( file.open( QIODevice::ReadOnly ) ) {
            QMap<QString, QString> prefixes;
            QTextStream s( &file );
            while ( !s.atEnd() ) {
                QString line = s.readLine();
                // each rule can only span one line
                if ( !line.isNull() && line.length() > 0 && line[0] != '#' ) {
                    if ( !d->parseRule( line, prefixes ) ) {
                        kDebug() << "Failed to parse rule:" << line;
                    }
                }
            }

        }
        else {
            kDebug() << "Failed to open rule file" << ruleFile;
        }
    }
}
