/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "graphcompletor.h"

#include "autoptr.h"
#include "graphretriever.h"
#include "graphtranslator.h"
#include "graphutility.h"

#include <QtCore/QMultiHash>
#include <QtCore/QUrl>

#include <Soprano/Model>

#include <kglobal.h>

class SemClip::GraphCompletorHelper
{
public:
    GraphCompletor instance;
};

K_GLOBAL_STATIC( SemClip::GraphCompletorHelper, graphCompletorHelper )

class SemClip::GraphCompletor::Private
{
public:
};

SemClip::GraphCompletor::GraphCompletor()
    : QObject( 0 ),
      GraphCompletion(),
      d( new Private() )
{
}

SemClip::GraphCompletor::~GraphCompletor()
{
    delete d;
}

SemClip::GraphCompletor* SemClip::GraphCompletor::instance()
{
    return &graphCompletorHelper->instance;
}

void SemClip::GraphCompletor::complete( Soprano::Model* model, const QSet< QUrl >& completionGoals,
                                        const QSet< QUrl >& targetOntologies, CompletionMode mode )
{
    GraphUtility mainGraph( model );
    QSet< QUrl > resources;

    if ( mode == CompleteByProperty ) {
        // find the resource objects of those properties
        resources = mainGraph.identifyResources( completionGoals );
    } else {
        // URIs are the resources themselves
        resources = completionGoals;
    }

    // group the URIs by authority to enable potential HTTP connection reuse
    QMultiMap< QString, QUrl > connectionMap;
    foreach ( const QUrl uri, resources ) {
        connectionMap.insert( uri.authority(), uri );
    }

    // create a new model to accumulate the fragments; one translation on the
    // whole fragment graph is preferred to many single fragment translations
    AutoPtr< Soprano::Model > fragments( SemClip::createModel() );
    GraphUtility fragmentGraph( fragments.data() );

    // for each URI group, retrieve fragments and merge into the fragment graph
    foreach ( const QString key, connectionMap.uniqueKeys() ) {
        GraphRetriever retriever;
        foreach ( const QUrl uri, connectionMap.values( key ) ) {
            retriever.get( uri );
        }

        AutoPtr< Soprano::Model > fragment( retriever.model() );
        if ( !fragment.isNull() ) {
            fragmentGraph.mergeModel( fragment.constData() );
        }
    }

    // translate fragment graph to the desired target ontologies before merge
    GraphTranslator::instance()->translate( fragments.data(), targetOntologies );
    mainGraph.mergeModel( fragments.constData() );
}

void SemClip::GraphCompletor::complete( QString& serialization, const QSet< QUrl >& completionGoals,
                                        const QSet< QUrl >& targetOntologies, CompletionMode mode )
{
    AutoPtr< Soprano::Model > model( SemClip::parseModelFromRdfXml( serialization ) );
    complete( model.data(), completionGoals, targetOntologies, mode );
    serialization = SemClip::serializeModelToRdfXml( model.constData() );
}

#include "graphcompletor.moc"
