/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "graphretriever.h"

#include "autoptr.h"
#include "graphutility.h"

#include <QtCore/QByteArray>
#include <QtCore/QEventLoop>
#include <QtCore/QHash>
#include <QtCore/QPair>
#include <QtCore/QString>
#include <QtCore/QTextStream>
#include <QtCore/QTimer>
#include <QtCore/QUrl>
#include <QtNetwork/QHttp>
#include <QtNetwork/QHttpRequestHeader>

#include <Soprano/Model>
#include <Soprano/Parser>
#include <Soprano/PluginManager>
#include <Soprano/StatementIterator>

const unsigned int SemClip::GraphRetriever::s_defaultTimeoutThreshold = 8;

class SemClip::GraphRetriever::Private
{
public:
    typedef QPair< QUrl, QByteArray > FragmentPair;

    Private();

    QEventLoop*                m_loop;
    QHash< QString, QHttp* >   m_connections;
    QHash< int, FragmentPair > m_fragments;
    QTimer*                    m_timer;
    unsigned int               m_idleCount;
    unsigned int               m_timeoutThreshold;
};

SemClip::GraphRetriever::Private::Private()
    : m_loop( 0 ),
      m_connections( QHash< QString, QHttp*>() ),
      m_fragments( QHash< int, FragmentPair >() ),
      m_timer( 0 ),
      m_idleCount( 0 ),
      m_timeoutThreshold( s_defaultTimeoutThreshold )
{
}

SemClip::GraphRetriever::GraphRetriever( QObject* parent /*= 0*/ )
    : QObject( parent ),
      d( new Private() )
{
    d->m_loop = new QEventLoop( this );
    d->m_timer = new QTimer( this );
    connect( d->m_timer, SIGNAL( timeout() ), this, SLOT( timeout() ) );
}

SemClip::GraphRetriever::~GraphRetriever()
{
    delete d;
}

bool SemClip::GraphRetriever::get( const QUrl& url )
{
    // only accept HTTP URLs
    if ( url.scheme().compare( "http", Qt::CaseInsensitive ) != 0 ) {
        return false;
    }

    // use different QHttp objects for different authorities to enable connection reuse
    if ( !d->m_connections.contains( url.authority() ) ) {
        QHttp* http = new QHttp( url.host(), url.port( 80 ), this );
        http->setUser( url.userName(), url.password() );
        connect( http, SIGNAL( requestFinished( int, bool ) ), this, SLOT( httpRequestFinished( int, bool ) ) );
        connect( http, SIGNAL( done( bool ) ), this, SLOT( httpDone( bool ) ) );
        d->m_connections[ url.authority() ] = http;
    }

    // create the header
    QHttpRequestHeader header( "GET", url.path() );
    header.setValue( "Host", url.host() );
    header.setValue( "Accept", Soprano::serializationMimeType( Soprano::SerializationRdfXml ) );
    header.setValue( "Accept-Charset", "utf-8" );

    // get a QHttp object for the request and queue it
    int requestId = d->m_connections.value( url.authority() )->request( header );
    d->m_fragments.insert( requestId, Private::FragmentPair( url, QByteArray() ) );

    return true;
}

Soprano::Model* SemClip::GraphRetriever::model() const
{
    // QHttp doesn't seem to offer a timeout, so use a timer for our own
    d->m_idleCount = 0;
    d->m_timer->start( 1000 );

    // block, waiting for all requests to be finished or a timeout to occur
    d->m_loop->exec();

    // create a new Soprano model and get a parser
    Soprano::Model* result = SemClip::createModel();
    GraphUtility utility( result );
    const Soprano::Parser* parser =
        Soprano::PluginManager::instance()->discoverParserForSerialization( Soprano::SerializationRdfXml );

    // parse each fragment and add it to the model, ignoring errors
    foreach ( const Private::FragmentPair fragment, d->m_fragments ) {
        const QByteArray& data = fragment.second;
        if ( data.isEmpty() ) {
            continue;
        }

        QTextStream stream( data );
        const QUrl& url = fragment.first;
        Soprano::StatementIterator it = parser->parseStream( stream, url.toString(), Soprano::SerializationRdfXml );
        if ( !parser->lastError() ) {
            utility.mergeStatementIterator( it );
        }
    }

    d->m_fragments.clear();
    return result;
}

QString SemClip::GraphRetriever::serialization() const
{
    AutoPtr< Soprano::Model > model( this->model() );
    return SemClip::serializeModelToRdfXml( model.data() );
}

void SemClip::GraphRetriever::httpRequestFinished( int id, bool error )
{
    // reset idle counter every time a request is finished
    d->m_idleCount = 0;

    // ignore requests not generated by our GETs
    if ( !d->m_fragments.contains( id ) ) {
        return;
    }

    QHttp* http = qobject_cast< QHttp* >( sender() );
    const QHttpResponseHeader response = http->lastResponse();

    switch ( response.statusCode() ) {
    case 200:  // OK
        // not all servers return the exact requested mime type, so accept any XML
        if ( response.contentType().contains( "xml", Qt::CaseInsensitive ) ) {
            d->m_fragments[ id ].second = http->readAll();
        }
        break;

    case 301:  // Moved Permanently
    case 302:  // Found
    case 303:  // See Other
    case 307:  // Temporary Redirect
        // follow redirects
        d->m_fragments.remove( id );
        get( QUrl( response.value( "Location" ) ) );
        break;

    default:
        // consider it an error
        d->m_fragments.remove( id );
    }
}

void SemClip::GraphRetriever::httpDone( bool error )
{
    QHttp* http = qobject_cast< QHttp* >( sender() );
    foreach ( const QString key, d->m_connections.keys( http ) ) {
        d->m_connections.remove( key );
    }

    if ( d->m_connections.isEmpty() ) {
        d->m_timer->stop();
        d->m_loop->quit();
    }
}

void SemClip::GraphRetriever::timeout()
{
    // abort if no request was finished in over 10 timeouts
    if ( ++d->m_idleCount >= d->m_timeoutThreshold ) {
        d->m_connections.clear();
        d->m_timer->stop();
        d->m_loop->quit();
    }
}

bool SemClip::GraphRetriever::hasTimedOut() const
{
    return ( d->m_idleCount >= d->m_timeoutThreshold );
}

void SemClip::GraphRetriever::setTimeoutThreshold( unsigned int timeoutThreshold )
{
    if ( timeoutThreshold ) {
        d->m_timeoutThreshold = timeoutThreshold;
    } else {
        d->m_timeoutThreshold = s_defaultTimeoutThreshold;
    }
}

#include "graphretriever.moc"
