/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "graphtranslator.h"

#include "autoptr.h"
#include "graphtransformer.h"
#include "graphutility.h"
#include "localmappingstorage.h"
#include "mappingelement.h"
#include "nepomukontologyelement.h"
#include "nepomukontologystorage.h"

#include <QtCore/QString>
#include <QtCore/QUrl>

#include <Soprano/Model>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/Vocabulary/OWL>
#include <Soprano/Vocabulary/NRL>
#include <Soprano/Vocabulary/XMLSchema>

#include <KConfigGroup>
#include <kglobal.h>
#include <ksharedconfig.h>

const char SemClip::GraphTranslator::s_translatorGroupName[] = "Graph Translator";
const char SemClip::GraphTranslator::s_blacklistGroupName[]  = "Ontology Blacklist";
const char SemClip::GraphTranslator::s_vCardRdfNamespace[]   = "http://www.w3.org/2001/vcard-rdf/3.0#";

class SemClip::GraphTranslatorHelper
{
public:
    GraphTranslator instance;
};

K_GLOBAL_STATIC( SemClip::GraphTranslatorHelper, graphTranslatorHelper )

class SemClip::GraphTranslator::Private
{
public:
};

SemClip::GraphTranslator::GraphTranslator()
    : QObject( 0 ),
      GraphTranslation(),
      d( new Private() )
{
}

SemClip::GraphTranslator::~GraphTranslator()
{
    delete d;
}

SemClip::GraphTranslator* SemClip::GraphTranslator::instance()
{
    return &graphTranslatorHelper->instance;
}

void SemClip::GraphTranslator::translate( Soprano::Model* model, const QSet< QUrl >& targetOntologies )
{
    // extract the ontologies used in the graph
    GraphUtility graph( model );
    const QSet< QUrl > sourceOntologies = filterOntologies( graph.extractOntologies() );

    // retrieve ontology definitions and apply reasoning for each one found
    foreach ( const QUrl uri, sourceOntologies ) {
        const OntologyElement* ontology = NepomukOntologyStorage::instance()->findOntology( uri );
        // FIXME: Reasoning only enabled for the SWRC ontology. It is the only
        //        one that doesn't result in permanent InferenceModel errors at
        //        the moment, and without inference on SWRC our mappings won't
        //        work at all (missing Person class hierarchy type inference).
        if ( ontology && uri == QUrl( "http://swrc.ontoware.org/ontology#" ) ) {
            graph.applyReasoner( ontology->content() );
        }
    }

    // retrieve mappings and apply them
    const QSet< const MappingElement* > mappings = findMappings( sourceOntologies, targetOntologies );
    foreach ( const MappingElement* mapping, mappings ) {
        applyMapping( graph, mapping );
    }
}

void SemClip::GraphTranslator::translate( Soprano::Model* model, const QUrl& targetOntology )
{
    QSet< QUrl > targetOntologies;
    targetOntologies.insert( targetOntology );
    translate( model, targetOntologies );
}

void SemClip::GraphTranslator::translate( QString& serialization, const QSet< QUrl >& targetOntologies )
{
    AutoPtr< Soprano::Model > model( SemClip::parseModelFromRdfXml( serialization ) );
    translate( model.data(), targetOntologies );
    serialization = SemClip::serializeModelToRdfXml( model.constData() );
}

void SemClip::GraphTranslator::translate( QString& serialization, const QUrl& targetOntology )
{
    QSet< QUrl > targetOntologies;
    targetOntologies.insert( targetOntology );
    translate( serialization, targetOntologies );
}

QSet< const SemClip::MappingElement* > SemClip::GraphTranslator::findMappings( const QSet< QUrl >& sources, const QSet< QUrl >& targets )
{
    // iterate over all combinations of source and target URIs and attempt to find corresponding mappings
    QSet< const MappingElement* > mappings;
    foreach ( const QUrl sourceUri, sources ) {
        foreach ( const QUrl targetUri, targets ) {
            if ( sourceUri != targetUri ) {
                const MappingElement* element = LocalMappingStorage::instance()->findMapping( sourceUri, targetUri );
                if ( element ) {
                    mappings.insert( element );
                }
            }
        }
    }
    return mappings;
}

void SemClip::GraphTranslator::applyMapping( GraphUtility& graph, const MappingElement* mapping )
{
    // set up the graph transformer
    AutoPtr< GraphTransformer > transformer( mapping->createTransformer() );
    transformer->setModel( graph.model() );
    transformer->setTransformation( mapping->content() );

    // execute the transformation and merge the result into the graph
    AutoPtr< Soprano::Model > result( transformer->execute() );
    if ( !result.isNull() ) {
        graph.mergeModel( result.constData() );
    }
}

QSet< QUrl > SemClip::GraphTranslator::defaultOntologyBlacklist()
{
    QSet< QUrl > result;
    result.insert( Soprano::Vocabulary::RDF::rdfNamespace() );
    result.insert( Soprano::Vocabulary::RDFS::rdfsNamespace() );
    result.insert( Soprano::Vocabulary::OWL::owlNamespace() );
    result.insert( Soprano::Vocabulary::NRL::nrlNamespace() );
    result.insert( Soprano::Vocabulary::XMLSchema::xsdNamespace() );
    result.insert( QUrl( s_vCardRdfNamespace ) );
    return result;
}

QSet< QUrl > SemClip::GraphTranslator::customOntologyBlacklist()
{
    KSharedConfigPtr config = KSharedConfig::openConfig( SemClip::configFile, KConfig::NoGlobals );
    const KConfigGroup translatorGroup( config, s_translatorGroupName );
    const KConfigGroup blacklistGroup( &translatorGroup, s_blacklistGroupName );

    QSet< QUrl > result;
    foreach ( const QString key, blacklistGroup.keyList() ) {
        const QString ontologyUri = blacklistGroup.readEntry( key, QString() );
        if ( !ontologyUri.isEmpty() ) {
            result.insert( QUrl ( ontologyUri ) );
        }
    }
    return result;
}

QSet< QUrl > SemClip::GraphTranslator::filterOntologies( const QSet< QUrl >& ontologies )
{
    QSet< QUrl > result = ontologies;
    result.subtract( defaultOntologyBlacklist() );
    result.subtract( customOntologyBlacklist() );
    return result;
}

#include "graphtranslator.moc"
