/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "graphutility.h"

#include "autoptr.h"

#include <QtCore/QHash>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QTextStream>

#include <Soprano/Backend>
#include <Soprano/BackendSetting>
#include <Soprano/Global>
#include <Soprano/Inference/InferenceModel>
#include <Soprano/Inference/Rule>
#include <Soprano/Inference/StandardRuleSet>
#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/Parser>
#include <Soprano/PluginManager>
#include <Soprano/QueryResultIterator>
#include <Soprano/Serializer>
#include <Soprano/Statement>
#include <Soprano/StatementIterator>
#include <Soprano/StorageModel>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/SIL>

#include <krandom.h>

QUrl SemClip::extractNamespace( const QUrl& uri )
{
    // two types of namespace respresentations to cover:
    // - http://www.example.org/onto#identifier --> set fragment to empty
    // - http://www.example.org/onto/identifier --> copy up to rightmost '/'
    QUrl result = uri;

    if ( !result.fragment().isEmpty() ) {
        result.setFragment( QString( "" ) );
    } else {
        result = result.toString().left( result.toString().lastIndexOf( '/') + 1 );
    }

    return result;
}

Soprano::Model* SemClip::createModel()
{
    Soprano::BackendSettings settings;
    settings.append( Soprano::BackendSetting( Soprano::BackendOptionStorageMemory ) );

    const Soprano::Backend* backend = Soprano::discoverBackendByName( "sesame2" );
    if ( backend ) {
        return backend->createModel( settings );
    } else {
        return Soprano::createModel( settings );
    }
}

QString SemClip::serializeModelToRdfXml( const Soprano::Model* model )
{
    const Soprano::Serializer* serializer =
        Soprano::PluginManager::instance()->discoverSerializerForSerialization( Soprano::SerializationRdfXml );
    QString result;
    QTextStream stream( &result );

    if ( model && serializer->serialize( model->listStatements(), stream, Soprano::SerializationRdfXml ) ) {
        return result;
    } else {
        return QString();
    }
}

Soprano::Model* SemClip::parseModelFromRdfXml( const QString& serialization )
{
    const Soprano::Parser* parser =
        Soprano::PluginManager::instance()->discoverParserForSerialization( Soprano::SerializationRdfXml );
    Soprano::StatementIterator it = parser->parseString( serialization, QString(), Soprano::SerializationRdfXml );

    if ( parser->lastError() ) {
        return 0;
    } else {
        return createModelFromStatementIterator( it );
    }
}

Soprano::Model* SemClip::createModelFromStatementIterator( Soprano::StatementIterator& it, ConstructionMode mode )
{
    const QList< Soprano::Statement > allStatements = it.allElements();

    if ( mode == ConstructNoEmpty && allStatements.isEmpty() ) {
        return 0;
    } else {
        Soprano::Model* result = createModel();
        foreach ( Soprano::Statement statement, allStatements ) {
            result->addStatement( statement );
        }
        return result;
    }
}

QUrl SemClip::generateUniqueUri( const Soprano::Model* model )
{
    // will currently not work reliably on Redland models (no union support)
    forever {
        const QString uri = QString( SemClip::uriSchema ).append( ":/" ).append( KRandom::randomString( 16 ) );
        const QString query = QString( "ask {"
                                       "    { <%1> ?p1 ?o1 } union"
                                       "    { ?s2 <%1> ?o2 } union"
                                       "    { ?s3 ?p3 <%1> }"
                                       "}" )
                              .arg( uri );

        if ( !model->executeQuery( query, Soprano::Query::QueryLanguageSparql ).boolValue() ) {
            return QUrl( uri );
        }
    }
}

void SemClip::convertBlankNodes( Soprano::Model* model )
{
    QHash< QString, Soprano::Node > idNodeHash;

    QList< Soprano::Statement > allStatements = model->listStatements().allElements();
    foreach ( Soprano::Statement statement, allStatements ) {
        Soprano::Statement replacement = statement;
        bool modified = false;

        // process subject
        Soprano::Node node = statement.subject();
        if ( node.isBlank() ) {
            const QString identifier = node.identifier();
            if ( !idNodeHash.contains( identifier ) ) {
                // new blank node, create a resource node with unique URI for it
                idNodeHash[ identifier ] = Soprano::Node::createResourceNode( generateUniqueUri( model ) );
            }
            // replace the blank node with the resource node
            replacement.setSubject( idNodeHash[ identifier ] );
            modified = true;
        }

        // process object
        node = statement.object();
        if ( node.isBlank() ) {
            const QString identifier = node.identifier();
            if ( !idNodeHash.contains( identifier ) ) {
                // new blank node, create a resource node with unique URI for it
                idNodeHash[ identifier ] = Soprano::Node::createResourceNode( generateUniqueUri( model ) );
            }
            // replace the blank node with the resource node
            replacement.setObject( idNodeHash[ identifier ] );
            modified = true;
        }

        if ( modified ) {
            model->removeStatement( statement );
            model->addStatement( replacement );
        }
    }
}

SemClip::GraphUtility::GraphUtility( Soprano::Model* model /*= 0*/ )
    : m_model( model )
{
}

SemClip::GraphUtility::~GraphUtility()
{
}

Soprano::Model* SemClip::GraphUtility::model()
{
    return m_model;
}

void SemClip::GraphUtility::setModel( Soprano::Model* model )
{
    m_model = model;
}

void SemClip::GraphUtility::mergeStatementIterator( Soprano::StatementIterator& it )
{
    while ( it.next() ) {
        m_model->addStatement( it.current() );
    }
}

void SemClip::GraphUtility::mergeStatementIterator( Soprano::StatementIterator& it, const Soprano::Node &context )
{
    while ( it.next() ) {
        const Soprano::Statement old = it.current();
        m_model->addStatement( old.subject(), old.predicate(), old.object(), context );
    }
}

void SemClip::GraphUtility::mergeModel( const Soprano::Model* model )
{
    Soprano::StatementIterator it = model->listStatements();
    mergeStatementIterator( it );
}

QSet< QUrl > SemClip::GraphUtility::extractOntologies() const
{
    // ontology namespaces may occur on the following nodes:
    // - properties
    // - subjects or objects of rdf:type properties
    QSet< QUrl > ontologies;

    Soprano::StatementIterator it = m_model->listStatements();
    while ( it.next() ) {
        const Soprano::Node predicate = it.current().predicate();
        ontologies.insert( extractNamespace( predicate.uri() ) );

        if ( predicate.uri() == Soprano::Vocabulary::RDF::type() ) {
            // don't use the subject, this case is only relevant for ontologies
            // themselves but causes lots of trouble with regular instance data
            ontologies.insert( extractNamespace( it.current().object().uri() ) );
        }
    }

    return ontologies;
}

QSet< QUrl > SemClip::GraphUtility::identifyResources( const QSet< QUrl >& properties ) const
{
    QSet< QUrl > resources;

    Soprano::StatementIterator it = m_model->listStatements();
    while ( it.next() ) {
        const Soprano::Node object = it.current().object();
        if ( properties.contains( it.current().predicate().uri() ) && object.isResource() ) {
            resources.insert( object.uri() );
        }
    }

    return resources;
}

void SemClip::GraphUtility::applyReasoner( const Soprano::Model* ontology )
{
    // we operate on a new model, first inserting the ontology, afterwards
    // inserting our own member model statements which will trigger inference
    // (this way no superflous inferencing will be done on the ontology)
    AutoPtr< Soprano::Model > tempModel( createModel() );
    GraphUtility tempUtility( tempModel.data() );

    // add the ontology within its own context to facilitate removal afterwards
    Soprano::StatementIterator it = ontology->listStatements();
    QUrl uniqueUri = generateUniqueUri( tempModel.data() );
    const Soprano::Node ontologyContext( uniqueUri );
    tempUtility.mergeStatementIterator( it , ontologyContext );

    using namespace Soprano::Inference;
    InferenceModel inferenceModel( tempModel.data() );
    inferenceModel.setOptimizedQueriesEnabled( true );
    inferenceModel.setRules( RuleSet::standardRuleSet( RDFS ).allRules() );

    // add our model within its own context to facilitate removal afterwards
    it = m_model->listStatements();
    uniqueUri = generateUniqueUri( tempModel.data() );
    const Soprano::Node modelContext( uniqueUri );
    tempUtility.setModel( &inferenceModel );
    tempUtility.mergeStatementIterator( it, modelContext );

    // remove metadata, ontology and model context
    tempModel->removeContext( Soprano::Vocabulary::SIL::InferenceMetaData() );
    tempModel->removeContext( ontologyContext );
    tempModel->removeContext( modelContext );

    // merge the remaining inferred statements into our member model
    mergeModel( tempModel.data() );
}
