/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_GRAPHTRANSFORMER_H_
#define _SEMCLIP_GRAPHTRANSFORMER_H_

namespace Soprano
{
    class Model;
}

namespace SemClip
{

    /**
     * \class GraphTransformer graphtransformer.h "graphtransformer.h"
     *
     * \brief Abstract base class for graph transformers.
     *
     * GraphTransformer defines the base class for graph transformers, which
     * are the entities capable of actually applying a concrete mapping to a
     * RDF graph.
     *
     * The base class defines setters for the Soprano model representing the
     * graph to operate on and the transformation to apply. The latter is using
     * a generic void pointer as different concrete mapping element classes may
     * use different mapping formats. The class also declares a function to
     * execute the transformation on the model.
     *
     * \sa MappingElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class GraphTransformer
    {
    public:
        /**
         * Default destructor.
         */
        virtual ~GraphTransformer();

        /**
         * Setter for the Soprano model to apply the mapping to.
         *
         * @param model The Soprano model to operate on.
         */
        void setModel( Soprano::Model* model );

        /**
         * Setter for the transformation to apply to the graph.
         *
         * @param transformation The transformation to apply.
         */
        void setTransformation( const void* transformation );

        /**
         * Applies the transformation to the graph and returns the result. The
         * client takes over ownership of the returned new Soprano model.
         *
         * @return A new Soprano model which is the transformation result, or
         *         a null pointer if the transformation did not produce a valid
         *         graph.
         */
        virtual Soprano::Model* execute() = 0;

    protected:
        /**
         * Default constructor.
         */
        GraphTransformer();

        /**
         * Used by setTransformation to set the subclass transformation data
         * pointer. Subclasses override this to cast the void pointer to their
         * specific data format.
         *
         * @param transformation
         */
        virtual void setSpecificTransformation( const void* transformation ) = 0;

        Soprano::Model* m_graph;
    };

}

#endif
