/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "localmappingstorage.h"

#include "graphutility.h"
#include "sparqlmappingelement.h"
#include "sparqlmappingstorage.h"

#include <QtCore/QCoreApplication>
#include <QtCore/QMap>
#include <QtCore/QPair>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QUrl>

#include <KConfigGroup>
#include <kglobal.h>
#include <ksharedconfig.h>

const char SemClip::LocalMappingStorage::s_mappingsSubDirectory[]    = "mappings/";
const char SemClip::LocalMappingStorage::s_storageGroupName[]        = "Mapping Storage";
const char SemClip::LocalMappingStorage::s_mappingsGroupName[]       = "Mappings";
const char SemClip::LocalMappingStorage::s_remoteStoragesGroupName[] = "Remote Mapping Storages";

class SemClip::MappingElementFactory
{
public:
    static MappingElement* create( const QString& id, const QUrl& source, const QUrl& target );
};

SemClip::MappingElement* SemClip::MappingElementFactory::create( const QString& id, const QUrl& source, const QUrl& target )
{
    if ( id == SparqlMappingElement::classNameStatic() ) {
        return new SparqlMappingElement( source, target, false );
    } else {
        return 0;
    }
}

class SemClip::RemoteMappingStorageFactory
{
public:
    static RemoteMappingStorage* create( const QString& id, const QUrl& service );
};

SemClip::RemoteMappingStorage* SemClip::RemoteMappingStorageFactory::create( const QString& id, const QUrl& service )
{
    if ( id == SparqlMappingStorage::classNameStatic() ) {
        return new SparqlMappingStorage( service );
    } else {
        return 0;
    }
}

class SemClip::LocalMappingStorageHelper
{
public:
    LocalMappingStorage instance;
};

K_GLOBAL_STATIC( SemClip::LocalMappingStorageHelper, localMappingStorageHelper )

class SemClip::LocalMappingStorage::Private
{
public:
    typedef QPair< QUrl, QUrl > QUrlPair;

    Private();

    QMap< QUrlPair, MappingElement* >   m_mappings;
    QMap< QUrl, RemoteMappingStorage* > m_remotes;
    QString                             m_directory;
};

SemClip::LocalMappingStorage::Private::Private()
    : m_mappings( QMap< QUrlPair, MappingElement* >() ),
      m_remotes( QMap< QUrl, RemoteMappingStorage* >() ),
      m_directory( QString( SemClip::dataDirectory ).append( s_mappingsSubDirectory ) )
{
}

SemClip::LocalMappingStorage::LocalMappingStorage()
    : QObject( 0 ),
      QueryableMappingStorage(),
      ManageableMappingStorage(),
      d( new Private() )
{
    qAddPostRoutine( localMappingStorageHelper.destroy );

    // auto load on creation
    load();
}

SemClip::LocalMappingStorage::~LocalMappingStorage()
{
    // auto store on destruction
    store();
    clearMappings();
    clearRemoteStorages();
    delete d;

    qRemovePostRoutine( localMappingStorageHelper.destroy );
}

SemClip::LocalMappingStorage* SemClip::LocalMappingStorage::instance()
{
    return &localMappingStorageHelper->instance;
}

const SemClip::MappingElement* SemClip::LocalMappingStorage::findMapping( const QUrl& source, const QUrl& target )
{
    // return the mapping directly if we have it already
    const Private::QUrlPair uriPair( source, target );
    if ( d->m_mappings.contains( uriPair ) ) {
        return d->m_mappings.value( uriPair );
    }

    // if not, forward the query to the remote mapping storages
    foreach ( RemoteMappingStorage* remote, d->m_remotes ) {
        MappingElement* mapping = ManageableMappingStorage::findMapping( remote, source, target );
        if ( mapping ) {
            addMapping( mapping );
            return mapping;
        }
    }

    // mapping not found
    return 0;
}

bool SemClip::LocalMappingStorage::addMapping( MappingElement* mapping )
{
    const Private::QUrlPair uriPair( mapping->sourceUri(), mapping->targetUri() );
    if ( !d->m_mappings.contains( uriPair ) ) {
        d->m_mappings.insert( uriPair, mapping );
        mapping->setDirectory( d->m_directory );
        return true;
    } else {
        return false;
    }
}

bool SemClip::LocalMappingStorage::removeMapping( const QUrl& source, const QUrl& target )
{
    const Private::QUrlPair uriPair( source, target );
    if ( d->m_mappings.contains( uriPair ) ) {
        MappingElement* mapping = d->m_mappings.take( uriPair );
        delete mapping;
        return true;
    } else {
        return false;
    }
}

bool SemClip::LocalMappingStorage::loadMapping( const QString& key, const KConfigGroup* configGroup )
{
    const QStringList values = configGroup->readEntry( key, QStringList() );
    const QString className = values.value( 0 );
    const QUrl sourceUri = QUrl( values.value( 1 ) );
    const QUrl targetUri = QUrl( values.value( 2 ) );
    if ( className.isEmpty() || sourceUri.isEmpty() || targetUri.isEmpty() ) {
        return false;
    }

    MappingElement* element = MappingElementFactory::create( className, sourceUri, targetUri );
    if ( !element ) {
        return false;
    }

    element->setFileName( key );
    if ( addMapping( element ) ) {
        return true;
    } else {
        delete element;
        return false;
    }
}

bool SemClip::LocalMappingStorage::loadRemoteStorage( const QString& key, const KConfigGroup* configGroup )
{
    const QStringList values = configGroup->readEntry( key, QStringList() );
    const QString className = values.value( 0 );
    const QUrl service = QUrl( values.value( 1 ) );
    if ( className.isEmpty() || service.isEmpty() ) {
        return false;
    }

    RemoteMappingStorage* remote = RemoteMappingStorageFactory::create( className, service );
    if ( !remote ) {
        return false;
    }

    if ( addRemoteStorage( remote ) ) {
        return true;
    } else {
        delete remote;
        return false;
    }
}

bool SemClip::LocalMappingStorage::load()
{
    clearMappings();
    clearRemoteStorages();

    KSharedConfigPtr config = KSharedConfig::openConfig( SemClip::configFile, KConfig::NoGlobals );
    const KConfigGroup storageGroup( config, s_storageGroupName );

    bool result = true;
    const KConfigGroup mappingsGroup( &storageGroup, s_mappingsGroupName );
    foreach ( const QString key, mappingsGroup.keyList() ) {
        result = loadMapping( key, &mappingsGroup ) && result;
    }

    const KConfigGroup remoteStoragesGroup( &storageGroup, s_remoteStoragesGroupName );
    foreach ( const QString key, remoteStoragesGroup.keyList() ) {
        result = loadRemoteStorage( key, &remoteStoragesGroup ) && result;
    }

    // make sure to have at least one default remote mapping storage
    if ( d->m_remotes.isEmpty() ) {
        addRemoteStorage( new SparqlMappingStorage() );
    }

    return result;
}

bool SemClip::LocalMappingStorage::storeMapping( const MappingElement* mapping, KConfigGroup* configGroup ) const
{
    if ( mapping->saveContent() ) {
        QStringList values;
        values.append( mapping->className() );
        values.append( mapping->sourceUri().toString() );
        values.append( mapping->targetUri().toString() );

        configGroup->writeEntry( mapping->fileName(), values );
        return true;
    } else {
        return false;
    }
}

bool SemClip::LocalMappingStorage::storeRemoteStorage( const RemoteMappingStorage* remote, KConfigGroup* configGroup ) const
{
    if ( remote ) {
        QStringList values;
        values.append( remote->className() );
        values.append( remote->service().toString() );
        configGroup->writeEntry( QString::number( qHash( remote->service() ) ), values );
        return true;
    } else {
        return false;
    }
}

bool SemClip::LocalMappingStorage::store() const
{
    KSharedConfigPtr config = KSharedConfig::openConfig( SemClip::configFile, KConfig::NoGlobals );
    KConfigGroup storageGroup( config, s_storageGroupName );

    bool result = true;
    KConfigGroup mappingsGroup( &storageGroup, s_mappingsGroupName );
    mappingsGroup.deleteGroup();
    foreach ( const MappingElement* element, d->m_mappings ) {
        result = storeMapping( element, &mappingsGroup ) && result;
    }

    // FIXME: are remote mapping storages better read-only or read/write?
//     KConfigGroup remoteStoragesGroup( &storageGroup, s_remoteStoragesGroupName );
//     remoteStoragesGroup.deleteGroup();
//     foreach( const RemoteMappingStorage* remote, d->m_remotes ) {
//         result = storeRemoteStorage( remote, &remoteStoragesGroup ) && result;
//     }

    config->sync();
    return result;
}

bool SemClip::LocalMappingStorage::addRemoteStorage( RemoteMappingStorage* remote )
{
    const QUrl service = remote->service();
    if ( !d->m_remotes.contains( service ) ) {
        d->m_remotes.insert( service, remote );
        return true;
    } else {
        return false;
    }
}

void SemClip::LocalMappingStorage::clearMappings()
{
    foreach ( const Private::QUrlPair key, d->m_mappings.keys() ) {
        MappingElement* mapping = d->m_mappings.take( key );
        delete mapping;
    }
}

void SemClip::LocalMappingStorage::clearRemoteStorages()
{
    foreach ( const QUrl key, d->m_remotes.keys() ) {
        RemoteMappingStorage* remote = d->m_remotes.take( key );
        delete remote;
    }
}

#include "localmappingstorage.moc"
