/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_LOCALMAPPINGSTORAGE_H_
#define _SEMCLIP_LOCALMAPPINGSTORAGE_H_

#include "manageablemappingstorage.h"
#include "queryablemappingstorage.h"

#include <QtCore/QObject>

class QString;
class KConfigGroup;

namespace SemClip
{

    class LocalMappingStorageHelper;
    class MappingElementFactory;
    class RemoteMappingStorage;
    class RemoteMappingStorageFactory;

    /**
     * \class LocalMappingStorage localmappingstorage.h "localmappingstorage.h"
     *
     * \brief The local mapping storage.
     *
     * LocalMappingStorage inherits from both QueryableMappingStorage and
     * ManageableMappingStorage. It is a singleton and central access point
     * for clients looking for mappings. It holds a collection of mapping
     * elements locally and also holds a list of remote mapping storages it
     * can query if a mapping is not yet available locally.
     *
     * \sa QueryableMappingStorage, ManageableMappingStorage,
     *     RemoteMappingStorage, MappingElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class LocalMappingStorage : public QObject,
                                public QueryableMappingStorage,
                                public ManageableMappingStorage
    {
        Q_OBJECT

    public:
        /**
         * @return The single unique instance of this class.
         */
        static LocalMappingStorage* instance();

        /**
         * Default destructor.
         */
        virtual ~LocalMappingStorage();

        /**
         * Attempts to retrieve a mapping element from the storage that defines
         * a translation from the given source ontology to the given target
         * ontology. Ownership of the mapping element remains with the storage.
         *
         * @param source The URI of the ontology to translate from.
         * @param target The URI of the ontology to translate to.
         * @return A mapping element if available, null otherwise.
         *
         * \sa MappingElement
         */
        virtual const MappingElement* findMapping( const QUrl& source, const QUrl& target );

        /**
         * Adds a mapping element to the storage which takes over ownership.
         *
         * @param mapping The mapping element to add.
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool addMapping( MappingElement* mapping );

        /**
         * Removes a mapping element from the storage and deletes it.
         *
         * @param source The source URI of the mapping element to remove.
         * @param target The target URI of the mapping element to remove.
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool removeMapping( const QUrl& source, const QUrl& target );

        /**
         * Loads the mapping storage state from disk.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool load();

        /**
         * Saves the mapping storage state to disk.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool store() const;

    private:
        static const char s_mappingsSubDirectory[];
        static const char s_storageGroupName[];
        static const char s_mappingsGroupName[];
        static const char s_remoteStoragesGroupName[];

        friend class LocalMappingStorageHelper;

        /**
         * Private default constructor, object is a singleton.
         */
        LocalMappingStorage();

        /**
         * Loads a mapping element entry from the config file.
         *
         * @param key The key of the config entry (equals mapping file name).
         * @param configGroup Which config group to read the entry from.
         * @return True if the operation was successful, false otherwise.
         */
        bool loadMapping( const QString& key, const KConfigGroup* configGroup );

        /**
         * Stores a mapping element entry to the config file. This will only
         * be done if the element content could successfully be saved to the
         * file system as well.
         *
         * @param mapping The mapping element to store.
         * @param configGroup Which config group to write the entry to.
         * @return True if the operation was successful, false otherwise.
         */
        bool storeMapping( const MappingElement* mapping, KConfigGroup* configGroup ) const;

        /**
         * Loads a remote mapping storage entry from the config file.
         *
         * @param key The key of the config entry (equals remote service URL).
         * @param configGroup Which config group to read the entry from.
         * @return True if the operation was successful, false otherwise.
         */
        bool loadRemoteStorage( const QString& key, const KConfigGroup* configGroup );

        /**
         * Stores a remote mapping storage entry to the config file.
         *
         * @param mapping The remote mapping storage to store.
         * @param configGroup Which config group to write the entry to.
         * @return True if the operation was successful, false otherwise.
         */
        bool storeRemoteStorage( const RemoteMappingStorage* remote, KConfigGroup* configGroup ) const;

        /**
         * Adds a remote mapping storage to the storage collection.
         *
         * @param storage The remote mapping storage to add.
         * @return True if the operation was successful, false otherwise.
         */
        bool addRemoteStorage( RemoteMappingStorage* remote );

        /**
         * Deletes and removes all elements from the mapping collection.
         */
        void clearMappings();

        /**
         * Deletes and removes all remote mapping storages from the collection.
         */
        void clearRemoteStorages();

        class Private;
        Private* const d;
    };

}

#endif
