/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_MAPPINGELEMENT_H_
#define _SEMCLIP_MAPPINGELEMENT_H_

#include "uncopyable.h"

#include <QtCore/QString>
#include <QtCore/QUrl>

namespace SemClip
{

    class GraphTransformer;

    /**
     * \class MappingElement mappingelement.h "mappingelement.h"
     *
     * \brief Abstract base class for mapping elements.
     *
     * MappingElement defines the base class for mapping elements, which can
     * be managed by a mapping storage and will be used by GraphTranslator.
     *
     * The base class defines functions for accessing the source and target
     * ontology URI, and declares functions for accessing the actual mapping
     * content as well as for file system based persistence management.
     *
     * Because the mapping formats may differ between subclasses, the content
     * getter uses a generic void pointer, and a factory method that creates a
     * GraphTransformer capable of actually applying a specific mapping is also
     * supplied.
     *
     * \sa GraphTransformer
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class MappingElement : private Uncopyable
    {
    public:
        /**
         * Default destructor.
         */
        virtual ~MappingElement();

        /**
         * Getter for the source ontology URI.
         *
         * @return The source ontology URI.
         */
        QUrl sourceUri() const;

        /**
         * Getter for the target ontology URI.
         *
         * @return The target ontology URI.
         */
        QUrl targetUri() const;

        /**
         * Getter for the content file name.
         *
         * @return The file name under which the mapping content is stored.
         */
        QString fileName() const;

        /**
         * Setter for the content file name.
         *
         * @param fileName The file name to store the mapping content to.
         */
        void setFileName( const QString& fileName );

        /**
         * Getter for the content directory.
         *
         * @return The directory where the mapping content is stored.
         */
        QString directory() const;

        /**
         * Setter for the content directory.
         *
         * @param directory The directory to store the mapping content in.
         */
        void setDirectory( const QString& directory );

        /**
         * Getter for the mapping content. Uses lazy initialization.
         *
         * @return A generic pointer to the concrete subclass content.
         */
        const void* content() const;

        /**
         * Factory method that returns a GraphTransformer which is able to
         * actually apply a concrete mapping to a RDF graph. The client is
         * responsible for deleting the GraphTransformer after usage.
         *
         * @return A new GraphTransformer corresponding to the mapping element.
         *
         * \sa GraphTransformer
         */
        virtual GraphTransformer* createTransformer() const = 0;

        /**
         * Loads the mapping content from the file system. Subclasses have
         * to implement loadContentInternal instead of overriding this.
         *
         * @return True if the operation was successful, false otherwise.
         */
        bool loadContent();

        /**
         * Saves the mapping content to the file system.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool saveContent() const = 0;

        /**
         * Erases the mapping content from the file system.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool eraseContent() const = 0;

        /**
         * Returns the name of a conrete subclass.
         *
         * @return The class name.
         */
        virtual QString className() const = 0;

    protected:
        /**
         * Custom contructor, no default contructor is supplied.
         *
         * @param source The URI of the source ontology for the mapping.
         * @param target The URI of the target ontology for the mapping.
         * @param isNew Defines whether the element truly is a new one or just
                        recreated again and to be loaded from persistence.
         */
        MappingElement( const QUrl& source, const QUrl& target, bool isNew = true );

        /**
         * Used by content() to get the actual content data pointer. Subclasses
         * override this to cast their own content format to a void pointer.
         *
         * @return The mapping content cast to a void pointer.
         */
        virtual const void* genericContent() const = 0;

        /**
         * Loads the mapping content from the file system. This const version
         * is used internally, to ensure that only mutable data members are
         * modified.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool loadContentInternal() const = 0;

        const QUrl   m_source;
        const QUrl   m_target;
        QString      m_fileName;
        QString      m_directory;
        mutable bool m_needsInitialization;
    };

}

#endif
