/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_REMOTEMAPPINGSTORAGE_H_
#define _SEMCLIP_REMOTEMAPPINGSTORAGE_H_

#include "queryablemappingstorage.h"

#include <QtCore/QUrl>

class QString;

namespace SemClip
{

    class ManageableMappingStorage;

    /**
     * \class RemoteMappingStorage remotemappingstorage.h "remotemappingstorage.h"
     *
     * \brief Abstract base class for accessing a remote mapping storage.
     *
     * RemoteMappingStorage is the base class for adapters to existing
     * remote mapping storages and inherits from QueryableMappingStorage.
     * It encapsulates the remote communication and hides potential mapping
     * element format differences.
     *
     * In addition to findMapping, a similar function findMappingInternal is
     * offered which can be used by other mapping storages and allows those to
     * take over ownership of the found mapping, while the regular findMapping
     * leaves ownership with the queried storage.
     *
     * The actual remote access and creation of a suitable mapping element is
     * deferred to retrieveMapping, which child classes have to implement.
     *
     * \sa QueryableMappingStorage, MappingElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class RemoteMappingStorage : public QueryableMappingStorage
    {
    public:
        /**
         * Default destructor.
         */
        virtual ~RemoteMappingStorage();

        /**
         * Getter for the remote service URL.
         *
         * @return The URL where the remote mapping service can be reached.
         */
        QUrl service() const;

        /**
         * Attempts to retrieve a mapping element from the storage that defines
         * a translation from the given source ontology to the given target
         * ontology. Ownership of the mapping element remains with the storage.
         *
         * Subclasses only need to implement retrieveMapping, there is no need
         * to override this function.
         *
         * @param source The URI of the ontology to translate from.
         * @param target The URI of the ontology to translate to.
         * @return A mapping element if available, null otherwise.
         *
         * \sa MappingElement
         */
        virtual const MappingElement* findMapping( const QUrl& source, const QUrl& target );

        /**
         * Returns the name of a conrete subclass.
         *
         * @return The class name.
         */
        virtual QString className() const = 0;

    protected:
        /**
         * Custom constructor, no default constructor is supplied.
         *
         * @param service An URL to the remote mapping storage service.
         */
        explicit RemoteMappingStorage( const QUrl& service );

        /**
         * Like the public findMapping, but used internally by other mapping
         * storages to gain full access to a new found mapping element. As a
         * consequence, the client takes over ownership.
         *
         * \sa findMapping
         */
        MappingElement* findMappingInternal( const QUrl& source, const QUrl& target );

        friend class ManageableMappingStorage;

        /**
         * Attempts to retrieve a mapping from the remote service. Subclasses
         * implement the actual communication with their remote mapping service
         * in this function.
         *
         * @param source The URI of the ontology to translate from.
         * @param target The URI of the ontology to translate to.
         * @return A mapping element if available, null otherwise.
         */
        virtual MappingElement* retrieveMapping( const QUrl& source, const QUrl& target ) = 0;

        const QUrl              m_service;
        mutable MappingElement* m_mapping;
    };

}

#endif
