/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "sparqlmappingelement.h"

#include "sparqlgraphtransformer.h"

#include <QtCore/QFileInfo>

#include <kstandarddirs.h>
#include <kzip.h>

const char SemClip::SparqlMappingElement::s_prefixFile[]      = "prefixes";
const char SemClip::SparqlMappingElement::s_queryFile[]       = "queries";
const char SemClip::SparqlMappingElement::s_lineSeperator[]   = "\n";
const char SemClip::SparqlMappingElement::s_prefixSeperator[] = ":=";
const char SemClip::SparqlMappingElement::s_querySeperator[]  = "###QUERY-SEPERATOR###\n";

SemClip::SparqlMappingElement::SparqlMappingElement( const QUrl& source, const QUrl& target, bool isNew /*= true*/ )
    : MappingElement( source, target, isNew ),
      m_content( 0 ),
      m_lastModification( QDateTime() )
{
}

SemClip::SparqlMappingElement::~SparqlMappingElement()
{
    delete m_content;
}

void SemClip::SparqlMappingElement::setContent( Content* content )
{
    m_needsInitialization = false;
    if ( content != m_content ) {
        delete m_content;
        m_content = content;
        m_lastModification = QDateTime::currentDateTime();
    }
}

SemClip::GraphTransformer* SemClip::SparqlMappingElement::createTransformer() const
{
    return new SemClip::SparqlGraphTransformer();
}

bool SemClip::SparqlMappingElement::loadContentInternal() const
{
    delete m_content;
    m_content = 0;
    m_lastModification = QDateTime();

    if ( m_fileName.isEmpty() ) {
        return false;
    }

    const QString filePath = KStandardDirs::locate( "data", m_directory + m_fileName );
    const QFileInfo fileInfo( filePath );
    if ( filePath.isEmpty() || !fileInfo.exists() ) {
        return false;
    }

    m_content = new Content();
    m_lastModification = fileInfo.lastModified();

    KZip input( filePath );
    input.open( QIODevice::ReadOnly );

    bool result = contentFromZip( &input, m_content );
    if ( result ) {
        m_needsInitialization = false;
    }
    return result;
}

bool SemClip::SparqlMappingElement::saveContent() const
{
    if ( m_needsInitialization ) {
        return true;
    }

    // only save if we have content and a file name...
    if ( !m_content || m_fileName.isEmpty() ) {
        return false;
    }

    const QString filePath = KStandardDirs::locateLocal( "data", m_directory + m_fileName, true );
    const QFileInfo fileInfo( filePath );
    const QDateTime storageDate = fileInfo.lastModified();

    // ...and the mapping isn't on disk yet or outdated there
    if ( fileInfo.exists() && storageDate >= m_lastModification ) {
        return true;
    }

    if ( fileInfo.exists() && !eraseContent() ) {
        return false;
    }

    KZip output( filePath );
    output.open( QIODevice::WriteOnly );
    return contentToZip( &output, m_content );
}

bool SemClip::SparqlMappingElement::eraseContent() const
{
    const QString filePath = KStandardDirs::locateLocal( "data", m_directory + m_fileName );
    QFile file( filePath );
    return file.remove();
}

QString SemClip::SparqlMappingElement::className() const
{
    return classNameStatic();
}

QString SemClip::SparqlMappingElement::classNameStatic()
{
    return QString( "SemClip::SparqlMappingElement" );
}

const void* SemClip::SparqlMappingElement::genericContent() const
{
    return static_cast< const void* >( m_content );
}

QString SemClip::SparqlMappingElement::generateFileName( const QUrl& source, const QUrl& target, const Content* content )
{
    if ( !content ) {
        return QString();
    }

    const QString sourcePrefix = content->prefixes.key( source );
    const QString targetPrefix = content->prefixes.key( target );

    QString result;
    if ( !sourcePrefix.isEmpty() && !targetPrefix.isEmpty() ) {
        result = sourcePrefix + "2" + targetPrefix + ".zip";
    }

    return result;
}

bool SemClip::SparqlMappingElement::contentFromZip( const KZip* zip, Content* content )
{
    content->prefixes = readPrefixes( zip );
    content->queries = readQueries( zip );
    return ( !content->prefixes.isEmpty() && !content->queries.isEmpty() );
}

bool SemClip::SparqlMappingElement::contentToZip( KZip* zip, const Content* content )
{
    bool result = writePrefixes( zip, content->prefixes );
    result = writeQueries( zip, content->queries ) && result;
    return result;
}

SemClip::SparqlMappingElement::Content::PrefixMap SemClip::SparqlMappingElement::readPrefixes( const KZip* zip )
{
    Content::PrefixMap result;
    const KArchiveEntry* entry = zip->directory()->entry( s_prefixFile );

    if ( entry && entry->isFile() ) {
        const KArchiveFile* file = static_cast< const KArchiveFile* >( entry );
        const QStringList lines = QString::fromUtf8( file->data() ).split( s_lineSeperator, QString::SkipEmptyParts );

        foreach ( const QString line, lines ) {
            const QStringList parts = line.split( s_prefixSeperator );
            const QString lhs = parts.value( 0 );
            const QString rhs = parts.value( 1 );

            if ( !lhs.isEmpty() && !rhs.isEmpty() ) {
                result.insert( lhs, QUrl( rhs ) );
            }
        }
    }

    return result;
}

QStringList SemClip::SparqlMappingElement::readQueries( const KZip* zip )
{
    QStringList result;
    const KArchiveEntry* entry = zip->directory()->entry( s_queryFile );

    if ( entry && entry->isFile() ) {
        const KArchiveFile* file = static_cast< const KArchiveFile* >( entry );
        result = QString::fromUtf8( file->data() ).split( s_querySeperator, QString::SkipEmptyParts );
    }

    return result;
}

bool SemClip::SparqlMappingElement::writePrefixes( KZip* zip, const Content::PrefixMap& prefixes )
{
    QByteArray data;
    foreach ( const QString key, prefixes.keys() ) {
        data.append( key.toUtf8() );
        data.append( s_prefixSeperator );
        data.append( prefixes.value( key ).toString().toUtf8() );
        data.append( s_lineSeperator );
    }

    return zip->writeFile( s_prefixFile, QString(), QString(), data.constData(), data.size() );
}

bool SemClip::SparqlMappingElement::writeQueries( KZip* zip, const QStringList& queries )
{
    QByteArray data;
    foreach ( const QString query, queries ) {
        data.append( query.toUtf8() );
        if ( !query.endsWith( s_lineSeperator ) ) {
            data.append( s_lineSeperator );
        }
        data.append( s_querySeperator );
    }

    return zip->writeFile( s_queryFile, QString(), QString(), data.constData(), data.size() );
}
