/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_SPARQLMAPPINGELEMENT_H_
#define _SEMCLIP_SPARQLMAPPINGELEMENT_H_

#include "mappingelement.h"

#include <QtCore/QDateTime>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QUrl>

class KZip;

namespace SemClip
{

    class GraphTransformer;

    /**
     * \class SparqlMappingElement sparqlmappingelement.h "sparqlmappingelement.h"
     *
     * \brief Mapping element using SPARQL construct queries.
     *
     * SparqlMappingElement inherits from MappingElement and encapsulates
     * mappings defined by SPARQL construct queries as created by the RDF
     * Transformer prototype from Matthias Hert (Department of Informatics,
     * University of Zurich).
     *
     * \sa MappingElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class SparqlMappingElement : public MappingElement
    {
    public:
        /**
         * Helper struct wrapping the different content elements.
         */
        struct Content
        {
            typedef QMap< QString, QUrl > PrefixMap;

            PrefixMap prefixes;
            QStringList queries;
        };

        /**
         * Custom contructor, no default contructor is supplied.
         *
         * @param source The URI of the source ontology for the mapping.
         * @param target The URI of the target ontology for the mapping.
         * @param isNew Defines whether the element truly is a new one or just
                        recreated again and to be loaded from persistence.
         */
        SparqlMappingElement( const QUrl& source, const QUrl& target, bool isNew = true );

        /**
         * Default destructor.
         */
        virtual ~SparqlMappingElement();

        /**
         * Setter for the mapping content. The mapping element takes over
         * ownership of the new content. Any old content is deleted first.
         *
         * @param content The new mapping content.
         */
        void setContent( Content* content );

        /**
         * Factory method that returns a new SparqlGraphTranslator.
         *
         * @return A new SparqlGraphTransformer.
         *
         * \sa GraphTransformer
         */
        virtual GraphTransformer* createTransformer() const;

        /**
         * Saves the mapping content to the file system.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool saveContent() const;

        /**
         * Erases the mapping content from the file system.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool eraseContent() const;

        /**
         * Returns the name of the class.
         *
         * @return The class name.
         */
        virtual QString className() const;

        /**
         * Returns the name of the class.
         *
         * @return The class name.
         */
        static QString classNameStatic();

        /**
         * Generates a file name from the source URI, target URI and the
         * supplied content of the form 'sourcePrefix2targetPrefix.zip'.
         *
         * @param source The URI of the source ontology for the mapping.
         * @param target The URI of the target ontology for the mapping.
         * @param content The content to look up the prefixes from.
         * @return A generated file name, or an empty string on failure.
         */
        static QString generateFileName( const QUrl& source, const QUrl& target, const Content* content );

        /**
         * Reads a Content struct from an open zip file.
         *
         * @param zip The zip file to read from.
         * @param content The Content struct to write to.
         * @return True if the operation was successful, false otherwise.
         */
        static bool contentFromZip( const KZip* zip, Content* content );

        /**
         * Writes a Content struct to an open zip file.
         *
         * @param zip The zip file to write to.
         * @param content The Content struct to read from.
         * @return True if the operation was successful, false otherwise.
         */
        static bool contentToZip( KZip* zip, const Content* content );

    protected:
        /**
         * Casts the interally used content to a generic void pointer.
         *
         * @return The mapping content cast to a void pointer.
         */
        virtual const void* genericContent() const;

        /**
         * Loads the mapping content from the file system. This const version
         * is used internally, to ensure that only mutable data members are
         * modified.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool loadContentInternal() const;

    private:
        static const char s_prefixFile[];
        static const char s_queryFile[];
        static const char s_lineSeperator[];
        static const char s_prefixSeperator[];
        static const char s_querySeperator[];

        /**
         * Reads a prefix map from the open zip file.
         *
         * @param zip The zip file to read from.
         * @return The prefix map read, or an empty map on failure.
         */
        static Content::PrefixMap readPrefixes( const KZip* zip );

        /**
         * Reads a query list from the open zip file.
         *
         * @param zip The zip file to read from.
         * @return The query list read, or an empty list on failure.
         */
        static QStringList readQueries( const KZip* zip );

        /**
         * Writes a prefix map to the open zip file.
         *
         * @param zip The zip file to write to.
         * @param prefixes The prefix map to write.
         * @return True if the operation was successful, false otherwise.
         */
        static bool writePrefixes( KZip* zip, const Content::PrefixMap& prefixes );

        /**
         * Writes a query list to the open zip file.
         *
         * @param zip The zip file to write to.
         * @param queries The query list to write.
         * @return True if the operation was successful, false otherwise.
         */
        static bool writeQueries( KZip* zip, const QStringList& queries );

        mutable Content*  m_content;
        mutable QDateTime m_lastModification;
    };

}

#endif
