/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "sparqlmappingstorage.h"

#include "autoptr.h"
#include "sparqlmappingelement.h"
#include "xmlrpcwrapper.h"

#include <QtCore/QBuffer>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QVariant>

#include <kzip.h>

const char SemClip::SparqlMappingStorage::s_sealMappingLib[]   = "http://seal.ifi.uzh.ch/mappinglib";
const char SemClip::SparqlMappingStorage::s_getMappingMethod[] = "RemoteMappingStorage.getMapping";

SemClip::SparqlMappingStorage::SparqlMappingStorage( const QUrl& service )
    : RemoteMappingStorage( service )
{
}

SemClip::SparqlMappingStorage::SparqlMappingStorage()
    : RemoteMappingStorage( QUrl( s_sealMappingLib ) )
{
}

SemClip::SparqlMappingStorage::~SparqlMappingStorage()
{
}

QString SemClip::SparqlMappingStorage::className() const
{
    return classNameStatic();
}

QString SemClip::SparqlMappingStorage::classNameStatic()
{
    return QString( "SemClip::SparqlMappingStorage" );
}

SemClip::MappingElement* SemClip::SparqlMappingStorage::retrieveMapping( const QUrl& source, const QUrl& target )
{
    // prepare XML-RPC call and execute it
    QList< QVariant > parameters;
    parameters.append( source.toString() );
    parameters.append( target.toString() );
    AutoPtr< XmlRpcWrapper > xmlRpc( new XmlRpcWrapper( m_service ) );
    xmlRpc->call( s_getMappingMethod, parameters );

    // abort if we have no result data
    if ( !xmlRpc->hasData() ) {
        return 0;
    }

    // abort if the received data is empty
    QByteArray zipData = xmlRpc->data().first().toByteArray();
    if ( zipData.isEmpty() ) {
        return 0;
    }

    // abort if we are unable to read a mapping content from the remote result
    AutoPtr< SparqlMappingElement::Content > content( new SparqlMappingElement::Content() );
    QBuffer zipBuffer( &zipData );
    KZip zip( &zipBuffer );
    if ( !zip.open( QIODevice::ReadOnly ) || !SparqlMappingElement::contentFromZip( &zip, content.data() ) ) {
        return 0;
    }

    // abort if generating a file name failed (prefix map invalid/incomplete)
    const QString fileName = SparqlMappingElement::generateFileName( source, target, content.data() );
    if ( fileName.isEmpty() ) {
        return 0;
    }

    // create a new mapping element, set its members and return it
    SparqlMappingElement* result = new SparqlMappingElement( source, target, true );
    result->setFileName( fileName );
    result->setContent( content.data() );
    content.detach();
    return result;
}
