/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "nepomukontologyelement.h"

#include "graphutility.h"
#include "ontologymanagermodel.h"

#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/XMLSchema>

#include <Nepomuk/ResourceManager>

SemClip::NepomukOntologyElement::NepomukOntologyElement( const QUrl& uri, bool isNew /*= true*/ )
    : OntologyElement( uri, isNew ),
      m_lastModification( QDateTime() )
{
}

SemClip::NepomukOntologyElement::~NepomukOntologyElement()
{
}

void SemClip::NepomukOntologyElement::setContent( Soprano::Model* content )
{
    if ( content != m_content ) {
        m_lastModification = QDateTime::currentDateTime();
    }
    OntologyElement::setContent( content );
}

bool SemClip::NepomukOntologyElement::loadContentInternal() const
{
    delete m_content;
    m_content = 0;
    m_lastModification = QDateTime();

    Soprano::Model* main = Nepomuk::ResourceManager::instance()->mainModel();
    const QString query = QString( "construct { ?s ?p ?o }"
                                   "where {"
                                   "    graph ?g { ?s ?p ?o } ."
                                   "    ?g <%1> \"%2\"^^<%3> ."
                                   "}" )
                          .arg( Soprano::Vocabulary::NAO::hasDefaultNamespace().toString() )
                          .arg( QString::fromUtf8( m_uri.toEncoded() ) )
                          .arg( Soprano::Vocabulary::XMLSchema::string().toString() );

    Soprano::QueryResultIterator qit = main->executeQuery( query, Soprano::Query::QueryLanguageSparql );
    if ( !qit.isValid() ) {
        return false;
    }

    Soprano::StatementIterator sit = qit.iterateStatements();
    m_content = SemClip::createModelFromStatementIterator( sit, SemClip::ConstructNoEmpty );

    Nepomuk::OntologyManagerModel ontologyManager( main );
    m_lastModification = ontologyManager.ontoModificationDate( m_uri );

    bool result = ( m_content != 0 && m_lastModification.isValid() );
    if ( result ) {
        m_needsInitialization = false;
    }
    return result;
}

bool SemClip::NepomukOntologyElement::saveContent() const
{
    if ( m_needsInitialization ) {
        return true;
    }

    // only save if we have content...
    if ( !m_content ) {
        return false;
    }

    Nepomuk::OntologyManagerModel ontologyManager( Nepomuk::ResourceManager::instance()->mainModel() );
    const QDateTime storageDate = ontologyManager.ontoModificationDate( m_uri );

    // ...and the ontology isn't in storage yet or outdated there
    if ( !storageDate.isNull() && storageDate >= m_lastModification ) {
        return true;
    }

    return ontologyManager.updateOntology( m_content->listStatements(), m_uri );
}

bool SemClip::NepomukOntologyElement::eraseContent() const
{
    Nepomuk::OntologyManagerModel ontologyManager( Nepomuk::ResourceManager::instance()->mainModel() );
    return ontologyManager.removeOntology( m_uri );
}
