/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_NEPOMUKONTOLOGYELEMENT_H_
#define _SEMCLIP_NEPOMUKONTOLOGYELEMENT_H_

#include "ontologyelement.h"

#include <QtCore/QDateTime>

namespace SemClip
{

    /**
     * \class NepomukOntologyElement nepomukontologyelement.h "nepomukontologyelement.h"
     *
     * \brief Ontology element using the Nepomuk main storage for persistence.
     *
     * NepomukOntologyElement inherits from the OntologyElement base class, and
     * uses the Nepomuk main RDF storage for ontology persistence management.
     *
     * \sa OntologyElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class NepomukOntologyElement : public OntologyElement
    {
    public:
        /**
         * Custom contructor, no default contructor is supplied.
         *
         * @param uri The ontology URI.
         * @param isNew Defines whether the element truly is a new one or just
                        recreated again and to be loaded from persistence.
         */
        explicit NepomukOntologyElement( const QUrl& uri, bool isNew = true  );

        /**
         * Default destructor.
         */
        virtual ~NepomukOntologyElement();

        /**
         * Setter for the ontology content. The ontology element takes over
         * ownership of the supplied Soprano model. Also updates the internal
         * modification timestamp.
         *
         * @param content The new ontology content in form of a Soprano model.
         */
        virtual void setContent( Soprano::Model* content );

        /**
         * Saves the ontology content to the Nepomuk main RDF storage.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool saveContent() const;

        /**
         * Erases the ontology content from the Nepomuk main RDF storage.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool eraseContent() const;

    protected:
        /**
         * Loads the ontology content from the Nepomuk main RDF storage. This
         * const version is used internally, to ensure that only mutable data
         * members are modified.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool loadContentInternal() const;

    private:
        mutable QDateTime m_lastModification;
    };

}

#endif
