/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "nepomukontologystorage.h"

#include "autoptr.h"
#include "graphretriever.h"
#include "graphutility.h"
#include "nepomukontologyelement.h"

#include <QtCore/QCoreApplication>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QUrl>

#include <Soprano/Node>
#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/Vocabulary/NAO>

#include <kglobal.h>

#include <Nepomuk/ResourceManager>

class SemClip::NepomukOntologyStorageHelper
{
public:
    NepomukOntologyStorage instance;
};

K_GLOBAL_STATIC( SemClip::NepomukOntologyStorageHelper, nepomukOntologyStorageHelper )

class SemClip::NepomukOntologyStorage::Private
{
public:
    Private();

    QMap< QUrl, NepomukOntologyElement* > m_ontologies;
};

SemClip::NepomukOntologyStorage::Private::Private()
    : m_ontologies( QMap< QUrl, NepomukOntologyElement* >() )
{
}

SemClip::NepomukOntologyStorage::NepomukOntologyStorage()
    : QObject( 0 ),
      OntologyStorage(),
      d( new Private() )
{
    qAddPostRoutine( nepomukOntologyStorageHelper.destroy );

    // auto load on creation
    load();
}

SemClip::NepomukOntologyStorage::~NepomukOntologyStorage()
{
    // auto store on destruction
    store();
    clearOntologies();
    delete d;

    qRemovePostRoutine( nepomukOntologyStorageHelper.destroy );
}

SemClip::NepomukOntologyStorage* SemClip::NepomukOntologyStorage::instance()
{
    return &nepomukOntologyStorageHelper->instance;
}

const SemClip::OntologyElement* SemClip::NepomukOntologyStorage::findOntology( const QUrl& uri )
{
    // return the ontology directly if we have it already
    if ( d->m_ontologies.contains( uri ) ) {
        return d->m_ontologies.value( uri );
    }

    // if not, refresh and look again (main model might have been modified)
    if ( refresh() && d->m_ontologies.contains( uri ) ) {
        return d->m_ontologies.value( uri );
    }

    // as a last resort, attempt to retrieve the ontology from the Web
    GraphRetriever retriever;
    retriever.get( uri );
    AutoPtr< Soprano::Model > model( retriever.model() );
    if ( !model->isEmpty() && addOntologyInternal( uri, model.data(), true ) ) {
        model.detach();
        return d->m_ontologies.value( uri );
    }

    // ontology not found
    return 0;
}

bool SemClip::NepomukOntologyStorage::addOntologyInternal( const QUrl& uri, Soprano::Model* model /*= 0*/, bool isNew /*= false*/ )
{
    if ( d->m_ontologies.contains( uri ) ) {
        return false;
    }

    NepomukOntologyElement* element = new NepomukOntologyElement( uri, isNew );
    if ( isNew ) {
        element->setContent( model );
    }

    d->m_ontologies.insert( uri, element );
    return true;
}

bool SemClip::NepomukOntologyStorage::addOntology( const QUrl& uri, Soprano::Model* model )
{
    refresh();
    return addOntologyInternal( uri, model, true );
}

bool SemClip::NepomukOntologyStorage::addOntology( const QUrl& uri, const QString& serialization )
{
    AutoPtr< Soprano::Model > model( SemClip::parseModelFromRdfXml( serialization ) );
    bool result = addOntology( uri, model.data() );
    if ( result ) {
        model.detach();
    }
    return result;
}

bool SemClip::NepomukOntologyStorage::removeOntology( const QUrl& uri, RemovalMode mode )
{
    refresh();
    if ( !d->m_ontologies.contains( uri ) ) {
        return false;
    }

    bool result = true;
    NepomukOntologyElement* ontology = d->m_ontologies.take( uri );
    if ( mode == RemoveAndErase ) {
        result = result && ontology->eraseContent();
    }

    delete ontology;
    return result;
}

bool SemClip::NepomukOntologyStorage::load()
{
    clearOntologies();
    const QSet< QUrl > ontologyUris = retrieveOntologyUris();

    bool result = true;
    foreach ( const QUrl uri, ontologyUris ) {
        result = addOntologyInternal( uri, 0, false ) && result;
    }

    return result;
}

bool SemClip::NepomukOntologyStorage::store() const
{
    bool result = true;
    foreach ( const NepomukOntologyElement* element, d->m_ontologies ) {
        result = element->saveContent() && result;
    }

    return result;
}

bool SemClip::NepomukOntologyStorage::refresh()
{
    const QSet< QUrl > ontologyUris = retrieveOntologyUris();

    bool result = false;
    foreach ( const QUrl uri, ontologyUris ) {
        if ( !d->m_ontologies.contains( uri ) ) {
            result = addOntologyInternal( uri, 0, false ) || result;
        }
    }

    return result;
}

void SemClip::NepomukOntologyStorage::clearOntologies()
{
    foreach ( const QUrl key, d->m_ontologies.keys() ) {
        NepomukOntologyElement* ontology = d->m_ontologies.take( key );
        delete ontology;
    }
}

QSet< QUrl > SemClip::NepomukOntologyStorage::retrieveOntologyUris()
{
    QSet< QUrl > result;

    const Soprano::Model* main = Nepomuk::ResourceManager::instance()->mainModel();
    const QString query = QString( "select ?ns "
                                   "where {"
                                   "    ?g <%1> ?ns"
                                   "}" )
                          .arg( Soprano::Vocabulary::NAO::hasDefaultNamespace().toString() );

    Soprano::QueryResultIterator it = main->executeQuery( query, Soprano::Query::QueryLanguageSparql );
    while ( it.next() ) {
        const Soprano::Node node = it.binding( "ns" );
        if ( node.isLiteral() && node.literal().isString() ) {
            result.insert( QUrl( node.literal().toString() ) );
        }
    }

    return result;
}

#include "nepomukontologystorage.moc"
