/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_NEPOMUKONTOLOGYSTORAGE_H_
#define _SEMCLIP_NEPOMUKONTOLOGYSTORAGE_H_

#include "ontologystorage.h"

#include <QtCore/QObject>
#include <QtCore/QSet>

class QUrl;

namespace SemClip
{

    class NepomukOntologyStorageHelper;

    /**
     * \class NepomukOntologyStorage nepomukontologystorage.h "nepomukontologystorage.h"
     *
     * \brief An ontology storage using the Nepomuk main storage.
     *
     * NepomukOntologyStorage inherits from OntologyStorage and is a singleton.
     * It stores ontology elements of the kind NepomukOntologyElement and uses
     * the Nepomuk main RDF storage for ontology persistence.
     *
     * \sa OntologyStorage, NepomukOntologyElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class NepomukOntologyStorage : public QObject,
                                   public OntologyStorage
    {
        Q_OBJECT

    public:
        /**
         * @return The single unique instance of this class.
         */
        static NepomukOntologyStorage* instance();

        /**
         * Default destructor.
         */
        virtual ~NepomukOntologyStorage();

        /**
         * Attempts to retrieve an ontology element from the storage. If the
         * requested ontology is not yet known, a first attempt is made to load
         * it from the Nepomuk main model. If that fails, a second attempt is
         * made to retrieve the ontology directly from the Web. Ownership of
         * the ontology element remains with the storage.
         *
         * @param uri The URI of the desired ontology.
         * @return An ontology element if it is available, null otherwise.
         *
         * \sa OntologyElement
         */
        virtual const OntologyElement* findOntology( const QUrl& uri );

        /**
         * Adds an ontology to the storage.
         *
         * @param uri The URI of the ontology to add.
         * @param model Soprano Model defining the ontology.
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool addOntology( const QUrl& uri, Soprano::Model* model );

        /**
         * Adds an ontology to the storage.
         *
         * @param uri The URI of the ontology to add.
         * @param serialization A RDF/XML serialization defining the ontology.
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool addOntology( const QUrl& uri, const QString& serialization );

        /**
         * Removes an ontology from the storage.
         *
         * @param uri The URI of the ontology to remove.
         * @param mode Specifies what to do with the persistent content.
         * @return True if the operation was successful, false otherwise.
         *
         * \sa RemovalMode
         */
        virtual bool removeOntology( const QUrl& uri, RemovalMode mode );

        /**
         * Loads the ontology storage from the Nepomuk main model.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool load();

        /**
         * Saves the ontology storage to the Nepomuk main model.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool store() const;

    private:
        friend class NepomukOntologyStorageHelper;

        /**
         * Private default constructor, object is a singleton.
         */
        NepomukOntologyStorage();

        /**
         * Adds an ontology to the storage. Used internally without a refresh
         * while the external addOntology functions refresh before delegating
         * to this function.
         *
         * @param uri The URI of the ontology to add.
         * @param ontology Soprano Model defining the ontology.
         * @param isNew Defines whether the element truly is a new one or just
                        recreated again and to be loaded from persistence.
         * @return True if the operation was successful, false otherwise.
         */
        bool addOntologyInternal( const QUrl& uri, Soprano::Model* model = 0, bool isNew = false );

        /**
         * Refreshes the ontology storage from the Nepomuk main model.
         *
         * @return True if the refresh changed storage state, false otherwise.
         */
        bool refresh();

        /**
         * Deletes and removes all elements from the ontology collection.
         */
        void clearOntologies();

        /**
         * Retrieves the list of ontologies stored in the Nepomuk main model.
         *
         * @return A set of URIs identifying the available ontologies.
         */
        static QSet< QUrl > retrieveOntologyUris();

        class Private;
        Private* const d;
    };

}

#endif
