/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_ONTOLOGYELEMENT_H_
#define _SEMCLIP_ONTOLOGYELEMENT_H_

#include "uncopyable.h"

#include <QtCore/QUrl>

namespace Soprano
{
    class Model;
}

namespace SemClip
{

    /**
     * \class OntologyElement ontologyelement.h "ontologyelement.h"
     *
     * \brief Abstract base class for ontology elements.
     *
     * OntologyElement defines the base class for ontology elements, which can
     * be managed by an ontology storage and will be used by GraphTranslator.
     *
     * The base class defines functions for accessing the ontology URI and the
     * actual ontology RDF definition in the form of a Soprano model (loaded on
     * demand), and declares functions for persistence management.
     *
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class OntologyElement : private Uncopyable
    {
    public:
        /**
         * Default destructor.
         */
        virtual ~OntologyElement();

        /**
         * Getter for the ontology URI.
         *
         * @return The ontolgy URI.
         */
        QUrl uri() const;

        /**
         * Getter for the ontology content. Uses lazy initialization.
         *
         * @return The ontology content in the form of a Soprano model.
         */
        const Soprano::Model* content() const;

        /**
         * Setter for the ontology content. The ontology element takes over
         * ownership of the supplied Soprano model.
         *
         * @param content The new ontology content in form of a Soprano model.
         */
        virtual void setContent( Soprano::Model* content );

        /**
         * Loads the ontology content from the persistent storage. Subclasses
         * have to implement loadContentInternal instead of overriding this.
         *
         * @return True if the operation was successful, false otherwise.
         */
        bool loadContent();

        /**
         * Saves the ontology content to the persistent storage.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool saveContent() const = 0;

        /**
         * Erases the ontology content from the persistent storage.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool eraseContent() const = 0;

    protected:
        /**
         * Custom contructor, no default contructor is supplied.
         *
         * @param uri The ontology URI.
         * @param isNew Defines whether the element truly is a new one or just
                        recreated again and to be loaded from persistence.
         */
        explicit OntologyElement( const QUrl& uri, bool isNew = true );

        /**
         * Loads the ontology content from the persistent storage. This const
         * version is used internally, to ensure that only mutable data members
         * are modified.
         *
         * @return True if the operation was successful, false otherwise.
         */
        virtual bool loadContentInternal() const = 0;

        const QUrl              m_uri;
        mutable Soprano::Model* m_content;
        mutable bool            m_needsInitialization;
    };

}

#endif
