/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "semanticclipboard.h"

#include "graphcompletor.h"
#include "graphtranslator.h"
#include "graphutility.h"

#include <QtCore/QByteArray>
#include <QtCore/QMimeData>
#include <QtCore/QString>
#include <QtGui/QApplication>
#include <QtGui/QClipboard>

#include <Soprano/Global>

class SemClip::SemanticClipboard::Private
{
public:
};

SemClip::SemanticClipboard::SemanticClipboard()
    : QObject( 0 ),
      GraphTranslation(),
      GraphCompletion(),
      d( new Private() )
{
}

SemClip::SemanticClipboard::~SemanticClipboard()
{
    delete d;
}

void SemClip::SemanticClipboard::clear()
{
    QApplication::clipboard()->clear();
}

void SemClip::SemanticClipboard::setSerialization( const QString& serialization )
{
    if ( serialization.isEmpty() ) {
        return;
    }

    QMimeData* mimeData = new QMimeData();
    addSerialization( mimeData, serialization );
    QApplication::clipboard()->setMimeData( mimeData );
}

void SemClip::SemanticClipboard::addSerialization( QMimeData* mimeData, const QString& serialization )
{
    if ( !serialization.isEmpty() ) {
        mimeData->setData( mimeType(), serialization.toUtf8() );
    }
}

void SemClip::SemanticClipboard::setModel( const Soprano::Model* model )
{
    setSerialization( SemClip::serializeModelToRdfXml( model ) );
}

void SemClip::SemanticClipboard::addModel( QMimeData* mimeData, const Soprano::Model* model )
{
    addSerialization( mimeData, SemClip::serializeModelToRdfXml( model ) );
}

QString SemClip::SemanticClipboard::serialization() const
{
    if ( !hasData() ) {
        return QString();
    }

    const QMimeData* mimeData = QApplication::clipboard()->mimeData();
    const QByteArray rawData = mimeData->data( mimeType() );
    return QString::fromUtf8( rawData.constData(), rawData.size() );
}

Soprano::Model* SemClip::SemanticClipboard::model() const
{
    const QString serialization = this->serialization();
    if ( serialization.isEmpty() ) {
        return 0;
    }

    return SemClip::parseModelFromRdfXml( serialization );
}

bool SemClip::SemanticClipboard::hasData() const
{
    return QApplication::clipboard()->mimeData()->hasFormat( mimeType() );
}

QString SemClip::SemanticClipboard::mimeType()
{
    return Soprano::serializationMimeType( Soprano::SerializationRdfXml );
}

void SemClip::SemanticClipboard::translate( Soprano::Model* model, const QSet< QUrl >& targetOntologies )
{
    GraphTranslator::instance()->translate( model, targetOntologies );
}

void SemClip::SemanticClipboard::translate( Soprano::Model* model, const QUrl& targetOntology )
{
    GraphTranslator::instance()->translate( model, targetOntology );
}

void SemClip::SemanticClipboard::translate( QString& serialization, const QSet< QUrl >& targetOntologies )
{
    GraphTranslator::instance()->translate( serialization, targetOntologies );
}

void SemClip::SemanticClipboard::translate( QString& serialization, const QUrl& targetOntology )
{
    GraphTranslator::instance()->translate( serialization, targetOntology );
}

void SemClip::SemanticClipboard::complete( Soprano::Model* model, const QSet< QUrl >& completionGoals,
                                           const QSet< QUrl >& targetOntologies, CompletionMode mode )
{
    GraphCompletor::instance()->complete( model, completionGoals, targetOntologies, mode );
}

void SemClip::SemanticClipboard::complete( QString& serialization, const QSet< QUrl >& completionGoals,
                                           const QSet< QUrl >& targetOntologies, CompletionMode mode )
{
    GraphCompletor::instance()->complete( serialization, completionGoals, targetOntologies, mode );
}

#include "semanticclipboard.moc"
