/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#ifndef DOCUMENT_H_
#define DOCUMENT_H_

#include <QObject>
#include <QList>
#include <QDebug>
#include <nepomuk/class.h>
#include <annotation.h>

namespace Nepomuk
{
    class DocumentPrivate;

    /**
     * @brief A document to be saved through SmartFileDialog.
     *
     * This object type contains all the parameters that the
     * user sets through the dialog.
     *
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com>
     */
    class Document: public QObject
    {
        friend class DocumentPrivate;
        
        Q_OBJECT

        public:
            /**
             * Constructor.
             *
             * @sa Nepomuk::Types::Class
             *
             * @param cls the Nepomuk::Types::Class which represents this document instance.
             * @param parent the object parent
             */
            Document(Nepomuk::Types::Class& cls, QObject* parent = 0);

            /**
             * Destructor.
             */
            virtual ~Document();

            /**
             * Class setter.
             *
             * @param cls the new class
             */
            void setTypeClass(const Nepomuk::Types::Class& cls);

            /**
             * Class getter.
             * 
             * @return class type
             */
            Nepomuk::Types::Class typeClass() const;
            
            /**
             * PIMO type setter (PIMO:Thing is the default).
             * 
             * @param cls the new PIMO type
             */
            void setPIMOType(const Nepomuk::Types::Class& cls);
            
            /**
             * PIMO type getter.
             * 
             * @return PIMO class type
             */
            Nepomuk::Types::Class PIMOType() const;

            /**
             * Add a new annotation to this document.
             *
             * @param annotation the new annotation
             */
            void addAnnotation(Nepomuk::Annotation* annotation);

            /**
             * Add a list of new annotations to this document.
             * 
             * @param annotations the new annotations
             */
            void addAnnotations(QList<Nepomuk::Annotation*> annotations);

            /**
             * Get all the document annotations.
             *
             * @return the all annotations
             */
            QList<Nepomuk::Annotation*> annotations() const;

            /**
             * Set the document label.
             *
             * @param label the label
             */
            void setLabel(const QString& label);

            /**
             * Get the document label.
             *
             * @return the label
             */
            QString label() const;

            /**
             * Set the document comment.
             *
             * @param comment the comment
             */
            void setComment(const QString& comment);

            /**
             * Get the document comment.
             *
             * @return the comment
             */
            QString comment() const;

            /**
             * Get the filename.
             * 
             * @return the filename
             */
            QString filename() const;
            
            void setFileExtension(const QString& extension);
            
            QString fileExtension() const;

            /**
             * Print this object.
             */
            void print();

        private:
            DocumentPrivate* const d;

    };

}

#endif /* DOCUMENT_H_ */
