/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#include <QDir>
#include <QDebug>
#include <QDateTime>
#include <kglobal.h>
#include <kdeversion.h>
#include <nepomuk/thing.h>
#include "nfo.h"
#include "pimo.h"
#include "documentmanager.h"

// we are going to use the new notification system, if we can...
#if KDE_IS_VERSION(4, 3, 73)
#include <kstatusnotifieritem.h>
#else
#include <knotification.h>
#endif

using namespace Nepomuk;

class Nepomuk::DocumentManagerPrivate
{
    public:
        DocumentManagerPrivate() :
            document(0)
        {
        }
        Document* document;
        QString filename;
};

DocumentManager::DocumentManager(Nepomuk::Types::Class& cls) :
    QThread(0), d(new DocumentManagerPrivate())
{
    d->document = new Document(cls, this);
}

DocumentManager::~DocumentManager()
{
    delete d;
}

Document* DocumentManager::document()
{
    return d->document;
}

QString DocumentManager::filename()
{
    return d->filename;
}

bool DocumentManager::createPath()
{
    // elaborating name
    QDir dir(QDir::homePath());
    if (!dir.cd(MAINDIR)) {
        if (!dir.mkdir(MAINDIR)) {
            return false;
        }
        dir.cd(MAINDIR);
    }
    if (!dir.cd(d->document->typeClass().name())) {
        if (!dir.mkdir(d->document->typeClass().name())) {
            return false;
        }
        dir.cd(d->document->typeClass().name());
    }
    // unique path made with timestamp - please refer to the
    // documentation for this function
    QDateTime dt = QDateTime::currentDateTime();
    QString name = d->document->label().replace(" ", "_").append(dt.toString("_yyyyMMdd_hhmmss")).append(d->document->fileExtension());
    d->filename = dir.filePath(name);

    return true;
}

void DocumentManager::save()
{
    connect(this, SIGNAL(finished()), this, SLOT(saved()));
    // avoid quitting of the application
    KGlobal::ref();
    this->start();
}

void DocumentManager::saved()
{
    // new notification
#if KDE_IS_VERSION(4, 3, 73)
    KStatusNotifierItem* item = new KStatusNotifierItem();
    item->setTitle("Smartsave dialog");
    item->setIconByName("nepomuk");
    item->setToolTipIconByName("nepomuk");
    item->setToolTipTitle("Smartsave dialog");
    item->setToolTipSubTitle("The document metadata has just been saved.");
    item->showMessage("Smartsave dialog", "Document metadata saved", "nepomuk", 5000);
#else
    KNotification::event(KNotification::Notification, "Document metadata saved.");
#endif
    
    // ok, now the app can safely quit
    KGlobal::deref();
    this->deleteLater();
}

void DocumentManager::run()
{
    // creating resource and saving informations
    Resource res(d->filename, d->document->typeClass().uri());
    res.setLabel(d->document->label());
    res.addType(d->document->PIMOType().uri());
    if (d->document->comment().length() > 0) {
        res.setDescription(d->document->comment());
    }
    for (int i = 0; i < d->document->annotations().size(); ++i) {
        d->document->annotations().at(i)->create(res);
    }
}

const QString DocumentManager::MAINDIR("Nepomuk");
const Nepomuk::Types::Class DocumentManager::DEFAULTCLS(Nepomuk::Vocabulary::NFO::FileDataObject());
const Nepomuk::Types::Class DocumentManager::DEFAULTPIMOCLS(Nepomuk::Vocabulary::PIMO::Document());

/*
 * Here we are making nothing extraordinary, but in perspective it's better
 * if DocumentManager instances are built here and not publicly; maybe in
 * the future we will need to do some different operations...
 */
DocumentManager* DocumentManager::createDocumentManager(Nepomuk::Types::Class& cls)
{
    DocumentManager* manager = new DocumentManager(cls);
    return manager;
}
