/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#ifndef DOCUMENTMANAGER_H_
#define DOCUMENTMANAGER_H_

#include <QThread>
#include <QRunnable>
#include <nepomuk/class.h>
#include "document.h"

namespace Nepomuk
{
    class DocumentManagerPrivate;

    /**
     * @brief Manager for document instances.
     * 
     * Each instance of this class has only one document associated,
     * and it is used for making operations on that document.
     * Each operation is handled in a
     * different thread for avoiding program freezes during
     * Nepomuk long operations; when an operation ends, the event is
     * notified through KNotification.
     * 
     * The document manager, its document instance and all the annotations
     * that have been associated to it will be deleted after the notification,
     * so there is no need of a parent object to be passed.
     * Because of this, create a manager only just before executing it, and
     * be sure to call QObject::deleteAfter() if you are not going to run
     * it as a new thread. 
     * 
     * This class implements the Factory pattern.
     * 
     * @sa Document, KNotification
     * 
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com>
     */
    class DocumentManager : public QThread
    {
        friend class DocumentManagerPrivate;
        
        Q_OBJECT
        
        public:
            /**
             * The home directory containing all Nepomuk types subdirectories
             * and files.
             */
            static const QString MAINDIR;
            
            /**
             * The default NFO type class to be associated to a 
             * document; it corresponds to NFO:Document.
             */
            static const Nepomuk::Types::Class DEFAULTCLS;
            
            /**
             * The default PIMO type class to be associated to
             * a document; it corresponds to PIMO:Document.
             */
            static const Nepomuk::Types::Class DEFAULTPIMOCLS;
            
            /**
             * Factory method for creating document managers; the user
             * has to provide only the Nepomuk class type of the managed
             * document.
             * There is no parent parameter, because each class instance
             * will delete itself after each execution.
             * 
             * @param cls the document type class
             */
            static DocumentManager* createDocumentManager(Nepomuk::Types::Class& cls);
            
            /**
             * Destructor.
             */
            virtual ~DocumentManager();
            
            /**
             * The unique document instance associated to this manager.
             * 
             * @return a Document instance
             */
            Nepomuk::Document* document();
            
            /**
             * The final filename given to the document; it is filled
             * by DocumentManager::createPath().
             * 
             * @return the filename
             */
            QString filename();
            
            /**
             * Alias for QThread::start().
             */
            void save();

            /**
             * Method for creating the final path.
             * This function creates a very simple path, waiting for an interesting
             * algorithm that should be introduced this autumn by Sebastian Faubler.
             * 
             * @return true if the path has been successfully created, false otherwise
             */
            bool createPath();

            /**
             * This method will create a resource associated to the
             * filename and it will associate to it a label, a
             * comment and all the annotations.
             */
            void run();
            
        protected:
            /**
             * Protected constructor, it creates a new Document instance
             * using the passed parameter.
             * 
             * @param cls the Nepomuk document class type
             */
            DocumentManager(Nepomuk::Types::Class& cls);
            
        private:
            DocumentManagerPrivate* d;
            
        private Q_SLOTS:
            void saved();
    };

}

#endif /* DOCUMENTMANAGER_H_ */
