/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
             (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <QList>
#include <QUrl>
#include <QDebug>
#include <QHash>

#include <KDebug>

#include <nepomuk/variant.h>
#include <nepomuk/resource.h>
#include <nepomuk/property.h>
#include <nepomuk/literal.h>
#include <nepomuk/resourcemanager.h>
#include "pimo.h"
#include "nfo.h"

#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/Xesam>
#include <Soprano/Vocabulary/XMLSchema>
#include <Soprano/Util/AsyncQuery>

#include "nie.h"
#include "facetedfiltermodel.h"
#include "asyncquery.h"
#include "filter.h"
#include "documentmanager.h"


class Nepomuk::FacetedFilterModelPrivate
{
public:
    FacetedFilterModelPrivate()
        : m_lastQuery( 0 )
    {
    }

    int increment( const Filter& filter ) {
    	int result;
        QHash<Filter, int>::iterator it = m_tmpFilterCandidates.find( filter );
        if ( it == m_tmpFilterCandidates.end() ) {
        	// this filter shows up for the first time
            m_tmpFilterCandidates.insert( filter, 1 );
            return 1;
        }
        else {
        	// increment the filter counter
        	result = it.value();
        	m_tmpFilterCandidates.insert( filter, ++result );
            return result;
        }
    }

    void updateFilters();
    void _k_filterQueryNext( Soprano::Util::AsyncQuery* query );
    void _k_filterQueryFinished( Soprano::Util::AsyncQuery* query );

    /// temporary list for building m_filters
    QHash<Filter, int> m_tmpFilterCandidates;

    /// the list of selected (and thus, fixed) filters,
    /// these are the ones displayed as checked at the top of the list
    QList<Filter> m_selectedFilters;

    /// The list of possible filters based on m_selectedFilters and m_forcedFilters
    /// This list is updated in updateFilters whenever m_selectedFilters or
    /// m_forcedFilters changes
    QList<Filter> m_filters;

    /// a list of filters that is forced from outside but not displayed
    /// can be used for dates and such which need a different GUI
    QList<Filter> m_forcedFilters;

    /// the currently running filter query
    Soprano::Util::AsyncQuery* m_lastQuery;

    FacetedFilterModel* q;
};


void Nepomuk::FacetedFilterModelPrivate::updateFilters()
{
    // cleanup (the query object will delete itself)
    if ( m_lastQuery ) {
        m_lastQuery->disconnect( q );
        m_lastQuery->close();
    }

    // clear the non-selected-filters
    if ( !m_filters.isEmpty() ) {
        q->beginRemoveRows( QModelIndex(), m_selectedFilters.count(), m_selectedFilters.count() + m_filters.count() );
        m_filters.clear();
        q->endRemoveRows();
    }

    // cleaup the temp list
    m_tmpFilterCandidates.clear();

    // build the query
    QString query = QLatin1String( "select ?r ?p ?o where { ?r a " )
                    + Soprano::Node::resourceToN3(Nepomuk::DocumentManager::DEFAULTCLS.uri() )
                    + QLatin1String( " . " );

    // use existing filters
    foreach (const Filter filter, m_selectedFilters) {
        query.append( filter.toSparqlPattern( QLatin1String("?r") ) );
    }

    // use forced filters
    foreach( const Filter& filter, m_forcedFilters ) {
        query.append( filter.toSparqlPattern( QLatin1String("?r") ) );
    }

    // This is an awful expression! But I have no idea how else to exclude folders
    query.append( QString( "?r ?p ?o . OPTIONAL { ?r2 a %1 . FILTER(?r=?r2) . } . FILTER(!BOUND(?r2)) . }" )
                  .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NFO::Folder() ) ) );

    kDebug() << "Filter query" << query;

    // start the new query
    m_lastQuery = Soprano::Util::AsyncQuery::executeQuery( ResourceManager::instance()->mainModel(),
                                                           query,
                                                           Soprano::Query::QueryLanguageSparql );
    q->connect( m_lastQuery, SIGNAL( nextReady( Soprano::Util::AsyncQuery* ) ),
                SLOT( _k_filterQueryNext( Soprano::Util::AsyncQuery* ) ) );
    q->connect( m_lastQuery, SIGNAL( finished( Soprano::Util::AsyncQuery* ) ),
                SLOT( _k_filterQueryFinished( Soprano::Util::AsyncQuery* ) ) );
}


/*
 * A filter with a value of 1 (the lower bound)
 * is not interesting, at least as soon as it does not get
 * to at least two; from there up we want to show it, so let's show
 * it in the first place and then ignore it.
 * The model is not interested (for now) in how much files
 * are associated to a filter (like a real faceted browsing
 * system).
 */
void Nepomuk::FacetedFilterModelPrivate::_k_filterQueryNext( Soprano::Util::AsyncQuery* query )
{
    Filter filter( query->binding( QLatin1String( "p" ) ).uri(),
                   query->binding( QLatin1String( "o" ) ) );

    if ( !q->hiddenProperties().contains(filter.property().uri()) &&
         !m_selectedFilters.contains( filter ) &&
         !(filter.property().uri() == Soprano::Vocabulary::RDF::type() &&
           ( filter.value().uri() == Soprano::Vocabulary::Xesam::File() ||
             filter.value().uri() == Nepomuk::Vocabulary::NFO::FileDataObject())) &&
         ( filter.value().isResource() || filter.value().literal().isString() ) ) {
        if ( increment( filter ) == 2 ) {
            q->beginInsertRows( QModelIndex(), m_selectedFilters.count() + m_filters.count(), m_selectedFilters.count() + m_filters.count() );
            m_filters.append(filter);
            q->endInsertRows();
        }
    }

    query->next();
}


void Nepomuk::FacetedFilterModelPrivate::_k_filterQueryFinished( Soprano::Util::AsyncQuery* )
{
    m_lastQuery = 0;
    kDebug() << "Filter query done.";
}


Nepomuk::FacetedFilterModel::FacetedFilterModel(QObject* parent)
    : QAbstractListModel(parent),
      d(new FacetedFilterModelPrivate())
{
    d->q = this;
}

Nepomuk::FacetedFilterModel::~FacetedFilterModel()
{
    delete d;
}

int Nepomuk::FacetedFilterModel::rowCount(const QModelIndex &parent) const
{
    if ( !parent.isValid() )
        return d->m_selectedFilters.count() + d->m_filters.count();
    else
        return 0;
}

QVariant Nepomuk::FacetedFilterModel::data(const QModelIndex& index, int role) const
{
    if (index.row() < rowCount() && index.column() == 0) {
        Filter filter;
        if ( index.row() < d->m_selectedFilters.count() )
            filter = d->m_selectedFilters[index.row()];
        else
            filter = d->m_filters[index.row() - d->m_selectedFilters.count()];

        if (role == Qt::DisplayRole) {
            QString value;
            if ( filter.value().isLiteral() ) {
                // handle datetime as a special case,
                // date and time are almost never used so we ignore them
                if ( filter.value().literal().isDateTime() )
                    value = KGlobal::locale()->formatDateTime( filter.value().literal().toDateTime(), KLocale::FancyShortDate );
                else
                    value = filter.value().toString();
            }
            else {
                Nepomuk::Resource res( filter.value().uri() );
                value = res.genericLabel();
            }
            return QString("%1 '%2'")
                .arg(filter.propertyLabel())
                .arg(value);
        }
        else if ( role == PropertyLabelRole ) {
            return filter.property().label();
        }
        else if (role == Qt::CheckStateRole) {
            return( index.row() < d->m_selectedFilters.count() ? Qt::Checked : Qt::Unchecked );
        }
    }

    return QVariant();
}

bool Nepomuk::FacetedFilterModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
    if (index.isValid() && role == Qt::CheckStateRole) {
        if ( index.row() < d->m_selectedFilters.count() &&
             value.toInt() == Qt::Unchecked ) {
            // unchecking a filter
            beginRemoveRows( QModelIndex(), index.row(), index.row() );
            d->m_selectedFilters.removeAt( index.row() );
            endRemoveRows();
            emit filterSelectionChanged();
            d->updateFilters();
            return true;
        }
        else if ( index.row() >= d->m_selectedFilters.count() &&
                  index.row() < d->m_selectedFilters.count() + d->m_filters.count() &&
                  value.toInt() == Qt::Checked ) {
            // checking a filter
            beginInsertRows( QModelIndex(), d->m_selectedFilters.count(), d->m_selectedFilters.count() );
            d->m_selectedFilters.append( d->m_filters.at( index.row() - d->m_selectedFilters.count() ) );
            endInsertRows();
            emit filterSelectionChanged();
            d->updateFilters();
            return true;
        }
    }

    return false;
}

Qt::ItemFlags Nepomuk::FacetedFilterModel::flags(const QModelIndex& index) const
{
    Q_UNUSED(index);

    return Qt::ItemIsEnabled | Qt::ItemIsUserCheckable;
}

QList<Nepomuk::Filter> Nepomuk::FacetedFilterModel::selectedFilters( bool includeForcedFilters ) const
{
    QList<Filter> filters = d->m_selectedFilters;
    if ( includeForcedFilters )
        filters << d->m_forcedFilters;
    return filters;
}

void Nepomuk::FacetedFilterModel::setForcedFilters(const QList<Nepomuk::Filter>& filters)
{
    kDebug();
    d->m_forcedFilters = filters;
    d->updateFilters();
    emit filterSelectionChanged();
}


K_GLOBAL_STATIC( QList<QUrl>, s_hiddenProperties );

QList<QUrl> Nepomuk::FacetedFilterModel::hiddenProperties()
{
    if (s_hiddenProperties->empty()) {
        s_hiddenProperties->append(Nepomuk::Vocabulary::NIE::isPartOf());
        s_hiddenProperties->append(Nepomuk::Vocabulary::NIE::url());
        s_hiddenProperties->append(Nepomuk::Vocabulary::NIE::lastModified());
        s_hiddenProperties->append(Nepomuk::Vocabulary::NIE::contentSize());
        s_hiddenProperties->append(Nepomuk::Vocabulary::NIE::plainTextContent());
        s_hiddenProperties->append(Nepomuk::Vocabulary::NFO::fileName());
        s_hiddenProperties->append(Nepomuk::Vocabulary::PIMO::groundingOccurrence());
        s_hiddenProperties->append(Nepomuk::Vocabulary::PIMO::Thing());
        s_hiddenProperties->append(Nepomuk::Vocabulary::PIMO::Document());
        s_hiddenProperties->append(Soprano::Vocabulary::NAO::created());
        s_hiddenProperties->append(Soprano::Vocabulary::NAO::lastModified());
        s_hiddenProperties->append(Soprano::Vocabulary::NAO::prefLabel());
        s_hiddenProperties->append(Soprano::Vocabulary::NAO::description());
        s_hiddenProperties->append(Soprano::Vocabulary::NAO::identifier());

        // legacy Xesam stuff
        *s_hiddenProperties << Soprano::Vocabulary::Xesam::url();
        *s_hiddenProperties << Soprano::Vocabulary::Xesam::asText();
        *s_hiddenProperties << Soprano::Vocabulary::Xesam::name();
        *s_hiddenProperties << Soprano::Vocabulary::Xesam::fileExtension();

        // the ugly strigi depth field that is higher than 0 for embedded files (including files in archives)
        *s_hiddenProperties << QUrl::fromEncoded( "http://strigi.sf.net/ontologies/0.9#depth" );
        // legacy strigi parent url
        *s_hiddenProperties << QUrl::fromEncoded( "http://strigi.sf.net/ontologies/0.9#parentUrl" );
        // I doubt anyone would filter for this
        *s_hiddenProperties << QUrl::fromEncoded( "http://strigi.sf.net/ontologies/homeless#lineEndingFormat");
    }

    return *s_hiddenProperties;
}

#include "facetedfiltermodel.moc"
