/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
             (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef FACETEDFILTERMODEL_H_
#define FACETEDFILTERMODEL_H_

#include <QtCore/QAbstractListModel>
#include <QtCore/QList>
#include <QtCore/QUrl>

#include "filter.h"

namespace Soprano {
    namespace Util {
        class AsyncQuery;
    }
}

namespace Nepomuk
{
    class FacetedFilterModelPrivate;


    /**
     * @brief Model for filters in the open dialog.
     *
     * This model registers, for each item, all properties
     * given through the methods; for each property, a label,
     * a boolean indicator for the selection and a counter,
     * setting for each property how many files are given, are
     * saved.
     *
     * @sa QAbstractListModel
     *
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com><br/>
     *         Sebastian Trueg <trueg@kde.org>
     */
    class FacetedFilterModel : public QAbstractListModel
    {
        Q_OBJECT

    public:
        /**
         * Constructor.
         *
         * @param parent list parent
         */
        FacetedFilterModel(QObject* parent=0);

        /**
         * Destructor.
         */
        virtual ~FacetedFilterModel();

        /**
         * Number of rows in this model.
         *
         * @param parent not used
         */
        int rowCount(const QModelIndex& parent = QModelIndex()) const;

        /**
         * Data for this model; the given role can be the standard
         * display or the roles provided by this class.
         *
         * @param index the index for the data
         * @param role the role
         * @return a variant with data; it can be a string, a boolean or an integer
         */
        QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;

        /**
         * This method set a new boolean to the FacetedFilterModel::BoolRole;
         * that role is the only one accepted.
         *
         * @param index the index to be changed
         * @param value the new value
         * @param role the role to be set
         * @return true if the new value has been correctly saved
         */
        bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);

        /**
         * Flags for this model; it is overridden for having checkboxes
         * for each row.
         *
         * @param index the index of this model
         * @return the flags associated to that index
         */
        Qt::ItemFlags flags(const QModelIndex& index) const;

        QList<Filter> selectedFilters( bool includeForcedFilters = true ) const;

        /**
         * A list of nodes to be blacklisted.
         *
         * @return a list of URIs that are currently blacklisted
         */
        static QList<QUrl> hiddenProperties();

        enum CustomRoles {
            /**
             * Returns the label of the property.
             */
            PropertyLabelRole = 87689
        };

    Q_SIGNALS:
        void filterSelectionChanged();

    public Q_SLOTS:
        /**
         * Set properties for this model; all the old properties
         * are deleted, and for each resource all new properties
         * are saved.
         * Length of the two lists is checked, and it must be the
         * same.
         *
         * @param filters a list of names of properties with values
         */
        void setForcedFilters(const QList<Nepomuk::Filter>& filters);

    private:
        FacetedFilterModelPrivate* d;
        friend class FacetedFilterModelPrivate;

        Q_PRIVATE_SLOT( d, void _k_filterQueryNext( Soprano::Util::AsyncQuery* ) )
        Q_PRIVATE_SLOT( d, void _k_filterQueryFinished( Soprano::Util::AsyncQuery* ) )
    };
}

#endif /* FACETEDFILTERMODEL_H_ */
