/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) version 3, or any
   later version accepted by the membership of KDE e.V. (or its
   successor approved by the membership of KDE e.V.), which shall
   act as a proxy defined in Section 6 of version 3 of the license.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filter.h"

#include <QtCore/QSharedData>

#include <Nepomuk/Types/Property>

#include <krandom.h>

#include <Soprano/Node>



namespace {
    QString tunedLabel(const QString& label)
    {
        QString tunedLabel;
        const int labelLength = label.length();
        if (labelLength > 0) {
            tunedLabel.reserve(labelLength);
            tunedLabel = label[0].toUpper();
            for (int i = 1; i < labelLength; ++i) {
                if (label[i].isUpper() && !label[i - 1].isSpace() && !label[i - 1].isUpper()) {
                    tunedLabel += ' ';
                    tunedLabel += label[i].toLower();
                } else {
                    tunedLabel += label[i];
                }
            }
        }
        return tunedLabel;
    }

    QString generateVarName()
    {
        return QLatin1String( "?v" ) + KRandom::randomString( 5 );
    }
}


class Nepomuk::Filter::Private : public QSharedData
{
public:
    Types::Property m_property;
    Soprano::Node m_value;
    Operators m_operators;

    QString operatorString() const {
        QString op;
        if ( m_operators & LessThan )
            op += '<';
        else if ( m_operators & MoreThan )
            op += '>';
        if ( m_operators & Equals )
            op += '=';
        return op;
    }
};

Nepomuk::Filter::Filter()
    : d( new Private() )
{
}


Nepomuk::Filter::Filter( const Filter& other )
{
    d = other.d;
}


Nepomuk::Filter::Filter( const Types::Property& property, const Soprano::Node& value, Operators operators )
    : d( new Private() )
{
    d->m_property = property;
    d->m_value = value;
    d->m_operators = operators;
}


Nepomuk::Filter::~Filter()
{
}


Nepomuk::Filter& Nepomuk::Filter::operator=( const Filter& other )
{
    d = other.d;
    return *this;
}


bool Nepomuk::Filter::operator==( const Filter& other ) const
{
    return( d.constData() == other.d.constData() ||
            ( d->m_property == other.d->m_property &&
              d->m_value == other.d->m_value &&
              d->m_operators == other.d->m_operators ) );
}


Nepomuk::Types::Property Nepomuk::Filter::property() const
{
    return d->m_property;
}


Soprano::Node Nepomuk::Filter::value() const
{
    return d->m_value;
}


Nepomuk::Filter::Operators Nepomuk::Filter::operators() const
{
    return d->m_operators;
}


QString Nepomuk::Filter::propertyLabel() const
{
    return tunedLabel( const_cast<Filter*>( this )->d->m_property.label() );
}


QString Nepomuk::Filter::toSparqlPattern( const QString& varName ) const
{
    if ( d->m_operators == Equals ) {
        return QString( "%1 %2 %3 . " )
            .arg( varName )
            .arg( Soprano::Node::resourceToN3( d->m_property.uri() ) )
            .arg( d->m_value.toN3() );
    }
    else {
        return QString( "%1 %2 %3 . FILTER(%3 %4 %5) . " )
            .arg( varName )
            .arg( Soprano::Node::resourceToN3( d->m_property.uri() ) )
            .arg( generateVarName() )
            .arg( d->operatorString() )
            .arg( d->m_value.toN3() );
    }
}


uint Nepomuk::qHash( const Filter& filter )
{
    return ( qHash( filter.property().uri() ) << 16 ) | qHash( filter.value() );
}
