/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
             (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef _SMART_FILE_MODULE_H_
#define _SMART_FILE_MODULE_H_

#include <kabstractfilemodule.h>

namespace Nepomuk {
    
    /**
     * @brief A Factory class for the semantic file widget.
     * 
     * This class is an exported plugin, which acts as a Factory
     * class for creating instances of the SmartFileWidget.
     * It returns also an instance of the standard file widget, from
     * kfile library.
     * 
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com>
     * @author Sebastian Trueg <trueg@kde.org>
     * 
     * @sa SmartFileWidget
     */
    class SmartFileModule : public KAbstractFileModule
    {
        Q_OBJECT

    public:
        /**
         * Constructor.
         * 
         * @param parent the parent widget
         * @param args not used
         */
        SmartFileModule( QObject* parent, const QVariantList& args);
        
        /**
         * Destructor.
         */
        ~SmartFileModule();

        /**
         * This is the core method of the whole class: it creates an
         * instance of the SmartFileWidget, returning it as a QWidget.
         * 
         * @param startDir the start directory, set to the classic file widget
         * @param parent the parent widget
         */
        QWidget* createFileWidget(const KUrl& startDir, QWidget *parent);

        /**
         * Get the start URL.
         * Falls back to the classic implementation.
         * 
         * @param startDir the start directory
         * @param recentDirClass the recent directories
         * @return the start URL
         */
        KUrl getStartUrl( const KUrl& startDir, QString& recentDirClass );

        /**
         * Set the start directory.
         * Falls back to the classic implementation.
         * 
         * @param directory the start directory as a URL
         */
        void setStartDir( const KUrl& directory );

        /**
         * Select a directory.
         * Falls back to the classic implementation.
         * 
         * @param startDir the start directory
         * @param localOnly only local directories are shown
         * @param parent the parent widget
         * @param caption the dialog caption
         * @return the selected URL
         */
        KUrl selectDirectory( const KUrl& startDir,
                              bool localOnly, QWidget *parent,
                              const QString& caption );

        /**
         * This returns an instance of the classic file module.
         * It acts like a Singleton method, returning the same
         * file module.
         * 
         * @return the classic file module
         */
        KAbstractFileModule* classicFileModule();

    private:
        /// An instance of the classic file module for fallback
        KAbstractFileModule* m_classicFileModule;
    };
}

#endif
