/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
   (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "smartfilewidget.h"
#include "smartopenwidget.h"
#include "smartsavewidget.h"
#include "smartfilemodule.h"
#include "documentmanager.h"

#include "annotation.h"
#include "annotationrequest.h"

#include <nepomuk/resourcemanager.h>

#include <ktoolbar.h>
#include <kurlcombobox.h>
#include <kfilefiltercombo.h>
#include <kdebug.h>

#include <QtGui/QStackedWidget>
#include <QtGui/QFormLayout>

using namespace Nepomuk;

class Nepomuk::SmartFileWidgetPrivate
{
public:
    SmartFileWidgetPrivate() :
        suggestions(QList<Annotation*> ()), request(AnnotationRequest()),
        cls(DocumentManager::DEFAULTCLS),
        pimocls(DocumentManager::DEFAULTPIMOCLS),
        semantic(false), mode(KAbstractFileWidget::Opening),
        stdWidget(0), stack(0), switchButton(0), custom(0)
    {
    }

    QList<Annotation*> suggestions;
    AnnotationRequest request;
    Nepomuk::Types::Class cls;
    Nepomuk::Types::Class pimocls;
    bool semantic;
    KAbstractFileWidget::OperationMode mode;
    QString selectedFile;

    SmartOpenWidget* openWidget;
    SmartSaveWidget* saveWidget;
    QWidget* stdWidget;
    KAbstractFileWidget* kFileWidget;
    QStackedWidget* stack;
    KPushButton *okButton, *cancelButton;
    KPushButton* switchButton;
    QWidget* custom;
};

Nepomuk::SmartFileWidget::SmartFileWidget( const KUrl& startDir, SmartFileModule* module, QWidget* parent )
    : QWidget( parent ),
      KAbstractFileWidget(),
      d( new SmartFileWidgetPrivate() ),
      m_module( module )
{
    // initialize Nepomuk
    // -------------------------------------
    Nepomuk::ResourceManager::instance()->init();

    // setup file widget stack
    // -------------------------------------
    d->stack = new QStackedWidget(this);

    d->openWidget = new SmartOpenWidget( this, d->stack );
    d->stack->addWidget(d->openWidget);

    d->saveWidget = new SmartSaveWidget( this, d->stack );
    d->stack->addWidget(d->saveWidget);

    d->stdWidget = m_module->classicFileModule()->createFileWidget(startDir, d->stack);
    d->kFileWidget = qobject_cast<KAbstractFileWidget*>(d->stdWidget);
    d->stack->addWidget(d->stdWidget);

    // pipe signals from the classic module through to us
    // -------------------------------------
    connect(d->stdWidget, SIGNAL(fileSelected(KUrl)),
            SIGNAL(fileSelected(KUrl)));
    connect(d->stdWidget, SIGNAL(fileHighlighted(KUrl)),
            SIGNAL(fileHighlighted(KUrl)));
    connect(d->stdWidget, SIGNAL(selectionChanged()),
            SIGNAL(selectionChanged()));
    connect(d->stdWidget, SIGNAL(filterChanged(const QString&)),
            SIGNAL(filterChanged(const QString&)));
    connect(d->stdWidget, SIGNAL(accepted()), SIGNAL(accepted()));

    // setup buttons
    // -------------------------------------
    d->okButton = new KPushButton(KStandardGuiItem::ok(), this);
    d->okButton->setDefault(true);
    d->cancelButton = new KPushButton(KStandardGuiItem::cancel(), this);
    // The dialog shows them
    d->okButton->hide();
    d->cancelButton->hide();

    d->switchButton = new KPushButton();
    connect(d->switchButton, SIGNAL(clicked()), this, SLOT(switchDialog()));

    // layout
    // -------------------------------------
    QGridLayout* layout = new QGridLayout(this);
    layout->addWidget( d->stack, 0, 0, 1, 4 );
    layout->addWidget( d->switchButton, 1, 1, 1, 1 );
    layout->addWidget( d->okButton, 1, 2, 1, 1 );
    layout->addWidget( d->cancelButton, 1, 3, 1, 1 );
    layout->setColumnStretch( 0, 1 );

    // we always start out in legacy mode
    setSemanticModeEnabled( false );
}

Nepomuk::SmartFileWidget::~SmartFileWidget()
{
    delete d;
}

KUrl Nepomuk::SmartFileWidget::selectedUrl() const
{
    if (d->semantic) {
        if (d->mode == Opening) {
            return d->openWidget->getFilenameUrl();
        }
        else {
            return d->saveWidget->getFilenameUrl();
        }
    }
    else {
        return d->kFileWidget->selectedUrl();
    }
}

KUrl::List Nepomuk::SmartFileWidget::selectedUrls() const
{
    if (d->semantic) {
        if (d->mode == Opening) {
            return KUrl::List() << d->openWidget->getFilenameUrl();
        }
        else {
            return KUrl::List() << d->saveWidget->getFilenameUrl();
        }
    }
    else {
        return d->kFileWidget->selectedUrls();
    }
}

KUrl Nepomuk::SmartFileWidget::baseUrl() const
{
    return d->kFileWidget->baseUrl();
}

QString Nepomuk::SmartFileWidget::selectedFile() const
{
    if (d->semantic) {
        if (d->mode == Opening) {
            return d->openWidget->getFilename();
        }
        else {
            return d->saveWidget->getFilename();
        }
    }
    else {
        return d->kFileWidget->selectedFile();
    }
}

QStringList Nepomuk::SmartFileWidget::selectedFiles() const
{
    if (d->semantic) {
        if (d->mode == Opening) {
            return QStringList() << d->openWidget->getFilename();
        }
        else {
            return QStringList() << d->saveWidget->getFilename();
        }
    }
    else {
        return d->kFileWidget->selectedFiles();
    }
}

void Nepomuk::SmartFileWidget::setUrl(const KUrl &url, bool clearforward)
{
    d->kFileWidget->setUrl(url, clearforward);
}

void Nepomuk::SmartFileWidget::setSelection(const QString& name)
{
    d->kFileWidget->setSelection(name);
}

void Nepomuk::SmartFileWidget::setOperationMode( OperationMode mode )
{
    d->mode = mode;

    if (d->mode == Opening) {
        // don't use KStandardGuiItem::open() here which has trailing ellipsis!
        d->okButton->setGuiItem( KGuiItem( i18n( "&Open" ), "document-open") );
    }
    else if ( mode == Saving ) {
        d->okButton->setGuiItem( KStandardGuiItem::save() );
    }
    else {
        d->okButton->setGuiItem( KStandardGuiItem::ok() );
    }

    d->kFileWidget->setOperationMode(mode);

    // make sure we show the correct widget in the stack
    setSemanticModeEnabled( d->semantic );
}

KAbstractFileWidget::OperationMode Nepomuk::SmartFileWidget::operationMode() const
{
    return d->mode;
}

void Nepomuk::SmartFileWidget::setKeepLocation( bool keep )
{
    d->kFileWidget->setKeepLocation(keep);
}

bool Nepomuk::SmartFileWidget::keepsLocation() const
{
    return d->kFileWidget->keepsLocation();
}

void Nepomuk::SmartFileWidget::setFilter(const QString& filter)
{
    kDebug() << filter;

    // semantic thing
    int pos = filter.indexOf('/');

    // Check for an un-escaped '/', if found
    // interpret as a MIME filter.

    if (pos > 0 && filter[pos - 1] != '\\') {
        QStringList filters = filter.split(' ', QString::SkipEmptyParts);
        setMimeFilter( filters );
        return;
    }

    // Strip the escape characters from
    // escaped '/' characters.

    QString copy (filter);
    for (pos = 0; (pos = copy.indexOf("\\/", pos)) != -1; ++pos)
        copy.remove(pos, 1);

    if ( d->semantic )
        d->saveWidget->filterWidget()->setFilter(copy);

    // standard thing
    d->kFileWidget->setFilter(filter);
}

QString Nepomuk::SmartFileWidget::currentFilter() const
{
    if (d->semantic) {
        return d->saveWidget->filterWidget()->currentFilter();
    }
    else {
        return d->kFileWidget->currentFilter();
    }
}

KMimeType::Ptr Nepomuk::SmartFileWidget::currentFilterMimeType()
{
    return KMimeType::mimeType( currentMimeFilter() );
}

void Nepomuk::SmartFileWidget::setMimeFilter( const QStringList& types,
                                              const QString& defaultType )
{
    kDebug() << types << defaultType;
    d->saveWidget->filterWidget()->setMimeFilter(types, defaultType);
    d->kFileWidget->setMimeFilter(types, defaultType);
}

QString Nepomuk::SmartFileWidget::currentMimeFilter() const
{
    if (d->semantic) {
        return d->saveWidget->filterWidget()->currentFilter();
    }
    else {
        return d->kFileWidget->currentMimeFilter();
    }
}

void Nepomuk::SmartFileWidget::clearFilter()
{
    d->saveWidget->filterWidget()->setFilter(QString());
    d->kFileWidget->clearFilter();
}

void Nepomuk::SmartFileWidget::setPreviewWidget(KPreviewWidgetBase *w)
{
    d->kFileWidget->setPreviewWidget(w);
}

void Nepomuk::SmartFileWidget::setMode( KFile::Modes m )
{
    if ( ( m & KFile::Directory ) && !( m & KFile::File|KFile::Files ) ) {
        d->saveWidget->filterWidget()->setDefaultFilter( i18n("*|All Folders") );
    }
    else {
        d->saveWidget->filterWidget()->setDefaultFilter( i18n("*|All Files") );
    }
    d->kFileWidget->setMode(m);
}

KFile::Modes Nepomuk::SmartFileWidget::mode() const
{
    return d->kFileWidget->mode();
}

void Nepomuk::SmartFileWidget::setLocationLabel(const QString& text)
{
    d->kFileWidget->setLocationLabel(text);
}

KToolBar *Nepomuk::SmartFileWidget::toolBar() const
{
    return d->kFileWidget->toolBar();
}

KPushButton *Nepomuk::SmartFileWidget::okButton() const
{
    return d->okButton;
}

KPushButton *Nepomuk::SmartFileWidget::cancelButton() const
{
    return d->cancelButton;
}

KUrlComboBox *Nepomuk::SmartFileWidget::locationEdit() const
{
    return d->kFileWidget->locationEdit();
}

KFileFilterCombo *Nepomuk::SmartFileWidget::filterWidget() const
{
    return d->kFileWidget->filterWidget();
}

KActionCollection *Nepomuk::SmartFileWidget::actionCollection() const
{
    return d->kFileWidget->actionCollection();
}

void Nepomuk::SmartFileWidget::setCustomWidget(QWidget* widget)
{
    d->custom = widget;
    d->custom->setParent(this);

    qobject_cast<QGridLayout*>(layout())->addWidget(d->custom, 1, 0, 1, 1);
}

void Nepomuk::SmartFileWidget::setCustomWidget(const QString& text, QWidget* widget)
{
    d->custom = new QWidget(this);
    QFormLayout* l = new QFormLayout(d->custom);
    l->setWidget(0, QFormLayout::LabelRole, new QLabel(text));
    l->setWidget(0, QFormLayout::FieldRole, widget);

    qobject_cast<QGridLayout*>(layout())->addWidget(d->custom, 1, 0, 1, 1);
}

void Nepomuk::SmartFileWidget::slotOk()
{
    if ( d->semantic ) {
        emit accepted();
    }
    else {
        d->kFileWidget->slotOk();
    }
}

void Nepomuk::SmartFileWidget::accept()
{
    if ( !d->semantic ) {
        d->kFileWidget->accept();
    }
}

void Nepomuk::SmartFileWidget::slotCancel()
{
    if ( !d->semantic ) {
        d->kFileWidget->slotCancel();
    }
}

void Nepomuk::SmartFileWidget::switchDialog()
{
    setSemanticModeEnabled( !d->semantic );
}

void Nepomuk::SmartFileWidget::setSemanticModeEnabled( bool semantic )
{
    kDebug() << semantic;
    if (semantic) {
        if ( d->mode == Opening )
            d->stack->setCurrentWidget(d->openWidget);
        else
            d->stack->setCurrentWidget(d->saveWidget);
        d->switchButton->setText(i18n("File view"));
        d->switchButton->setIcon(KIcon("system-file-manager"));
    }
    else {
        d->stack->setCurrentWidget(d->stdWidget);
        d->switchButton->setText(i18n("Semantic view"));
        d->switchButton->setIcon(KIcon("nepomuk"));
    }

    d->semantic = semantic;

    // tell whoever is listening that we have just changed the dialog
    if (d->semantic) {
        emit dialogActivated(SemanticDialog, operationMode());
    }
    else {
        emit dialogActivated(StandardDialog, operationMode());
    }
}

Nepomuk::Types::Class Nepomuk::SmartFileWidget::typeClass() const
{
    return d->cls;
}

void Nepomuk::SmartFileWidget::setTypeClass(const Nepomuk::Types::Class& cls)
{
    d->cls = cls;
}

Nepomuk::Types::Class Nepomuk::SmartFileWidget::pimoTypeClass() const
{
    return d->pimocls;
}

void Nepomuk::SmartFileWidget::setPimoTypeClass(const Nepomuk::Types::Class& pimocls)
{
    d->pimocls = pimocls;
}

void Nepomuk::SmartFileWidget::addSuggestions(const QList<Nepomuk::Annotation*> suggestions)
{
    d->suggestions.append(suggestions);
}

QList<Nepomuk::Annotation*> Nepomuk::SmartFileWidget::suggestions() const
{
    return d->suggestions;
}

void Nepomuk::SmartFileWidget::setContent(const QString& content)
{
    d->request.setText(content);
}

QString Nepomuk::SmartFileWidget::content() const
{
    return d->request.text();
}

AnnotationRequest Nepomuk::SmartFileWidget::request() const
{
    return d->request;
}

#include "smartfilewidget.moc"
