/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
   (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/


#ifndef _SMART_FILE_WIDGET_H_
#define _SMART_FILE_WIDGET_H_

#include <kabstractfilewidget.h>
#include <QtGui/QWidget>

#include "annotation.h"
#include "annotationrequest.h"


namespace Nepomuk {
    class SmartFileWidgetPrivate;
    class SmartFileModule;
    namespace Types {
        class Class;
    }

    /**
     * @brief Nepomuk implementation of the abstract file widget.
     *
     * This is a new implementation of the abstract file widget interface
     * in KIO, offering semantic services to the open/save dialog.
     * It implements all the required methods for KAbstractFileWidget,
     * so it can be used as a drop-in replacement of the KFileWidget
     * from kfile library.
     * It contains also an instance of the KFileWidget
     * itself, so the user is able to switch between the two dialogs
     * dinamically; each set method sets values in both dialogs, and
     * each get method gets the correct value from the dialog
     * currently shown.
     *
     * This class contains also some public methods, specific to
     * the semantic implementation (thus for smartsave internal use
     * only).
     *
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com>
     * @author Sebastian Trueg <trueg@kde.org>
     *
     * @sa KAbstractFileWidget
     */
    class SmartFileWidget : public QWidget, public KAbstractFileWidget
    {
        Q_OBJECT
        Q_INTERFACES(KAbstractFileWidget)

    public:
        /**
         * Description of the two possible dialog types that this
         * plugin handles.
         */
        enum DialogType {
            StandardDialog = 1,
            SemanticDialog = 2
        };

        /**
         * Constructor.
         *
         * @param startDir the start directory
         * @param module the file module which creates this class
         * @param parent the parent class
         */
        SmartFileWidget( const KUrl& startDir, SmartFileModule* module, QWidget* parent = 0 );

        /**
         * Destructor.
         */
        ~SmartFileWidget();

        /**
         * @returns The selected fully qualified filename.
         */
        KUrl selectedUrl() const;

        /**
         * @returns The list of selected URLs.
         */
        KUrl::List selectedUrls() const;

        /**
         * @returns the currently shown directory.
         */
        KUrl baseUrl() const;

        /**
         * Returns the full path of the selected file in the local filesystem.
         * (Local files only)
         */
        QString selectedFile() const;

        /**
         * Returns a list of all selected local files.
         */
        QStringList selectedFiles() const;

        /**
         * Sets the directory to view.
         *
         * @param url URL to show.
         * @param clearforward Indicates whether the forward queue
         * should be cleared.
         */
        void setUrl(const KUrl &url, bool clearforward = true);

        /**
         * Sets the file name to preselect to @p name
         *
         * This takes absolute URLs and relative file names.
         */
        void setSelection(const QString& name);

        /**
         * Sets the operational mode of the filedialog to @p Saving, @p Opening
         * or @p Other. This will set some flags that are specific to loading
         * or saving files. E.g. setKeepLocation() makes mostly sense for
         * a save-as dialog. So setOperationMode( KFileDialog::Saving ); sets
         * setKeepLocation for example.
         *
         * The mode @p Saving, together with a default filter set via
         * setMimeFilter() will make the filter combobox read-only.
         *
         * The default mode is @p Opening.
         *
         * Call this method right after instantiating KFileDialog.
         *
         * @see operationMode
         * @see KFileDialog::OperationMode
         */
        void setOperationMode( OperationMode );

        /**
         * @returns the current operation mode, Opening, Saving or Other. Default
         * is Other.
         *
         * @see operationMode
         * @see KFileDialog::OperationMode
         */
        OperationMode operationMode() const;

        /**
         * Sets whether the filename/url should be kept when changing directories.
         * This is for example useful when having a predefined filename where
         * the full path for that file is searched.
         *
         * This is implicitly set when operationMode() is KFileDialog::Saving
         *
         * getSaveFileName() and getSaveUrl() set this to true by default, so that
         * you can type in the filename and change the directory without having
         * to type the name again.
         */
        void setKeepLocation( bool keep );

        /**
         * @returns whether the contents of the location edit are kept when
         * changing directories.
         */
        bool keepsLocation() const;

        /**
         * Sets the filter to be used to @p filter.
         *
         * You can set more
         * filters for the user to select separated by '\n'. Every
         * filter entry is defined through namefilter|text to diplay.
         * If no | is found in the expression, just the namefilter is
         * shown. Examples:
         *
         * \code
         * kfile->setFilter("*.cpp|C++ Source Files\n*.h|Header files");
         * kfile->setFilter("*.cpp");
         * kfile->setFilter("*.cpp|Sources (*.cpp)");
         * kfile->setFilter("*.cpp|" + i18n("Sources (*.cpp)"));
         * kfile->setFilter("*.cpp *.cc *.C|C++ Source Files\n*.h *.H|Header files");
         * \endcode
         *
         * Note: The text to display is not parsed in any way. So, if you
         * want to show the suffix to select by a specific filter, you must
         * repeat it.
         *
         * If the filter contains an unescaped '/', a mimetype-filter is assumed.
         * If you would like a '/' visible in your filter it can be escaped with
         * a '\'. You can specify multiple mimetypes like this (separated with
         * space):
         *
         * \code
         * kfile->setFilter( "image/png text/html text/plain" );
         * kfile->setFilter( "*.cue|CUE\\/BIN Files (*.cue)" );
         * \endcode
         *
         * @see filterChanged
         * @see setMimeFilter
         */
        void setFilter(const QString& filter);

        /**
         * Returns the current filter as entered by the user or one of the
         * predefined set via setFilter().
         *
         * @see setFilter()
         * @see filterChanged()
         */
        QString currentFilter() const;

        /**
         * Returns the mimetype for the desired output format.
         *
         * This is only valid if setFilterMimeType() has been called
         * previously.
         *
         * @see setFilterMimeType()
         */
        KMimeType::Ptr currentFilterMimeType();

        /**
         * Sets the filter up to specify the output type.
         *
         * @param types a list of mimetypes that can be used as output format
         * @param defaultType the default mimetype to use as output format, if any.
         * If @p defaultType is set, it will be set as the current item.
         * Otherwise, a first item showing all the mimetypes will be created.
         * Typically, @p defaultType should be empty for loading and set for saving.
         *
         * Do not use in conjunction with setFilter()
         */
        void setMimeFilter( const QStringList& types,
                                    const QString& defaultType = QString() );

        /**
         * The mimetype for the desired output format.
         *
         * This is only valid if setMimeFilter() has been called
         * previously.
         *
         * @see setMimeFilter()
         */
        QString currentMimeFilter() const;

        /**
         *  Clears any mime- or namefilter. Does not reload the directory.
         */
        void clearFilter();

        /**
         * Adds a preview widget and enters the preview mode.
         *
         * In this mode the dialog is split and the right part contains your
         * preview widget.
         *
         * Ownership is transferred to KFileDialog. You need to create the
         * preview-widget with "new", i.e. on the heap.
         *
         * @param w The widget to be used for the preview.
         */
        void setPreviewWidget(KPreviewWidgetBase *w);

        /**
         * Sets the mode of the dialog.
         *
         * The mode is defined as (in kfile.h):
         * \code
         *    enum Mode {
         *         File         = 1,
         *         Directory    = 2,
         *         Files        = 4,
         *         ExistingOnly = 8,
         *         LocalOnly    = 16
         *    };
         * \endcode
         * You can OR the values, e.g.
         * \code
         * KFile::Modes mode = KFile::Files |
         *                     KFile::ExistingOnly |
         *                     KFile::LocalOnly );
         * setMode( mode );
         * \endcode
         */
        void setMode( KFile::Modes m );

        /**
         * Returns the mode of the filedialog.
         * @see setMode()
         */
        KFile::Modes mode() const;

        /**
         * Sets the text to be displayed in front of the selection.
         *
         * The default is "Location".
         * Most useful if you want to make clear what
         * the location is used for.
         */
        void setLocationLabel(const QString& text);

        /**
         * Returns a pointer to the toolbar.
         *
         * You can use this to insert custom
         * items into it, e.g.:
         * \code
         *      yourAction = new KAction( i18n("Your Action"), 0,
         *                                this, SLOT( yourSlot() ),
         *                                this, "action name" );
         *      yourAction->plug( kfileDialog->toolBar() );
         * \endcode
         */
        KToolBar *toolBar() const;

        /**
         * @returns a pointer to the OK-Button in the filedialog.
         * Note that the button is hidden and unconnected when using KFileWidget alone;
         * KFileDialog shows it and connects to it.
         */
        KPushButton *okButton() const;

        /**
         * @returns a pointer to the Cancel-Button in the filedialog.
         * Note that the button is hidden and unconnected when using KFileWidget alone;
         * KFileDialog shows it and connects to it.
         */
        KPushButton *cancelButton() const;

        /**
         * @returns the combobox used to type the filename or full location of the file.
         */
        KUrlComboBox *locationEdit() const;

        /**
         * @returns the combobox that contains the filters
         */
        KFileFilterCombo *filterWidget() const;

        /**
         * @returns a pointer to the action collection, holding all the used
         * KActions.
         */
        KActionCollection *actionCollection() const;

        /**
         * Set a custom widget that should be added to the bottom of the file dialog.
         * @param widget A widget, or a widget of widgets, for displaying custom
         *               data in the file widget. This can be used, for example, to
         *               display a check box with the caption "Open as read-only".
         *               When creating this widget, you don't need to specify a parent,
         *               since the widget's parent will be set automatically by KFileWidget.
         */
        void setCustomWidget(QWidget* widget);

        /**
         * Sets a custom widget that should be added below the location and the filter
         * editors.
         * @param text     Label of the custom widget, which is displayed below the labels
         *                 "Location:" and "Filter:".
         * @param widget   Any kind of widget, but preferable a combo box or a line editor
         *                 to be compliant with the location and filter layout.
         *                 When creating this widget, you don't need to specify a parent,
         *                 since the widget's parent will be set automatically by KFileWidget.
         */
        void setCustomWidget(const QString& text, QWidget* widget);

        /**
         * This method returns the current NFO class.
         *
         * @return an instance of a Nepomuk::Types::Class
         */
        Nepomuk::Types::Class typeClass() const;

        /**
         * This sets the current NFO class to be associated to
         * the file to be saved.
         *
         * @param cls an instance of Nepomuk::Types::Class
         */
        void setTypeClass(const Nepomuk::Types::Class& cls);

        /**
         * This method returns the current PIMO class.
         *
         * @return an instance of a Nepomuk::Types::Class
         */
        Nepomuk::Types::Class pimoTypeClass() const;

        /**
         * This sets the current PIMO class to be associated to
         * the file to be saved.
         *
         * @param pimocls an instance of Nepomuk::Types::Class
         */
        void setPimoTypeClass(const Nepomuk::Types::Class& pimocls);

        /**
         * Add some personalized suggestions to the save dialog.
         *
         * @param suggestions a list of suggestions to be shown to the user
         */
        void addSuggestions(const QList<Nepomuk::Annotation*> suggestions);

        /**
         * Get the current personalized suggestions.
         *
         * @return a list of suggestions added before (if any)
         */
        QList<Nepomuk::Annotation*> suggestions() const;

        /**
         * Set some text content to be analyzed through the annotation
         * plugins currently registered.
         *
         * @param content the content to be analyzed
         */
        void setContent(const QString& content);

        /**
         * Return the content that has been analyzed, if any.
         *
         * @return the text content as a string
         */
        QString content() const;

        /**
         * Return the request that is used if the text content is used.
         *
         * @return an instance of the annotation request
         */
        AnnotationRequest request() const;

    public Q_SLOTS:
        /**
         * Called when clicking ok (when this widget is used in KFileDialog)
         * Might or might not call accept().
         */
        void slotOk();

        /**
         * Called when the user accepts the current choices.
         */
        void accept();

        /**
         * Called when clicking cancel.
         */
        void slotCancel();

    Q_SIGNALS:
        /**
         * Emitted when the user selects a file. It is only emitted in single-
         * selection mode. The best way to get notified about selected file(s)
         * is to connect to the okClicked() signal inherited from KDialog
         * and call selectedFile(), selectedFiles(),
         * selectedUrl() or selectedUrls().
         *
         * \since 4.4
         */
        void fileSelected(const KUrl&);

        /**
         * @deprecated, connect to fileSelected(const KUrl&) instead
         */
        QT_MOC_COMPAT void fileSelected(const QString&); // TODO KDE5: remove

        /**
         * Emitted when the user highlights a file.
         * \since 4.4
         */
        void fileHighlighted(const KUrl&);
        /**
         * @deprecated, connect to fileSelected(const KUrl&) instead
         */
        QT_MOC_COMPAT void fileHighlighted(const QString&); // TODO KDE5: remove

        /**
         * Emitted when the user hilights one or more files in multiselection mode.
         *
         * Note: fileHighlighted() or fileSelected() are @em not
         * emitted in multiselection mode. You may use selectedItems() to
         * ask for the current highlighted items.
         * @see fileSelected
         */
        void selectionChanged();

        /**
         * Emitted when the filter changed, i.e. the user entered an own filter
         * or chose one of the predefined set via setFilter().
         *
         * @param filter contains the new filter (only the extension part,
         * not the explanation), i.e. "*.cpp" or "*.cpp *.cc".
         *
         * @see setFilter()
         * @see currentFilter()
         */
        void filterChanged( const QString& filter );

        /**
         * Emitted by slotOk() (directly or asynchronously) once everything has
         * been done. Should be used by the caller to call accept().
         */
        void accepted();

        /**
         * Emitted when the semantic dialog is activated the first
         * time, and when the dialog type switching button is clicked.
         *
         * @param type the type of the dialog currently shown
         * @param mode the mode of the dialog
         */
        void dialogActivated(Nepomuk::SmartFileWidget::DialogType type, KAbstractFileWidget::OperationMode mode);

        /// @internal for future extensions
        void virtual_hook( int id, void* data );

    private:
        SmartFileWidgetPrivate* d;
        SmartFileModule* m_module;

        void setSemanticModeEnabled( bool );

    private Q_SLOTS:
        void switchDialog();
    };
}

#endif
