/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
             (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "smartopenwidget.h"
#include "smartfilewidget.h"
#include "documentmanager.h"
#include "recentannotationplugin.h"
#include "facetedfiltermodel.h"
#include "facetedfilterproxysortmodel.h"
#include "daterangeselectionwidget.h"
#include "daterange.h"

#include "nie.h"
#include "annotationplugin.h"
#include "annotationpluginfactory.h"
#include "annotationpluginwrapper.h"
#include "annotationrequest.h"
#include "tagannotation.h"

#include <kmessagebox.h>
#include <kcomponentdata.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <kpushbutton.h>
#include <kdatepicker.h>
#include <kdirmodel.h>
#include <kfilefiltercombo.h>
#include <kdirlister.h>
#include <kdiroperator.h>
#include <kicon.h>
#include <klocale.h>
#include <kdebug.h>
#include <kconfiggroup.h>
#include <kglobal.h>

#include <QVariant>
#include <QDialog>
#include <QItemSelectionModel>
#include <QSortFilterProxyModel>
#include <QModelIndexList>
#include <QListView>
#include <QTreeView>
#include <QObject>
#include <QSize>
#include <QMenu>
#include <QUrl>
#include <QThreadPool>

#include <nepomuk/resourcemanager.h>
#include <nepomuk/tag.h>
#include <nepomuk/class.h>
#include <nepomuk/property.h>

#include <Soprano/QueryResultIterator>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/XMLSchema>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Model>
#include <Soprano/LiteralValue>


using namespace Nepomuk;

class Nepomuk::SmartOpenWidget::Private
{
public:
    Private(QWidget* parent) :
        filterModel(new FacetedFilterModel(parent))
    {
    }

    KDirOperator* fileOp;
    FacetedFilterModel* filterModel;

    SmartFileWidget* m_fileWidget;
};

SmartOpenWidget::SmartOpenWidget(SmartFileWidget* dlg, QWidget* parent) :
    QWidget(parent),
    d(new Private(dlg))
{
    d->m_fileWidget = dlg;

    setupUi( this );

    d->fileOp=new KDirOperator(KUrl(), this);
    d->fileOp->setView(KFile::Detail);
    KConfigGroup grp(KGlobal::config(), "Smartsave view");
    d->fileOp->readConfig(grp);
    verticalLayout->addWidget(d->fileOp);

    // set models
    FacetedFilterProxySortModel* sortModel = new FacetedFilterProxySortModel( m_filterView );
    sortModel->setDynamicSortFilter( true );
    sortModel->setSourceModel( d->filterModel );
    sortModel->sort( 0 );
    m_filterView->setModel(sortModel);
    m_filterView->setSelectionMode(QAbstractItemView::SingleSelection);

    // handlers
    connect(d->filterModel, SIGNAL(filterSelectionChanged()), this, SLOT(selectionChanged()));
    connect(m_refreshButton, SIGNAL(clicked()), d->fileOp, SLOT(rereadDir()));
    connect(m_smartDateEdit, SIGNAL(rangeChanged(DateRange)), this, SLOT(updateForcedFilters()));

    m_refreshButton->setIcon(KIcon("view-refresh"));

    connect(m_searchTextLine, SIGNAL(editingFinished()), this, SLOT(selectionChanged()));

    connect(dlg, SIGNAL(dialogActivated(Nepomuk::SmartFileWidget::DialogType, KAbstractFileWidget::OperationMode)),
            this, SLOT(makeConnections(Nepomuk::SmartFileWidget::DialogType, KAbstractFileWidget::OperationMode)));
}

SmartOpenWidget::~SmartOpenWidget()
{
    delete d;
}


void SmartOpenWidget::selectionChanged()
{
    kDebug();

    QList<Filter> filters = d->filterModel->selectedFilters();
    QString filterString;
    foreach( const Filter& filter, filters ) {
        filterString += filter.toSparqlPattern( QLatin1String( "?r" ) );
    }

    QString query = QString( "select distinct ?r where { ?r a %1 . %2 OPTIONAL { ?r2 a nfo:Folder . FILTER(?r=?r2) . } . FILTER(!BOUND(?r2)) } LIMIT 100")
                    .arg( Soprano::Node::resourceToN3(Nepomuk::DocumentManager::DEFAULTCLS.uri() ) )
                    .arg( filterString );

    KUrl url("nepomuksearch:/");
    url.addQueryItem("sparql", query);
    d->fileOp->setUrl(url, true);

    kDebug() << "File view url:" << url;
}


void SmartOpenWidget::updateForcedFilters()
{
    kDebug();

    QList<Filter> filters;

    // take a look at dates
    if (m_smartDateEdit->range().start().isValid()) {
        filters << Filter( Nepomuk::Vocabulary::NIE::lastModified(),
                           Soprano::LiteralValue( QDateTime( m_smartDateEdit->range().start(), QTime( 0, 0, 0 ) ) ),
                           Filter::MoreThan );
    }
    if (m_smartDateEdit->range().end().isValid()) {
        filters <<  Filter( Nepomuk::Vocabulary::NIE::lastModified(),
                            Soprano::LiteralValue( QDateTime( m_smartDateEdit->range().end(), QTime( 23, 59, 59, 999 ) ) ),
                            Filter::LessThan );
    }

    // mime filters
    QStringList mimeFilters = d->m_fileWidget->filterWidget()->filters();
    foreach(const QString& q, mimeFilters) {
        // using directly the property url: some obscure problems in linking to NIE...
        filters << Filter( Nepomuk::Vocabulary::NIE::mimeType(), Soprano::LiteralValue(q) );
    }

    d->filterModel->setForcedFilters(filters);
}

void SmartOpenWidget::dialogClosed()
{
    KConfigGroup grp(KGlobal::config(), "Smartsave view");
    d->fileOp->writeConfig(grp);
}

QString SmartOpenWidget::getFilename() const
{
    KFileItemList list = d->fileOp->selectedItems();
    if (list.size() > 0) {
        return list[0].localPath();
    }
    else {
        return QString();
    }
}

KUrl SmartOpenWidget::getFilenameUrl() const
{
    KFileItemList list = d->fileOp->selectedItems();
    if (list.size() > 0) {
        return list[0].url();
    }
    else {
        return KUrl();
    }
}

void Nepomuk::SmartOpenWidget::makeConnections(Nepomuk::SmartFileWidget::DialogType type, KAbstractFileWidget::OperationMode mode)
{
    if (type == Nepomuk::SmartFileWidget::SemanticDialog && mode == KAbstractFileWidget::Opening) {
        updateForcedFilters();
    }
}

#include "smartopenwidget.moc"
