/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
   (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "smartsavewidget.h"
#include "smartfilewidget.h"

#include <kcomponentdata.h>
#include <kmimetype.h>
#include <kfiledialog.h>
#include <kfilefiltercombo.h>
#include <kmessagebox.h>

#include <QVariant>
#include <QDialog>
#include <QItemSelectionModel>
#include <QModelIndexList>
#include <nepomuk/resourcemanager.h>
#include <nepomuk/tag.h>
#include <QListView>
#include <QTreeView>
#include <QSize>
#include <QMenu>
#include <QUrl>
#include <QThreadPool>
#include <QtGui/QFormLayout>
#include <kicon.h>
#include <klocale.h>
#include <kdebug.h>

#include "annotationmodel.h"
#include "annotationplugin.h"
#include "annotationpluginfactory.h"
#include "annotationpluginwrapper.h"
#include "annotationrequest.h"
#include "tagannotation.h"
#include "pimomodel.h"
#include "pimo.h"

#include "kpixmapsequenceoverlaypainter.h"
#include "pimoitemmodel.h"
#include "recentannotationplugin.h"
#include "document.h"
#include "documentmanager.h"
#include "smartannotationwidget.h"
#include "smarttextedit.h"

using namespace Nepomuk;

// necessary for some reasons... it is already in AnnotationModel
Q_DECLARE_METATYPE(Nepomuk::Annotation*)

class Nepomuk::SmartSaveWidget::Private
{
public:
    Private(QWidget* parent) :
        suggestionModel(new AnnotationModel(parent)),
        recentsModel(new AnnotationModel(parent)),
        documentModel(new AnnotationModel(parent)),
        document(0), wrapper(new AnnotationPluginWrapper(parent)),
        connected(false)
    {
    }
    AnnotationModel* suggestionModel;
    AnnotationModel* recentsModel;
    AnnotationModel* documentModel;
    QTreeView* PIMOView;
    PIMOItemModel* PIMOModel;
    Document* document;
    AnnotationPluginWrapper* wrapper;
    SmartTextEdit* commentText;
    SmartFileWidget* m_fileWidget;
    bool connected;
};

SmartSaveWidget::SmartSaveWidget(SmartFileWidget* dlg, QWidget* parent) :
    QWidget(parent),
    d(new Private(dlg))
{
    d->m_fileWidget = dlg;

    setupUi( this );

    // make this connection asap: it is extremely important
    connect(dlg, SIGNAL(dialogActivated(Nepomuk::SmartFileWidget::DialogType, KAbstractFileWidget::OperationMode)),
            this, SLOT(makeConnections(Nepomuk::SmartFileWidget::DialogType, KAbstractFileWidget::OperationMode)));

    d->commentText = new SmartTextEdit(this);
    d->commentText->setObjectName(QString::fromUtf8("commentText"));
    formLayout->setWidget(1, QFormLayout::FieldRole, d->commentText);

    m_suggestionList->setModel(d->suggestionModel);
    m_suggestionList->setSelectionMode(QAbstractItemView::SingleSelection);
    m_recentList->setModel(d->recentsModel);
    m_recentList->setSelectionMode(QAbstractItemView::SingleSelection);
    m_savingList->setModel(d->documentModel);
    m_savingList->setSelectionMode(QAbstractItemView::SingleSelection);

    // set handlers
    connect(m_annWidget, SIGNAL(newAnnotation(Nepomuk::Annotation*)), d->documentModel, SLOT(addNewAnnotation(Nepomuk::Annotation*)));
    connect(d->commentText, SIGNAL(editingFinished()), this, SLOT(analyzeDescription()));

    m_typeClassEdit->setText(d->m_fileWidget->pimoTypeClass().name());
    m_left2rightSuggestionButton->setIcon(KIcon("go-next-view"));
    m_left2rightRecentButton->setIcon(KIcon("go-next-view"));
    m_removeButton->setIcon(KIcon("list-remove"));
    m_clearButton->setIcon(KIcon("edit-clear-list"));
    m_selectTypeButton->setIcon(KIcon("view-choose"));

    connect(m_removeButton, SIGNAL(clicked(bool)), this, SLOT(deleteTag()));
    connect(m_clearButton, SIGNAL(clicked(bool)), this, SLOT(clearTags()));
    connect(m_left2rightSuggestionButton, SIGNAL(clicked(bool)), this, SLOT(addSuggestion()));
    connect(m_left2rightRecentButton, SIGNAL(clicked(bool)), this, SLOT(addRecent()));
    connect(m_selectTypeButton, SIGNAL(clicked(bool)), this, SLOT(showModelTree()));
    connect(m_filenameLine, SIGNAL(editingFinished()), this, SLOT(analyzeLabel()));
}

SmartSaveWidget::~SmartSaveWidget()
{
    delete d;
}

void SmartSaveWidget::connectSuggestions()
{
    // suggestions
    AnnotationPluginFactory* factory = AnnotationPluginFactory::instance();
    QList<Nepomuk::AnnotationPlugin*> list = factory->getPluginsSupportingAnnotationOfType(d->m_fileWidget->typeClass());
    d->wrapper->setPlugins(list);
    connect(d->wrapper, SIGNAL(newAnnotation(Nepomuk::Annotation*)), d->suggestionModel,
            SLOT(addNewAnnotation(Nepomuk::Annotation*)));
    // recents
    AnnotationPlugin* recents = new RecentAnnotationPlugin(this);
    connect(recents, SIGNAL(newAnnotation(Nepomuk::Annotation*)), d->recentsModel,
            SLOT(addNewAnnotation(Nepomuk::Annotation*)));
    // requests
    // start the overlay painting and then launch all requests
    launchOverlay(m_suggestionList, d->wrapper);
    launchOverlay(m_recentList, recents);
    d->wrapper->getPossibleAnnotations(d->m_fileWidget->request());
    recents->getPossibleAnnotations(AnnotationRequest());

    d->connected = true;
}

QString SmartSaveWidget::getFilename() const
{
    // create document instance and associate everything
    Nepomuk::Types::Class cls=d->m_fileWidget->typeClass();
    DocumentManager* manager = DocumentManager::createDocumentManager(cls);
    d->document = manager->document();
    for (int i = 0; i < d->documentModel->rowCount(); ++i) {
        QModelIndex index = d->documentModel->index(i, 0);
        QVariant data = d->documentModel->data(index, AnnotationModel::AnnotationRole);
        Nepomuk::Annotation* ann = data.value<Nepomuk::Annotation*>();
        d->document->addAnnotation(ann);
    }
    d->document->setLabel(m_filenameLine->text());
    d->document->setComment(d->commentText->toPlainText());
    d->document->setPIMOType(d->m_fileWidget->pimoTypeClass());
    KMimeType::Ptr type = KMimeType::mimeType(d->m_fileWidget->currentMimeFilter());
    if (type) {
        d->document->setFileExtension(type->mainExtension());
    }

    // return the real file string to the caller
    if (!manager->createPath()) {
        KMessageBox::error(d->m_fileWidget, "Unable to save file metadata.");
        // manual deletion: the manager will delete itself only after
        // its execution, but in this case it will not be executed
        manager->deleteLater();
        return QString();
    }
    else {
        manager->save();
        return manager->filename();
    }
}

KUrl SmartSaveWidget::getFilenameUrl() const
{
    QString filename = getFilename();

    return KUrl(filename);
}

void SmartSaveWidget::launchOverlay(QWidget* widget, AnnotationPlugin* plugin)
{
    KPixmapSequenceOverlayPainter* p=widget->findChild<KPixmapSequenceOverlayPainter*>();
    if (!p) {
        p=new KPixmapSequenceOverlayPainter(widget);
        p->setWidget(widget);
        connect(plugin, SIGNAL(finished()), p, SLOT(stop()));
    }
    p->start();
}

void SmartSaveWidget::launchOverlay(QWidget* widget, AnnotationPluginWrapper* wrapper)
{
    KPixmapSequenceOverlayPainter* p=widget->findChild<KPixmapSequenceOverlayPainter*>();
    if (!p) {
        p=new KPixmapSequenceOverlayPainter(widget);
        p->setWidget(widget);
        connect(wrapper, SIGNAL(finished()), p, SLOT(stop()));
    }
    p->start();
}

void SmartSaveWidget::clearTags()
{
    d->documentModel->clear(false);
}

void SmartSaveWidget::deleteTag()
{
    QItemSelectionModel* selmodel = m_savingList->selectionModel();
    if (selmodel->hasSelection()) {
        QModelIndexList list = selmodel->selectedRows(0);
        // just one selection permitted
        QModelIndex index = list.at(0);
        d->documentModel->takeAnnotation(index);
    }
}

void SmartSaveWidget::addSuggestion()
{
    QItemSelectionModel* selmodel = m_suggestionList->selectionModel();
    if (selmodel->hasSelection()) {
        QModelIndexList list = selmodel->selectedRows(0);
        if (list.size() > 0) {
            // just one selection permitted
            QModelIndex index = list.at(0);
            QVariant data = d->suggestionModel->data(index, AnnotationModel::AnnotationRole);
            Nepomuk::Annotation* ann = data.value<Nepomuk::Annotation*> ();
            d->documentModel->addNewAnnotation(ann);
        }
    }
}

void SmartSaveWidget::addRecent()
{
    QItemSelectionModel* selmodel = m_recentList->selectionModel();
    if (selmodel->hasSelection()) {
        QModelIndexList list = selmodel->selectedRows(0);
        // just one selection permitted
        QModelIndex index = list.at(0);
        QVariant data = d->recentsModel->data(index, AnnotationModel::AnnotationRole);
        Nepomuk::Annotation* ann = data.value<Nepomuk::Annotation*> ();
        d->documentModel->addNewAnnotation(ann);
    }
}

void SmartSaveWidget::showModelTree()
{
    KDialog dlg(this);
    dlg.setCaption("Select document type");
    dlg.setButtons(KDialog::Ok | KDialog::Cancel);
    d->PIMOView = new QTreeView(&dlg);
    d->PIMOView->header()->hide();
    d->PIMOView->setSelectionMode(QAbstractItemView::SingleSelection);
    d->PIMOModel = new PIMOItemModel(d->PIMOView);
    d->PIMOModel->setParentClass(Nepomuk::Types::Class(Nepomuk::Vocabulary::PIMO::Document()));
    d->PIMOView->setModel(d->PIMOModel);
    d->PIMOView->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(d->PIMOView, SIGNAL(customContextMenuRequested(const QPoint&)), this,
            SLOT(addNewClassMenu(const QPoint&)));
    dlg.setMainWidget(d->PIMOView);

    // set the selection and open its tree
    QItemSelectionModel* selmodel=d->PIMOView->selectionModel();
    QModelIndex idx=d->PIMOModel->indexForClass(d->m_fileWidget->pimoTypeClass());
    QItemSelection sel(idx, idx);
    selmodel->select(sel, QItemSelectionModel::Select);
    QModelIndexList list=d->PIMOModel->parentIndexList(d->m_fileWidget->pimoTypeClass());
    if (!list.isEmpty()) {
        foreach(idx, list) {
            d->PIMOView->expand(idx);
        }
    }

    if (dlg.exec() == QDialog::Accepted) {
        QItemSelectionModel* selmodel = d->PIMOView->selectionModel();
        QModelIndexList list = selmodel->selectedIndexes();
        if (list.size() > 0) {
            QVariant data = d->PIMOModel->data(list[0], PIMOItemModel::TypeRole);
            d->m_fileWidget->setPimoTypeClass(data.value<Nepomuk::Types::Class>());
            m_typeClassEdit->setText(d->m_fileWidget->pimoTypeClass().name());
        }
    }
}

void SmartSaveWidget::addNewClassMenu(const QPoint& pos)
{
    QModelIndex index = d->PIMOView->indexAt(pos);
    if (index.isValid()) {
        Nepomuk::Types::Class type = index.data(Nepomuk::PIMOItemModel::TypeRole).value<
        Nepomuk::Types::Class> ();
        QAction actionNewSubClass(KIcon("document-new"), i18n("Create Subclass"), this);
        QAction* a = QMenu::exec(QList<QAction*> () << &actionNewSubClass,
                                 d->PIMOView->viewport()->mapToGlobal(pos));

        if (a == &actionNewSubClass) {
            KDialog* dlg = new KDialog(d->PIMOView);
            dlg->setCaption("Set class name");
            dlg->setButtons(KDialog::Ok | KDialog::Cancel);
            KLineEdit* name = new KLineEdit(dlg);
            dlg->setMainWidget(name);
            if (dlg->exec() == QDialog::Accepted && name->text().length() > 0) {
                Nepomuk::PimoModel pimoModel(Nepomuk::ResourceManager::instance()->mainModel());
                Nepomuk::Types::Class newClass = pimoModel.createClass(type.uri(), name->text());
                if (newClass.isValid())
                    d->PIMOModel->updateClass(type);
            }
        }
    }
}

void SmartSaveWidget::analyzeLabel()
{
    // wrapper is already in place by now
    Nepomuk::AnnotationRequest request;
    if (m_filenameLine->text().length() > 0) {
        request.setText(m_filenameLine->text());
        launchOverlay(m_suggestionList, d->wrapper);
        d->wrapper->getPossibleAnnotations(request);
    }
}

void SmartSaveWidget::analyzeDescription()
{
    // wrapper is already in place by now
    Nepomuk::AnnotationRequest request;
    if (d->commentText->toPlainText().length() > 0) {
        request.setText(d->commentText->toPlainText());
        launchOverlay(m_suggestionList, d->wrapper);
        d->wrapper->getPossibleAnnotations(request);
    }
}

KFileFilterCombo *Nepomuk::SmartSaveWidget::filterWidget() const
{
    return m_fileFilter;
}

/*
 * We check here that the semantic dialog is shown and that we are going to be
 * used in the saving mode; if this is the truth, then we connect some signals
 * and make some requests.
 */
void Nepomuk::SmartSaveWidget::makeConnections(Nepomuk::SmartFileWidget::DialogType type, KAbstractFileWidget::OperationMode mode)
{
    if (type == Nepomuk::SmartFileWidget::SemanticDialog && mode == KAbstractFileWidget::Saving) {
        connectSuggestions();
        kDebug() << "Saving operations connected";
    }
}

#include "smartsavewidget.moc"
