/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#include <QDebug>
#include <QDateTime>
#include <kdebug.h>
#include <nepomuk/resourcemanager.h>
#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Node>
#include <Soprano/Util/AsyncModel>
#include <tagannotation.h>
#include "recentannotationplugin.h"

using namespace Nepomuk;

class Nepomuk::RecentAnnotationPluginPrivate
{
    public:
        RecentAnnotationPluginPrivate() :
            limit(10)
        {
        }
        int limit;
};

RecentAnnotationPlugin::RecentAnnotationPlugin(QObject* parent) :
    AnnotationPlugin(parent), d(new RecentAnnotationPluginPrivate())
{
}

RecentAnnotationPlugin::~RecentAnnotationPlugin()
{
    delete d;
}

void RecentAnnotationPlugin::doGetPossibleAnnotations(const AnnotationRequest& request)
{
    Q_UNUSED(request);

    QString query;
    QString addition = QString(
                            "select distinct ?label where { graph ?g { ?r %1 ?tag . } . ?tag %3 ?label . ?g %4 ?date . } ORDER BY DESC(?date) LIMIT %5")
                            .arg(Soprano::Node::resourceToN3(Soprano::Vocabulary::NAO::hasTag()))
                            .arg(Soprano::Node::resourceToN3(Soprano::Vocabulary::NAO::prefLabel()))
                            .arg(Soprano::Node::resourceToN3(Soprano::Vocabulary::NAO::created()))
                            .arg(this->limit());
    query.append(addition);
    kDebug() << query;
    Soprano::Util::AsyncQuery* aQuery =
            Soprano::Util::AsyncQuery::executeQuery(
                    Nepomuk::ResourceManager::instance()->mainModel(),
                    query,
                    Soprano::Query::QueryLanguageSparql);
    connect(aQuery, SIGNAL(nextReady(Soprano::Util::AsyncQuery*)), this, SLOT(queryNextReady(Soprano::Util::AsyncQuery*)));
    connect(aQuery, SIGNAL(finished(Soprano::Util::AsyncQuery*)), this, SLOT(queryFinished(Soprano::Util::AsyncQuery*)));
}

void RecentAnnotationPlugin::setLimit(int limit)
{
    d->limit = limit;
}

int RecentAnnotationPlugin::limit()
{
    return d->limit;
}

/*
 * This method will interpret each new binding, then it will call next()
 * again.
 */
void RecentAnnotationPlugin::queryNextReady(Soprano::Util::AsyncQuery* query)
{
    Soprano::Node node = query->binding("label");
    if (node.isValid()) {
        TagAnnotation* ann = new TagAnnotation();
        ann->setTagLabel(node.literal().toString());
        addNewAnnotation(ann);
    }
    node = query->binding("id");
    if (node.isValid()) {
        TagAnnotation* ann = new TagAnnotation();
        ann->setTagLabel(node.literal().toString());
        addNewAnnotation(ann);
    }
    query->next();
}

/*
 * Ok, the query is gone so we can finish ourselves, too.
 */
void RecentAnnotationPlugin::queryFinished(Soprano::Util::AsyncQuery* query)
{
    // the model has to know that nothing will come from here from now on
    emitFinished();
    // check the possible error
    Soprano::Error::Error e = query->lastError();
    if (e.code() != Soprano::Error::ErrorNone) {
        qDebug() << e.code() << ":" << e.message();
    }
}
