/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) version 3, or any
   later version accepted by the membership of KDE e.V. (or its
   successor approved by the membership of KDE e.V.), which shall
   act as a proxy defined in Section 6 of version 3 of the license.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourcenavigator.h"
#include "resourcenavigator_p.h"
#include "resourcenavigatormenu_p.h"
#include "pimo.h"

#include <QtGui/QFrame>
#include <QtGui/QLabel>
#include <QtGui/QHBoxLayout>

#include <Nepomuk/ResourceManager>
#include <Nepomuk/Query/Query>
#include <Nepomuk/Query/ResourceTypeTerm>
#include <Nepomuk/Query/ComparisonTerm>
#include <Nepomuk/Query/ResourceTerm>
#include <Nepomuk/Query/AndTerm>

#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/Node>
#include <Soprano/Vocabulary/RDFS>

#include <KDebug>

// DESIGN: one button for each resource and one button for each arrow in between and one button at the end filling up the row to select the next level


void Nepomuk::ResourceNavigatorPrivate::reset()
{
    delete m_topLevelButton->m_nextButton;
    m_topLevelButton->m_nextButton = 0;
}


Nepomuk::ResourceButton* Nepomuk::ResourceNavigatorPrivate::createResourceButton( const Nepomuk::Resource& res )
{
    ResourceButton* newButton = new ResourceButton( m_mainFrame );
    newButton->m_resource = res;
    newButton->setText( res.genericLabel() );
    m_mainFrame->layout()->addWidget( newButton );
    QObject::connect( newButton, SIGNAL( clicked() ), q, SLOT( _k_resourceButtonClicked() ) );
    return newButton;
}


Nepomuk::NavigatorButton* Nepomuk::ResourceNavigatorPrivate::createNavigatorButton( const Nepomuk::Resource& res )
{
    NavigatorButton* navButton = new NavigatorButton( m_mainFrame );
    ResourceNavigatorMenu* navMenu = new ResourceNavigatorMenu( this, navButton );
    navMenu->setResource( res );
    navButton->setMenu( navMenu );
    QObject::connect( navMenu, SIGNAL( resourceActivated( Nepomuk::NavigatorButton*, Nepomuk::Resource ) ),
                      q, SLOT( _k_resourceActivated( Nepomuk::NavigatorButton*, Nepomuk::Resource ) ) );
    m_mainFrame->layout()->addWidget( navButton );
    return navButton;
}


Nepomuk::NavigatorButton* Nepomuk::ResourceNavigatorPrivate::createButtons( const Nepomuk::Resource& res, NavigatorButton* lastButton )
{
    // check for parents
    Query::Query query( Query::ComparisonTerm( m_parentRelationProperty.inverseProperty(), Query::ResourceTerm( res ) ) );
    query.setLimit( 1 );
    Soprano::QueryResultIterator it =
        ResourceManager::instance()->mainModel()->executeQuery( query.toSparqlQuery( Query::Query::HandleInverseProperties ),
                                                                Soprano::Query::QueryLanguageSparql );
    if ( it.next() ) {
        // create buttons for the parent resources
        lastButton = createButtons( it[0].uri(), lastButton );
    }

    // now create the buttons for res
    ResourceButton* newButton = createResourceButton( res );
    lastButton->m_nextButton = newButton;
    newButton->m_nextButton = createNavigatorButton( res );
    return newButton->m_nextButton;
}


void Nepomuk::ResourceNavigatorPrivate::_k_resourceActivated( Nepomuk::NavigatorButton* button, const Nepomuk::Resource& res )
{
    kDebug();

    // reset selection
    delete button->m_nextButton;

    // create new button
    ResourceButton* newButton = createResourceButton( res );
    button->m_nextButton = newButton;

    // create next navigator button only if there are sub-resources
    if ( m_allowResourceCreation ||
         ResourceManager::instance()->mainModel()
         ->executeQuery( Query::Query( Query::AndTerm( Query::ResourceTypeTerm( m_subLevelType ),
                                                       Query::ComparisonTerm( m_parentRelationProperty, Query::ResourceTerm( res ) ) ) )
                         .toSparqlQuery( Query::Query::CreateCountQuery ),
                         Soprano::Query::QueryLanguageSparql )
         .allBindings().first().value( 0 ).literal().toInt() > 0 ) {
        newButton->m_nextButton = createNavigatorButton( res );
    }

    // inform clients
    m_currentResource = res;
    emit q->resourceChanged( res );
}


void Nepomuk::ResourceNavigatorPrivate::_k_resourceButtonClicked()
{
    ResourceButton* button = static_cast<ResourceButton*>( q->sender() );
    if ( button->m_nextButton ) {
        delete button->m_nextButton->m_nextButton;
    }
    m_currentResource = button->m_resource;
    emit q->resourceChanged( m_currentResource );
}


Nepomuk::ResourceNavigator::ResourceNavigator( QWidget* parent )
    : QWidget( parent ),
      d( new ResourceNavigatorPrivate() )
{
    d->q = this;

    // connect to context service
    // FIXME: handle project and task differently
    d->m_contextServiceInterface = new ContextServiceInterface( this );
    connect( d->m_contextServiceInterface, SIGNAL( projectChanged( Nepomuk::Resource ) ),
             this, SLOT( setResource( Nepomuk::Resource ) ) );
    connect( d->m_contextServiceInterface, SIGNAL( taskChanged( Nepomuk::Resource ) ),
             this, SLOT( setResource( Nepomuk::Resource ) ) );

    // default values
    d->m_topLevelType = Vocabulary::PIMO::Project();
    d->m_subLevelType = Vocabulary::PIMO::Task();
    d->m_parentRelationProperty = Vocabulary::PIMO::partOf();
    d->m_allowResourceCreation = true;

    // create main layout
    setBackgroundRole(QPalette::Base);
    d->m_mainFrame = new QFrame( this );
    d->m_mainFrame->setFrameStyle( QFrame::Sunken|QFrame::StyledPanel );

    QHBoxLayout* layout = new QHBoxLayout( this );
    layout->setMargin( 0 );
    layout->addWidget( d->m_mainFrame );

    QHBoxLayout* frameLayout = new QHBoxLayout( d->m_mainFrame );
    frameLayout->setMargin( 0 );

    // create top level button
    d->m_topLevelButton = new NavigatorButton( d->m_mainFrame );
    d->m_mainFrame->layout()->addWidget( d->m_topLevelButton );
    d->m_topLevelButton->setMenu( new ResourceNavigatorMenu( d, d->m_topLevelButton ) );
    connect( d->m_topLevelButton->menu(), SIGNAL( resourceActivated( Nepomuk::NavigatorButton*, Nepomuk::Resource ) ),
             this, SLOT( _k_resourceActivated( Nepomuk::NavigatorButton*, Nepomuk::Resource ) ) );
}


Nepomuk::ResourceNavigator::~ResourceNavigator()
{
    delete d;
}


void Nepomuk::ResourceNavigator::setTopLevelType( const Types::Class& type )
{
    d->m_topLevelType = type;
    d->reset();
}


void Nepomuk::ResourceNavigator::setParentRelationProperty( const Types::Property& prop )
{
    d->m_parentRelationProperty = prop;
    d->reset();
}


void Nepomuk::ResourceNavigator::setAllowResourceCreation( bool enabled )
{
    d->m_allowResourceCreation = enabled;
}


void Nepomuk::ResourceNavigator::setResource( const Nepomuk::Resource& res )
{
    if ( res != d->m_currentResource ) {
        d->m_currentResource = res;
        d->reset();
        d->createButtons( res, d->m_topLevelButton );
    }
}


Nepomuk::Resource Nepomuk::ResourceNavigator::currentResource() const
{
    return d->m_currentResource;
}

#include "resourcenavigator.moc"
