/*
  This file is part of the Nepomuk KDE project.
  Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) version 3, or any
  later version accepted by the membership of KDE e.V. (or its
  successor approved by the membership of KDE e.V.), which shall
  act as a proxy defined in Section 6 of version 3 of the license.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "taskfiltermodel.h"
#include "taskmodel.h"
#include "tmo.h"
#include "daterange.h"

#include <QtCore/QDateTime>

#include <Soprano/Util/SignalCacheModel>

#include <Nepomuk/ResourceManager>

#include <KDebug>


class Nepomuk::TaskFilterModel::Private
{
public:
    bool m_showCompletedTasks;
    DateRange m_filterDateRange;

    QDateTime minMaxDate( QModelIndex index, bool max );

    TaskFilterModel* q;
};


QDateTime Nepomuk::TaskFilterModel::Private::minMaxDate( QModelIndex index, bool max )
{
    if ( !m_showCompletedTasks ) {
        QUrl state = index.data( TaskModel::TaskStateRole ).toUrl();
        if( state != Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_New() &&
            state != Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Running() ) {
            return QDateTime();
        }
    }

    QDateTime dueDate = index.data( TaskModel::TaskDueDateRole ).toDateTime();
    int rows = q->sourceModel()->rowCount( index );
    for ( int row = 0; row < rows; ++row ) {
        QDateTime dt = minMaxDate( q->sourceModel()->index( row, 0, index ), max );
        if ( dt.isValid() &&
             ( !dueDate.isValid() ||
               ( max && dt > dueDate ) ||
               ( !max && dt < dueDate ) ) ) {
            dueDate = dt;
        }
    }
    return dueDate;
}


Nepomuk::TaskFilterModel::TaskFilterModel( QObject* parent )
    : QSortFilterProxyModel( parent ),
      d( new Private() )
{
    d->q = this;

    setShowCompletedTasks( false );
    setDynamicSortFilter( true );

    Soprano::Util::SignalCacheModel* scm = new Soprano::Util::SignalCacheModel( ResourceManager::instance()->mainModel() );
    scm->setParent( this ); // mem management
    scm->setCacheTime( 1000 );
    connect( scm, SIGNAL( statementsAdded() ), this, SLOT( invalidate() ) );
    connect( scm, SIGNAL( statementsRemoved() ), this, SLOT( invalidate() ) );
}


Nepomuk::TaskFilterModel::~TaskFilterModel()
{
}


void Nepomuk::TaskFilterModel::setShowCompletedTasks( bool b )
{
    d->m_showCompletedTasks = b;
    invalidateFilter();
}


void Nepomuk::TaskFilterModel::setFilterDateRange( const DateRange& range )
{
    d->m_filterDateRange = range;
    invalidateFilter();
}


bool Nepomuk::TaskFilterModel::filterAcceptsRow( int source_row, const QModelIndex& source_parent ) const
{
    QModelIndex index = sourceModel()->index( source_row, 0, source_parent );

    if ( !d->m_showCompletedTasks ) {
        QUrl state = index.data( TaskModel::TaskStateRole ).toUrl();
        if( state != Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_New() &&
            state != Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Running() ) {
            return false;
        }
    }

    if ( d->m_filterDateRange.start().isValid() ) {
        QDateTime dueDate = d->minMaxDate( index, false );
        if ( dueDate.date() < d->m_filterDateRange.start() ) {
            return false;
        }
    }

    if ( d->m_filterDateRange.end().isValid() ) {
        QDateTime dueDate = d->minMaxDate( index, true );
        if ( dueDate.date() > d->m_filterDateRange.end() ) {
            return false;
        }
    }

    return true;
}


bool Nepomuk::TaskFilterModel::lessThan( const QModelIndex& left, const QModelIndex& right ) const
{
    QDateTime dueDateLeft = left.data( TaskModel::TaskDueDateRole ).toDateTime();
    QDateTime dueDateRight = right.data( TaskModel::TaskDueDateRole ).toDateTime();
    int prioLeft = left.data( TaskModel::TaskPriorityNumRole ).toInt();
    int prioRight = right.data( TaskModel::TaskPriorityNumRole ).toInt();

    return( dueDateLeft < dueDateRight ||
            ( dueDateLeft == dueDateRight &&
              prioLeft < prioRight ) );
}

#include "taskfiltermodel.moc"
