/*
  This file is part of the Nepomuk KDE project.
  Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) version 3, or any
  later version accepted by the membership of KDE e.V. (or its
  successor approved by the membership of KDE e.V.), which shall
  act as a proxy defined in Section 6 of version 3 of the license.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "taskmodel.h"
#include "task.h"
#include "tmo.h"

#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/Node>
#include <Soprano/Util/SignalCacheModel>

#include <Nepomuk/ResourceManager>
#include <Nepomuk/Variant>

#include <KGlobal>
#include <KLocale>
#include <KDebug>
#include <KIcon>

#include <QtCore/QList>
#include <QtCore/QDateTime>
#include <QtCore/QMimeData>
#include <QtCore/QDataStream>
#include <QtCore/QByteArray>


class Nepomuk::TaskModel::TaskNode
{
public:
    TaskNode( const Nepomuk::Task& t, Nepomuk::TaskModel* model )
        : m_task( t ),
          m_parent( 0 ),
          m_model( model ) {
    }

    virtual ~TaskNode() {
        qDeleteAll( m_children );
    }

    virtual QModelIndex index();
    TaskNode* findTaskNode( const Nepomuk::Task& task, bool recursive = false );
    void updateChildren( bool recursive = false );
    void setCompleted( bool completed );

    Nepomuk::Task m_task;
    QList<TaskNode*> m_children;
    TaskNode* m_parent;

    Nepomuk::TaskModel* m_model;
};

class Nepomuk::TaskModel::RootTaskNode : public Nepomuk::TaskModel::TaskNode
{
public:
    RootTaskNode( Nepomuk::TaskModel* model )
        : TaskNode( Nepomuk::Task(), model )
    {
    }

    QModelIndex index() { return QModelIndex(); }
};

QModelIndex Nepomuk::TaskModel::TaskNode::index()
{
    return m_model->index( m_parent->m_children.indexOf( this ), 0, m_parent->index() );
}

Nepomuk::TaskModel::TaskNode* Nepomuk::TaskModel::TaskNode::findTaskNode( const Nepomuk::Task& task, bool recursive )
{
    foreach( TaskNode* node, m_children ) {
        if ( node->m_task == task ) {
            return node;
        }
        else if ( recursive ) {
            if ( TaskNode* rn = node->findTaskNode( task, recursive ) ) {
                return rn;
            }
        }
    }

    return 0;
}

void Nepomuk::TaskModel::TaskNode::updateChildren( bool recursive )
{
    // add new tasks
    QList<Nepomuk::Task> tasks;
    Soprano::QueryResultIterator it;
    if ( m_task.isValid() ) {
        // default children query
        it = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( QString( "select ?r where { "
                                                                                       "?r a %1 . "
                                                                                       "{ ?r %2 %4 . } UNION { %4 %3 ?r . } . }" )
                                                                              .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::TMO::Task() ) )
                                                                              .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::TMO::superTask() ) )
                                                                              .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::TMO::subTask() ) )
                                                                              .arg( Soprano::Node::resourceToN3( m_task.resourceUri() ) ),
                                                                              Soprano::Query::QueryLanguageSparql );
    }
    else {
        // this is where we query the parent tasks (only used in RootTaskNode)
        it = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( QString( "select ?r where { "
                                                                                       "?r a %1 . "
                                                                                       "OPTIONAL { "
                                                                                       "{ ?r %2 ?st . } UNION { ?st %3 ?r . } } . "
                                                                                       "FILTER(!BOUND(?st)) . }" )
                                                                              .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::TMO::Task() ) )
                                                                              .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::TMO::superTask() ) )
                                                                              .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::TMO::subTask() ) ),
                                                                              Soprano::Query::QueryLanguageSparql );
    }

    // append all tasks not in the list yet
    while ( it.next() ) {
        Nepomuk::Task task( it[0].uri() );
        tasks.append( task );
        if ( !findTaskNode( task ) ) {
            m_model->beginInsertRows( index(), m_children.count(), m_children.count() );
            TaskNode* node = new TaskNode( task, m_model );
            node->m_parent = this;
            m_children.append( node );
            m_model->endInsertRows();
        }
    }

    // remove old instances
    int i = 0;
    while ( i < m_children.count() ) {
        TaskNode* node = m_children[i];
        if ( !tasks.contains( node->m_task ) ) {
            m_model->beginRemoveRows( index(), i, i );
            m_children.removeAt( i );
            m_model->endRemoveRows();
            delete node;
        }
        else {
            if ( recursive )
                node->updateChildren( recursive );
            ++i;
        }
    }
}


void Nepomuk::TaskModel::TaskNode::setCompleted( bool completed )
{
    if ( completed ) {
        m_task.setTaskState( Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Completed() );
        // once a task is completed, so are its subtasks
        foreach( TaskNode* t, m_children ) {
            t->setCompleted( true );
        }
    }
    else {
        m_task.setTaskState( Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Running() );
    }
    emit m_model->dataChanged( index(), index() );
}


class Nepomuk::TaskModel::Private
{
public:
    Private( TaskModel* model )
        : m_root( model ) {
    }

    void _k_dataChanged();

    RootTaskNode m_root;
};


void Nepomuk::TaskModel::Private::_k_dataChanged()
{
    // TODO: it would be so nice to do this less brute-force!
    m_root.updateChildren( true );
}


Nepomuk::TaskModel::TaskModel( QObject* parent )
    : ResourceModel( parent ),
      d( new Private(this) )
{
    d->m_root.updateChildren( true );
    Soprano::Util::SignalCacheModel* scm = new Soprano::Util::SignalCacheModel( ResourceManager::instance()->mainModel() );
    scm->setParent( this ); // mem management
    scm->setCacheTime( 1000 );
    connect( scm, SIGNAL( statementsAdded() ), this, SLOT( _k_dataChanged() ) );
    connect( scm, SIGNAL( statementsRemoved() ), this, SLOT( _k_dataChanged() ) );
}


Nepomuk::TaskModel::~TaskModel()
{
}


QUrl Nepomuk::TaskModel::resourceForIndex( const QModelIndex& index ) const
{
    TaskNode* node = static_cast<TaskNode*>( index.internalPointer() );
    if ( node ) {
        return node->m_task.resourceUri();
    }
    else {
        return QUrl();
    }
}


QModelIndex Nepomuk::TaskModel::indexForResource( const QUrl& uri ) const
{
    if ( TaskNode* node = d->m_root.findTaskNode( uri, true ) ) {
        return node->index();
    }
    else {
        return QModelIndex();
    }
}


QModelIndex Nepomuk::TaskModel::parent( const QModelIndex& index ) const
{
    TaskNode* node = static_cast<TaskNode*>( index.internalPointer() );
    if ( node ) {
        return node->m_parent->index();
    }
    else {
        return QModelIndex();
    }
}


QModelIndex Nepomuk::TaskModel::index( int row, int column, const QModelIndex& parent ) const
{
    if ( parent.isValid() ) {
        TaskNode* node = static_cast<TaskNode*>( parent.internalPointer() );
        if ( row < node->m_children.count() )
            return createIndex( row, column, node->m_children[row] );
        else
            return QModelIndex();
    }
    else if ( row < d->m_root.m_children.count() ) {
        return createIndex( row, column, d->m_root.m_children[row] );
    }
    else {
        return QModelIndex();
    }
}


Qt::ItemFlags Nepomuk::TaskModel::flags( const QModelIndex& index ) const
{
    Qt::ItemFlags f = ResourceModel::flags( index );
    if ( index.isValid() ) {
        f |= Qt::ItemIsUserCheckable|Qt::ItemIsDragEnabled;
        if ( index.column() == 0 ||
             index.column() == 1 ) {
            f |= Qt::ItemIsEditable;
        }
    }
    return f|Qt::ItemIsDropEnabled;
}


QVariant Nepomuk::TaskModel::data( const QModelIndex& index, int role ) const
{
    Task task( resourceForIndex( index ) );
    if ( task.isValid() ) {
        if ( index.column() == 0 ) {
            switch( role ) {
            case Qt::DecorationRole: {
                QString icon = task.genericIcon();
                if ( icon.isEmpty() )
                    icon = QLatin1String( "view-calendar-tasks" );
                return KIcon( icon );
            }

            case TaskStateRole: {
                Resource state = task.taskState();
                if ( !state.isValid() ) {
                    state = Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_New();
                }
                return QVariant::fromValue( state.resourceUri() );
            }

            case TaskPriorityRole: {
                Resource prio = task.priority();
                if ( !prio.isValid() )
                    prio = Nepomuk::Vocabulary::TMO::TMO_Instance_Priority_Medium();

                return prio.resourceUri();
            }

            case TaskPriorityNumRole: {
                Resource prio = task.priority();
                if ( prio == Nepomuk::Vocabulary::TMO::TMO_Instance_Priority_Low() )
                    return -1;
                else if ( prio == Nepomuk::Vocabulary::TMO::TMO_Instance_Priority_High() )
                    return 1;
                else
                    return 0;
            }


            case TaskDueDateRole:
                return task.dueDate();

            case Qt::ToolTipRole:
                return task.description();

            case Qt::CheckStateRole:
                return( !task.taskState().isValid() ||
                        task.taskState() == Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_New() ||
                        task.taskState() == Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Running()
                        ? Qt::Unchecked
                        : Qt::Checked );
            }

            return ResourceModel::data( index, role );
        }

        else if ( index.column() == 1 ) {
            if ( role == Qt::DisplayRole ) {
                return KGlobal::locale()->formatDate( task.dueDate().date(), KLocale::FancyShortDate );
            }
            else if ( role == Qt::EditRole ) {
                // we store the due date as datetime but let the user edit only the date.
                if ( task.dueDate().isValid() )
                    return task.dueDate().date();
                else
                    return QDate::currentDate(); // make it simpler for the user
            }
        }

        else if ( index.column() == 2 ) {
            if ( role == Qt::DisplayRole )
                return ResourceModel::data( index, ResourceModel::ResourceCreationDate );
        }

        else if ( index.column() == 3 ) {
            if ( role == Qt::DisplayRole ) {
                int p = index.data( TaskPriorityNumRole ).toInt();
                return( p == -1 ? i18n( "Low" ) : ( p == 1 ? i18n( "High" ) : i18n( "Medium" ) ) );
            }
        }
    }

    return QVariant();
}


int Nepomuk::TaskModel::rowCount( const QModelIndex& parent ) const
{
    if ( parent.isValid() ) {
        TaskNode* node = static_cast<TaskNode*>( parent.internalPointer() );
        return node->m_children.count();
    }
    else {
        return d->m_root.m_children.count();
    }
}


int Nepomuk::TaskModel::columnCount( const QModelIndex& ) const
{
    return 4;
}


QVariant Nepomuk::TaskModel::headerData( int section, Qt::Orientation orientation, int role ) const
{
    if ( role == Qt::DisplayRole ) {
        switch( section ) {
        case 0:
            return i18n( "Task" );
        case 1:
            return i18n( "Due" );
        case 2:
            return i18n( "Creation Date" );
        case 3:
            return i18n( "Priority" );
        }
    }

    return ResourceModel::headerData( section, orientation, role );
}


bool Nepomuk::TaskModel::setData( const QModelIndex& index, const QVariant& value, int role )
{
    if ( !index.isValid() )
        return false;

    if ( role == Qt::CheckStateRole ) {
        TaskNode* node = static_cast<TaskNode*>( index.internalPointer() );
        node->setCompleted( value.toBool() );
        return true;
    }
    else if ( role == Qt::EditRole ) {
        Task task( resourceForIndex( index ) );

        switch( index.column() ) {
        case 0:
            task.setLabel( value.toString() );
            break;

        case 1:
            // we only allow to edit the date, not the time as that is probably too much for most users.
            task.setDueDate( QDateTime( value.toDate(), QTime( 23, 59, 59 ) ) );
            emit dataChanged( index, index );
            break;

        default:
            return false;
        }

        return true;
    }

    return false;
}


QStringList Nepomuk::TaskModel::mimeTypes() const
{
    return QStringList() << QLatin1String( "application/x-nepomuk-task-model-items" );
}


QMimeData* Nepomuk::TaskModel::mimeData( const QModelIndexList& indexes ) const
{
    QList<QUrl> uris;
    foreach( const QModelIndex& index, indexes ) {
        uris << index.data( ResourceModel::ResourceUriRole ).toUrl();
    }
    QMimeData* data = new QMimeData();
    QByteArray encodedData;
    QDataStream stream( &encodedData, QIODevice::WriteOnly );
    stream << uris;
    data->setData( mimeTypes().first(), encodedData );
    return data;
}


bool Nepomuk::TaskModel::dropMimeData( const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent )
{
    if ( action == Qt::IgnoreAction )
        return true;

    if ( !data->hasFormat( mimeTypes().first() ) )
        return false;

    QList<QUrl> uris;
    QByteArray encodedData = data->data( mimeTypes().first() );
    QDataStream stream( &encodedData, QIODevice::ReadOnly );
    stream >> uris;

    QModelIndex parentTaskItem;
    if ( parent.isValid() && row < 0 )
        parentTaskItem = parent;
    else if( parent.isValid() )
        parentTaskItem = index( row, column, parent );

    Task newSuperTask;
    if ( parentTaskItem.isValid() ) {
        newSuperTask = parentTaskItem.data( ResourceModel::ResourceUriRole ).toUrl();
    }

    foreach( const QUrl& uri, uris ) {
        Task task( uri );
        if ( task.exists() ) {
            Task oldSuperTask = task.superTask();
            if ( !oldSuperTask.isValid() ) {
                QList<Resource> stl = ResourceManager::instance()->allResourcesWithProperty( Vocabulary::TMO::subTask(), task );
                if ( !stl.isEmpty() )
                    oldSuperTask = stl.first();
            }

            if ( oldSuperTask.isValid() &&
                 oldSuperTask.resourceUri() != newSuperTask.resourceUri() ) {
                // remove task from the subtask list of oldSuperTask
                QList<Task> oldSuperTaskSubTasks = oldSuperTask.subTasks();
                oldSuperTaskSubTasks.removeAll( task );
                oldSuperTask.setSubTasks( oldSuperTaskSubTasks );

                d->m_root.findTaskNode( oldSuperTask, true )->updateChildren();
            }

            if ( newSuperTask.isValid() ) {
                // add task to the subtask list of newSuperTask
                newSuperTask.addSubTask( task );

                task.setSuperTask( newSuperTask );
            }
        }
    }

    if ( newSuperTask.isValid() )
        d->m_root.findTaskNode( newSuperTask, true )->updateChildren();

    return true;
}

#include "taskmodel.moc"
